/****************************************************************************
*
*	  Header Filename:  Interaction_Type.h
*	  Module Name:      Interaction_Type
*	  Description:      An Interaction_Type is a possible
*                       type of chemical interaction.
*
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			#include  "Kernel_Codes.h"Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#ifndef Interaction_Type_H
#define Interaction_Type_H

#include  "Kernel_Codes.h"

class   Network_Painter_Prot;
class   Molecule_Type;
class   VCSList;
class   Realisable_Interaction_Type;
class   Realised_Interaction_Type;

//      Parameter struct for Calculate_Reactor_Interaction_Rates

struct  Calculate_Reactor_Interaction_Rates_Args{int Reactant1_Num_Mols;
                                                 int Reactant2_Num_Mols;
                                                 int Product1_Num_Mols;
                                                 int Product2_Num_Mols;
                                                 int Reactor_Volume;};

//----------------------------------------------------------------------------------------------------------

class Interaction_Type

//  Base class. Objects of this base class define one Reactant and one Product.
//  Derived classes are used for the following categories of Interaction_Type.

//        Category Of
//        Interaction_Type              Interaction Scheme
//        ----------------              ------------------

//        Construction                  A + B       ->  C
//        Fission                       A           ->  B + C
//        Recombination                 A + B       ->  C + D
//        Transformation                A           ->  B
//        Catalytic_Construction        A + B + X   ->  C + X
//        Catalytic_Fission             A + X       ->  B + C + X
//        Catalytic_Recombination       A + B + X   ->  C + D + X
//        Catalytic_Transformation      A + X       ->  B + X

{
    protected:

//===== Data Relationships with other classes ============================================================
        Molecule_Type*                  Molecule_Type__Reactant1_Ptr;
        Molecule_Type*                  Molecule_Type__Product1_Ptr;
        Realisable_Interaction_Type*    Realisable_Interaction_Type__Ptr;
        Realised_Interaction_Type*      Realised_Interaction_Type_Ptr;
//==========================================================================================================

    public:
//      Public Functions

        void Add_To_Chemistry(int Reactant1_Index, int Product1_Index);
        static void  Clear_Static();
        virtual void Remove_From_Chemistry() = 0;
        virtual double  Calculate_Reactor_Interaction_Rate
                            (const Calculate_Reactor_Interaction_Rates_Args* const Args) const = 0;
//        virtual Interaction_Type*  Copy_With_Variation(int Old_MolType_Index, int New_MolType_Index) = 0;


        int     Get_Identifier() const;
        double  Get_Activated_Complex_Energy()const;
        virtual double Get_Heat_Release() const = 0;
        virtual double Get_Forward_Activation_Energy() const = 0;
        virtual double Get_Reverse_Activation_Energy() const = 0;
        static double Get_Max_Activated_Complex_Energy() {return Max_Activated_Complex_Energy;};
        Realised_Interaction_Type*    Get_Realised_Int_Type_Ptr();
        void  Link_To_Realised_Int_Type(Realised_Interaction_Type* Realised_Int_Type_Ptr);  
        virtual Interaction_Type_Enum   Get_Category() const = 0;     // Get Interaction_Type Category
        virtual string Get_Category_As_String()  const = 0;

        virtual void Get_Int_Type_Characteristics(  int& Interaction_Type_Identifier,
                                            Interaction_Type_Enum&  Interaction_Type_Enumr,
                                            int& Reactant1_Identifier,
                                            int& Reactant2_Identifier,
                                            int& Product1_Identifier,
                                            int& Product2_Identifier,
                                            int& Catalyst_Identifier) const = 0;

        virtual void Get_Int_Type_Inputs_No_Dups(VCSList* TList__Of_Interaction_Inputs_Ptr)  const = 0;
        virtual void Get_Int_Type_Products_With_Dups(VCSList* TList__Of_Int_Type_Reactants_Ptr) const =0;
        virtual void Get_Int_Type_Reactants_With_Dups(VCSList* TList__Of_Int_Type_Reactants_Ptr) const =0;

        bool    Is_Realisable();
        
        inline  Molecule_Type* Get_Reactant1_MolType_Ptr() const {return(Molecule_Type__Reactant1_Ptr);}
        virtual Molecule_Type* Get_Reactant2_MolType_Ptr() const {return(0);}
        inline  Molecule_Type* Get_Product1_MolType_Ptr() const  {return(Molecule_Type__Product1_Ptr);}
        virtual Molecule_Type* Get_Product2_MolType_Ptr() const  {return(0);}
        virtual Molecule_Type* Get_Catalyst_MolType_Ptr() const  {return(0);}
        
        double  Get_Forward_Rate_Constant() const {return Forward_Rate_Constant;}
        double  Get_Reverse_Rate_Constant() const {return Reverse_Rate_Constant;}
        inline  Realisable_Interaction_Type*  
	    Get_R_Int_Type_Ptr() const {return  Realisable_Interaction_Type__Ptr;};

        void    Link_To_R_Int_Type(Realisable_Interaction_Type*  R_Int_Type_Ptr);
        void    Unlink_From_R_Int_Type(Realisable_Interaction_Type*  R_Int_Type_Ptr);

        Interaction_Type();
        virtual ~Interaction_Type() {};


    protected:

//      Protected Class Data

        int Identifier;  // Unique identifier for the Interaction_Type
//        bool Removed;    // Indicates whether this Interaction_Type has been (logically) deleted  
        double  Activated_Complex_Energy;

        static double Max_Activated_Complex_Energy;    

        static int  Next_Interaction_Type_Identifier;   // Identifier for next
                                                        //  Interaction_Type to be created
        double Forward_Rate_Constant;
        double Reverse_Rate_Constant;
        
                    //  In SimSoup, the Reactor Interaction Rate for an Interaction_Type is defined
                    //  as follows:-
                    //       Reactor Interaction Rate:  Number of actual Interactions of an
                    //       Interaction_Type per SimSoup timestep.
                    //
                    //       The Reaction Rate as generally used by chemists is the rate of
                    //       change of reactant concentration per unit time. In SimSoup, the
                    //       relation between Reaction Rate and Reactor Interaction Rate is:-
                    //
                    //           Reaction Rate =  Reactor Interaction Rate
                    //                            ------------------------
                    //                                 Reactor Volume
                    //
                    //  SimSoup supports first order and second order Interaction Types. The
                    //  first order Interaction Types are Transformation and Fission. The second
                    //  order Interaction Type is the Construction
                    //
                    //  FIRST ORDER REACTIONS
                    //
                    //  Chemists define the rate equation for first order reactions as
                    //
                    //                     Reaction Rate = K[R]
                    //
                    //  Here K is the Rate Constant, and [R] is the concentration of the reactant.
                    //
                    //  In SimSoup terms, this can be re-arranged by multiplying both sides by
                    //  the reactor volume to give:-
                    //
                    //  Reactor Interaction Rate = K x Number of Molecules of Reactant in Reactor
                    //
                    //  For a first order reaction the Rate Constant (K) has units of
                    //  Interactions per second per Reactant Molecule.
                    //
                    //  SECOND ORDER REACTIONS
                    //
                    //  Chemists define the rate equation for a second order reaction as:-
                    //
                    //                     Reaction Rate = K[R1] [R2]
                    //
                    //  Here, K is the Rate Constant, R1 is the concentration of the first
                    //  Reactant, and R2 is the concentration of the second reactant.
                    //
                    //  In SimSoup terms, this can be re-arranged by multiplying both sides by
                    //  the reactor volume to give:-
                    //
                    //  Reactor Interaction Rate = K x Num Mols of Reactant 1 x Num Mols of Reactant 2
                    //                             ---------------------------------------------------
                    //                                            Reactor Volume
                    //
                    //  Thus, for a second order reaction the Rate Constant (K) has units of
                    //  (Interactions x Volume) per second per Reactant 1 Molecule per reactane 2 Molecule
                                    
//      Private Functions

//----------------------------------------------------------------------------------------------------------
public:
    class Support
    {
        public:
//            static std::string  Convert_Int_Type_Categ_To_Text(Interaction_Type_Enum Interaction_Type_Enumr);
            static std::string  Get_Reactant1_Identifier_Text(const Interaction_Type*  const Interaction_Type__Ptr);
            static std::string  Get_Reactant2_Identifier_Text(const Interaction_Type*  const Interaction_Type__Ptr);
            static std::string  Get_Product1_Identifier_Text(const Interaction_Type*  const Interaction_Type__Ptr);
            static std::string  Get_Product2_Identifier_Text(const Interaction_Type*  const Interaction_Type__Ptr);
            static std::string  Get_Forward_Rate_Constant_Text(const Interaction_Type*  const Interaction_Type__Ptr);
            static std::string  Get_Reverse_Rate_Constant_Text(const Interaction_Type*  const Interaction_Type__Ptr);
            static string Get_IntType_Summary_As_String(Interaction_Type*  IntType_Ptr);
        private:
            static const std::string   AnsiString__Int_Type_Categ_Names[8];
    };     
};
//------------------------------------------------------------------------------

class   Construction  : public Interaction_Type

//  Interaction_Type for interaction with schema:-
//   A + B -> C

{
    private:

//      Private Data

//===== Relationships with other classes ===================================================================

        Molecule_Type*  Molecule_Type__Reactant2_Ptr;
//==========================================================================================================

    public:
//      Public Functions

//        Construction();
        Construction(int R1_Id, int R2_Id, int P_Id,
                     double          Activation_Energy);

//        Interaction_Type*  Copy_With_Variation(int Old_MolType_Index, int New_MolType_Index);
//        void Draw(Network_Painter_Prot* const Network_Painter_Prot__Ptr,
//                  double Interaction_Rate,
 //                 int Conn_Selection_State) const;
        virtual Interaction_Type_Enum   Get_Category() const;
        virtual string Get_Category_As_String()  const;
        double Get_Heat_Release() const;
        double Get_Forward_Activation_Energy() const;
        double Get_Reverse_Activation_Energy() const;
        void Get_Int_Type_Characteristics(  int& Interaction_Type_Identifier,
                                            Interaction_Type_Enum&  Interaction_Type_Enumr,
                                            int& Reactant1_Identifier,
                                            int& Reactant2_Identifier,
                                            int& Product1_Identifier,
                                            int& Product2_Identifier,
                                            int& Catalyst_Identifier) const;
        void Get_Int_Type_Inputs_No_Dups(VCSList* TList__Of_Interaction_Inputs_Ptr) const;
        void Get_Int_Type_Products_With_Dups(VCSList* TList__Of_Int_Type_Products_Ptr) const;
        void Get_Int_Type_Reactants_With_Dups(VCSList* TList__Of_Int_Type_Reactants_Ptr) const;

        Molecule_Type* Get_Reactant2_MolType_Ptr() const
        {   if( Molecule_Type__Reactant2_Ptr == 0)
            THEN
            Assert (Molecule_Type__Reactant2_Ptr != 0, "Interaction_type.h: Construction  - Invalid Molecule Type 2 Pointer");
            ENDIF
            return(Molecule_Type__Reactant2_Ptr);
        }
        static double Get_Max_Rand_Rate_Constant() {return Max_Rand_Rate_Constant;}
        static void Set_Max_Rand_Rate_Constant(double Max_Rate); 
//        static double Get_Max_Activation_Energy() {return Max_Activation_Energy;};

        static bool Specify_Unique_Construction(bool R1_Random, int& Reactant1_Index,
                                                bool P_Random, int& Product1_Index,
                                                bool R2_Random, int& Reactant2_Index,
						                        bool ACE_Rnd_Fl, double& Activation_Energy,
                                                string*& Fail_Tx_Pt);


        static bool Validate_Construction_Parameters(int Reactant1_Index,
                                                     int Product1_Index,
                                                     int Reactant2_Index,
                                                     double Actv_Cplx_Energy,
                                                     bool Rep_Fail_Reas_Fl,
                                                     string*& Fail_Tx_Pt);


private:

//      Private Class Data

        static double Max_Rand_Rate_Constant;
//	    static double Max_Activation_Energy;

//      Private Functions


        void Add_To_Chemistry(int Reactant1_Index,
                              int Product1_Index,
                              int Reactant2_Index);
        double Calculate_Reactor_Interaction_Rate
                  (const Calculate_Reactor_Interaction_Rates_Args* const Args) const;
	    void Determine_Rate_Constants();
        void Remove_From_Chemistry();                      
                              
        static bool Check_If_Construction_Exists(int Reactant1_Index,
                                                 int Product1_Index,
                                                 int Reactant2_Index);
};
//------------------------------------------------------------------------------


class   Fission : public Interaction_Type

//  Interaction_Type for interaction with schema:-
//   A -> B + C

{
    private:

//      Private Data

//===== Relationships with other classes ===================================================================

        Molecule_Type*  Molecule_Type__Product2_Ptr;
//==========================================================================================================

    public:

//      Public Function


//        Interaction_Type*  Copy_With_Variation(int Old_MolType_Index, int New_MolType_Index);
//        void Draw(Network_Painter_Prot* const Network_Painter_Prot__Ptr,
//                  double Interaction_Rate,
//                  int Conn_Selection_State) const;
 
//        Fission();

        Fission(int R1_Id, int R2_Id, int P_Id, double  Activtn_Enrgy);

//        static bool Find_Candidate_Product_And_Energy(int     Reactant_Index, 
//                                                      int     Product1_Index,
//                                                      double& Fission_Actv_Cplx_Energy,
//                                                      int&    Cat_Trans_Product_Index);

        Interaction_Type_Enum   Get_Category() const;
        string Get_Category_As_String()  const;
        double Get_Heat_Release() const;
        double Get_Forward_Activation_Energy() const;
        double Get_Reverse_Activation_Energy() const;
        void Get_Int_Type_Inputs_No_Dups(VCSList* TList__Of_Interaction_Inputs_Ptr) const;
        void Get_Int_Type_Products_With_Dups(VCSList* TList__Of_Int_Type_Products_Ptr) const;
        void Get_Int_Type_Reactants_With_Dups(VCSList* TList__Of_Int_Type_Reactants_Ptr) const;
        Molecule_Type* Get_Product2_MolType_Ptr() const
        {
            Assert (Molecule_Type__Product2_Ptr != 0, "Interaction_type.h: Fission  - Invalid Molecule Type Product2 Pointer");
            return(Molecule_Type__Product2_Ptr);
        }
        static double Get_Max_Rand_Rate_Constant() {return Max_Rand_Rate_Constant;}
        static void Set_Max_Rand_Rate_Constant(double Max_Rate);
//	    static double Get_Max_Activation_Energy() {return  Max_Activation_Energy;}
//        static void Set_Max_Activation_Energy(double Max_Actvn_Energy) 
//           {Max_Activation_Energy = Max_Actvn_Energy;}

        static bool Validate_Fission_Parameters(int Reactant_Index,
                                                int Product1_Index,
                                                int Product2_Index,
                                                double Actv_Cplx_Energy,
                                                bool Rep_Fail_Reas_Fl,   // Whether to report fail reason
                                                string*& Fail_Tx_Pt);

        static bool Specify_Unique_Fission(bool R_Rnd, int& R_Id,   // Reactant
                                           bool P1_Rnd, int& P1_Id, // Product 1
                                           bool P2_Rnd, int& P2_Id, // Product 2
                                           bool ACE_Rnd_Fl, double& Actvtn_Enrgy,
                                           string*& Fail_Tx_Pt);
private:

//      Private Class Data

        static double Max_Rand_Rate_Constant;  // Max Rate Constant with the Random Rate Const. model
//	    static double Max_Activation_Energy;

//      Private Functions

        void Add_To_Chemistry(int Reactant1_Index,
                              int Product1_Index,
                              int Product2_Index);
        double Calculate_Reactor_Interaction_Rate
                    (const Calculate_Reactor_Interaction_Rates_Args* const Args) const;

        static bool Check_If_Fission_Exists(int Reactant1_Index,
                                            int Product1_Index,
                                            int Product2_Index);

        void Determine_Rate_Constants();

        void Get_Int_Type_Characteristics(  int& Interaction_Type_Identifier,
                                            Interaction_Type_Enum&  Interaction_Type_Enumr,
                                            int& Reactant1_Identifier,
                                            int& Reactant2_Identifier,
                                            int& Product1_Identifier,
                                            int& Product2_Identifier,
                                            int& Catalyst_Identifier) const;

        void Remove_From_Chemistry();                                                                                               
};
//------------------------------------------------------------------------------

class   Transformation : public Interaction_Type

//  Interaction_Type for interaction with schema:-
//   A -> B

{
    private:

//      Private Data

//===== Relationships with other classes ===================================================================
//==========================================================================================================

    public:

//      Public Function


        Interaction_Type*  Copy_With_Variation(int Old_MolType_Index, int New_MolType_Index);
//        void Draw(Network_Painter_Prot* const Network_Painter_Prot__Ptr,
//                  double Interaction_Rate,
//                  int Conn_Selection_State) const;
        Interaction_Type_Enum   Get_Category() const;
        string Get_Category_As_String()  const;
        double Get_Heat_Release() const;
        double Get_Activation_Energy() const;
        double Get_Forward_Activation_Energy() const  {return 0.0;};   //  *****  NEEDS COMPLETION
        double Get_Reverse_Activation_Energy() const  {return 0.0;};   //  *****  NEEDS COMPLETION
        void Get_Int_Type_Characteristics(  int& Interaction_Type_Identifier,
                                            Interaction_Type_Enum&  Interaction_Type_Enumr,
                                            int& Reactant1_Identifier,
                                            int& Reactant2_Identifier,
                                            int& Product1_Identifier,
                                            int& Product2_Identifier,
                                            int& Catalyst_Identifier) const;

        void Get_Int_Type_Inputs_No_Dups(VCSList* TList__Of_Interaction_Inputs_Ptr) const;
        void Get_Int_Type_Products_With_Dups(VCSList* TList__Of_Int_Type_Products_Ptr) const;
        void Get_Int_Type_Reactants_With_Dups(VCSList* TList__Of_Int_Type_Reactants_Ptr) const;
        static double Get_Max_Rand_Rate_Constant() {return Max_Rand_Rate_Constant;}
        static void Set_Max_Rand_Rate_Constant(double Max_Rate) {Max_Rand_Rate_Constant = Max_Rate;}

        Transformation(int R_Id, 
                       int P_Id,
                       double  Activation_Energy);

        static bool Specify_Unique_Transformation(bool R_Rand_Fl, int& Reactant_Index,
                                                  bool P_Rand_Fl, int& Product_Index,
                                                  bool ACE_Rnd_Fl, double& Actvtd_Cplex_Enrgy,
                                                  string*& Fail_Tx_Pt);


private:

//      Private Class Data

        static double Max_Rand_Rate_Constant;
//    	static double Max_Activation_Energy;

//      Private Functions

        void Add_To_Chemistry(int Reactant1_Index,
                              int Product1_Index);
        double Calculate_Reactor_Interaction_Rate
            (const Calculate_Reactor_Interaction_Rates_Args* const Args) const;
        static bool Check_If_Transformation_Exists(int Reactant1_Index, 
                                                   int Product1_Index);
        void Determine_Rate_Constants();
        void Remove_From_Chemistry();



        static bool Validate_Transformation_Parameters(int Reactant_Index,
                                                       int Product_Index,
                                                       double Actv_Cplx_Energy,
                                                       bool Rep_Fail_Reas_Fl,
                                                       string*& Fail_Tx_Pt);



//        bool Specify_Unique_Transformation(int& Reactant1_Index, int& Product1_Index, double& Rate_Constant);     
//        bool Validate_Transformation_Parameters(int& Reactant1_Index,
  //                                             int& Product1_Index);
};

//------------------------------------------------------------------------------


#endif  //Interaction_Type_H
