/****************************************************************************
*
*	  Module Filename:  Reactor.cpp
*	  Module Name:      Reactor
*	  Description:      Random Chemistry Reactor Class Implementation
*	  Author:   		Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

#include    "globldef.h"
#include    "Library_Headers.h"


namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

#include  <gsl/gsl_rng.h>    // GNU Scientific library
//---------------- Utilities Subsystem ----------------------------------------------------------------------------
#include  "Random_Sequence.h"
//----------------------------------------------------------------------------------------------------------

//--------------  Reactor  Subsystem  -----------------------------------------------------------------------------
#include    "Reactor.h"
#include    "Realised_Molecule_Type.h"
#include    "Realisable_Interaction_Type.h"
#include    "Realised_Interaction_Type.h"
//---------------------------------------------------------------------------------------------------------------

//---------------  Tracker Subsystem -----------------------------------------------------------------------------
#include    "Tracker.h"
#include    "Tracker_Manager.h"
#include    "Realised_Cycle_Type_Manager.h"
//---------------------------------------------------------------------------------------------------------------

//--------------  Chemistry  Subsystem  -----------------------------------------------------------------------------
#include    "Interaction_Type.h"
#include    "Chemistry.h"
#include    "Molecule_Type.h"
//---------------------------------------------------------------------------------------------------------------

//--------------  Statistics  Subsystem  -----------------------------------------------------------------------------
#include    "Simulation_Statistics.h"
#include    "Indexed_Series.h"
//#include    "Manhattan_Distance_Series.h"
//----------------------------------------------------------------------------------------------------------

//  Initialise Static Data

Reactor*  Reactor::Reactor__Ptr = NULL;


//----------------------------------------------------------------------------------------------------------------

void     Reactor::Add_Maintained_Molecules()

//  Add maintained Molecules to the Reactor, according to the
//  specified maintenance strategy 

{
    list<Maint_MolType_Specifier*>::iterator Iter;
    for (Iter = List_Of_Maint_MolType_Specs.begin();
         Iter not_eq List_Of_Maint_MolType_Specs.end();
         Iter++)
    BEGIN
        if ((*Iter)->MolType_Maintenance_Strategy == Maint_MolType_Specifier::Fixed_Inflow)
        THEN
            int Num_To_Add = (*Iter)->Parameter;
            for (int Count = 0;
                Count < Num_To_Add;
                Count++)
            BEGIN
                Add_Mol_Of_Type((*Iter)->MolType_Ptr, (*Iter)->Tracker_Probability);
            END
        ELSEIF ((*Iter)->MolType_Maintenance_Strategy == Maint_MolType_Specifier::Fixed_Number)
        THEN
            int Num_Mols_Of_Type;
            Realised_Molecule_Type* R_MolType_Ptr = (*Iter)->MolType_Ptr->Get_Realised_Molecule_Type__Ptr();
            if ((*Iter)->MolType_Ptr->Get_Realised_Molecule_Type__Ptr() == 0)
            THEN
                Num_Mols_Of_Type = 0;
            ELSE
                Num_Mols_Of_Type = R_MolType_Ptr->Get_Num_Of_Mols_In_Reactor();
            ENDIF

            int Num_Mols_Fixed = (*Iter)->Parameter;
            int Num_Mols_To_Add = Num_Mols_Fixed - Num_Mols_Of_Type;
            if (Num_Mols_To_Add > 0)
            THEN
                for (int Count = 0;
                     Count < Num_Mols_To_Add;
                     Count++)
                BEGIN
                    Add_Mol_Of_Type((*Iter)->MolType_Ptr, (*Iter)->Tracker_Probability);
                END
            ELSEIF (Num_Mols_To_Add < 0)
            THEN
                int Num_Mols_To_Remove = -Num_Mols_To_Add;
                for (int Count = 0;
                     Count < Num_Mols_To_Remove;
                     Count++)
                BEGIN
                    list<Tracker*>* Tracker_List_Ptr;
                    Tracker_List_Ptr = Remove_Molecule_Of_Type((*Iter)->MolType_Ptr);
                    Remove_Trackers_And_List(Tracker_List_Ptr);
                END
            ENDIF
        ELSE
            Assert (true == false,
                    " Reactor::Add_Maintained_Molecules() - Attempt to add maintained Molecule Type using a strategy that is not currently supported");
        ENDIF
    END
}
//----------------------------------------------------------------------------------------------------------------

bool    Reactor::Add_Maintained_MolType_Spec(int MolType_Index, 
                                             int Num_Mols_Fixed,
                                             double Tracker_Prob)

//  Add a Maintained Molecule Type. Select the Molecule Type at random
//  Set the inflow rate to 1
//  Set the Tracker Probability as specified
//  ******  THIS IS A LIMITED FUNCTIONALITY VERSION, JUST TO GET THINGS GOING  ***

{
//    Chemistry*  Chem_Ptr = Chemistry::Get_Chemistry__Ptr();
    int Num_MolTypes = Chemistry::Get_Num_MolTypes();
    if (Num_MolTypes > 0)
    THEN
//        MolType_Index = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Chem_Ptr->Get_Number_Of_Molecule_Types());
        
        Maint_MolType_Specifier* Spec_Ptr;
        CREATE_OBJECT(Maint_MolType_Specifier, Spec_Ptr = new Maint_MolType_Specifier);
        Spec_Ptr->MolType_Maintenance_Strategy = Maint_MolType_Specifier::Fixed_Number;
        Spec_Ptr->MolType_Ptr = Chemistry::Get_MolType_Pt(MolType_Index);
        Spec_Ptr->Parameter = Num_Mols_Fixed;
        Spec_Ptr->Tracker_Probability = Tracker_Prob;
        Spec_Ptr->Molecule_Type_Index = MolType_Index;
        
        List_Of_Maint_MolType_Specs.push_back(Spec_Ptr);    
        return true;
    ELSE           
        return false;
    ENDIF 
}
//----------------------------------------------------------------------------------------------------------

void    Reactor::Add_Mol_Of_Type(Molecule_Type*  Mol_Type_Ptr)

// Add a Molecule of the specified type to the Reactor (Overloaded version - no Tracker required)

{
    TRACE("Reactor::Add_Mol_Of_Type(Molecule_Type*  Mol_Type_Ptr");
    float  Tracker_Probability = 0.0;
    Add_Mol_Of_Type(Mol_Type_Ptr, Tracker_Probability);
}
//-------------------------------------------------------------------------------------------------------------

void    Reactor::Add_Mol_Of_Type(Molecule_Type*  Molecule_Type__Ptr, list<Tracker*>* Tracker_List_Ptr)

// Add a Molecule of the specified type to the Reactor (Overloaded version - attach the specified Tracker_List)

{
    TRACE("Reactor::Add_Mol_Of_Type(Molecule_Type*  Molecule_Type__Ptr, list<Tracker*>* Tracker_List_Ptr)");
    double  Tracker_Probability = 0.0;
    Add_Mol_Of_Type(Molecule_Type__Ptr, Tracker_Probability, Tracker_List_Ptr);
}
//--------------------------------------------------------------------------------------------------------------

void    Reactor::Add_Mol_Of_Type(Molecule_Type*  Molecule_Type__Ptr, double Tracker_Probability)

// Add a Molecule of the specified type to the Reactor (Overloaded version - attach a 
// Tracker with the specified probability)

{
    TRACE("Reactor::Add_Mol_Of_Type(Molecule_Type*  Molecule_Type__Ptr, double Tracker_Probability)");
    list<Tracker*>* Tracker_List_Ptr = NULL;
    Add_Mol_Of_Type(Molecule_Type__Ptr, Tracker_Probability, Tracker_List_Ptr);
}
//-------------------------------------------------------------------------------------------------------------
void    Reactor::Add_Mol_Of_Type(Molecule_Type*  Mol_Type_Ptr, 
                                 double Tracker_Probability, 
                                 list<Tracker*>* Tracker_List_Ptr)

//  Add a Molecule of the specified type to the Reactor. 

{   
    TRACE("Reactor::Add_Mol_Of_Type(Molecule_Type*  Mol_Type_Ptr, double Tracker_Probability, list<Tracker*>* Tracker_List_Ptr)");

//  If a Realised_Molecule_Type for the specified Molecule_type does not already exist
//  in the Reactor (ie the Molecule_Type has not previously been realised in the Reactor) then setup
//  the Realised_Molecule_Type

//  Setup a Tracker for the Molecule according to the specified probability

    Assert ((Tracker_List_Ptr == NULL) or (Tracker_Probability == 0.0), "Reactor::Add_Molecule_Of_Type - Attempt to attach Tracker and generate a Tracker at Random");

    Realised_Molecule_Type* R_Mol_Type_Ptr = Mol_Type_Ptr->Get_Realised_Molecule_Type__Ptr();
    
    if(R_Mol_Type_Ptr == NULL)
    THEN
        R_Mol_Type_Ptr = Reactor__Ptr->Add_New_Realised_Mol_Type(Mol_Type_Ptr);
    ENDIF

//  Add the Molecule. Attach the Tracker List if one specified, otherwise generate a Molecule with a Tracker List
//  and include a Tracker in the list according to the specified probability.
    
    if (Tracker_List_Ptr not_eq NULL)
    THEN
        R_Mol_Type_Ptr->Add_Molecule(Tracker_List_Ptr);
    ELSE
        R_Mol_Type_Ptr->Add_Molecule(Tracker_Probability);
    ENDIF

    Reactor__Ptr->Number_Of_Molecules_In_Reactor++;

//  Now setup any new Realisable Interaction_Types that have become possible
//  as a result of the new Molecule

    if(R_Mol_Type_Ptr->Get_Num_Of_Mols_In_Reactor() == 1)
    THEN
        Reactor__Ptr->Add_R_Int_Types_For_New_R_MolTyp(R_Mol_Type_Ptr);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
void  Reactor::Add_R_Int_Type(Interaction_Type*  Int_Type_Ptr)

//  Create a Realisable Interaction Type for the specified Interaction_Type
//  and Link to the appropriate Reactor Objects

{
    CREATE_OBJECT (Realisable_Interaction_Type,Realisable_Interaction_Type* Realisable_Int_Type__Ptr = new Realisable_Interaction_Type(Int_Type_Ptr));
    Link_R_IntType_To_Reactor_Objs(Realisable_Int_Type__Ptr);
    Assert(MolType_Is_In_Reactor(Realisable_Int_Type__Ptr->Get_Interaction_Type_Ptr()->Get_Reactant1_MolType_Ptr()),
           "Reactor::Add_R_Int_Type - Reactant 1 does not exist in Reactor");
    if(Realisable_Int_Type__Ptr->Get_Interaction_Type_Ptr()->Get_Reactant2_MolType_Ptr() not_eq 0)
    THEN
        Assert(MolType_Is_In_Reactor(Realisable_Int_Type__Ptr->Get_Interaction_Type_Ptr()->Get_Reactant2_MolType_Ptr()),
           "Reactor::Add_R_Int_Type - Reactant 2 does not exist in Reactor");
    ENDIF
       
}
//----------------------------------------------------------------------------------------------------------
Realised_Molecule_Type*  Reactor::Add_New_Realised_Mol_Type(Molecule_Type*  Molecule_Type__New_Ptr)

//  Create and initialise a Realised_Molecule_Type of the specified
//  Molecule_Type.

//  Link it to the Reactor, and to the Molecule_Type

{
    TRACE("Reactor::Add_New_Realised_Mol_Type");
    Realised_Molecule_Type*         Realised_MolType_Pt;

//  Create and initialise a Realised_Molecule_Type, with its Indexed Series
    string Ind_Ser_Name = Ind_Ser_Names_Prefixes_.MolCount
                        + To_String(Molecule_Type__New_Ptr->Get_Identifier());
    string Index_Name = Ind_Ser_Idx_Names_.Time;
    string Series_List = Ind_Ser_List_Names_.Mol_Counts;

    CREATE_OBJECT(Realised_Molecule_Type, Realised_MolType_Pt 
                    = new Realised_Molecule_Type(Molecule_Type__New_Ptr,
                                                 Ind_Ser_Name,
                                                 Index_Name, 
                                                 Ind_Ser_Name,   // Value name is same as the Indexed Series Name
                                                 Series_List));

//  Link it to the Reactor
    TList__Realised_Mol_Types_Ptr->Add(Realised_MolType_Pt);

//  Link Molecule_Type to Realised_Molecule_Type    
    Molecule_Type__New_Ptr->Link_To_Realised_Molecule_Type(Realised_MolType_Pt);

    return (Realised_MolType_Pt);
}
//----------------------------------------------------------------------------------------------------------

void Reactor::Add_R_Int_Type_Products(Realisable_Interaction_Type* Realisable_Interaction_Type__Ptr, 
                                      list<Tracker*>* Tracker_List_Ptr)

//  Add one Molecule of each of the Realisable_Interaction_Type's Products to
//  the Reactor.

//  If more than one of the Products is of the same Molecule_Type, then
//  add one Molecule for each Product.

//  Attach Trackers to the Product Molecules, if there are any in the Tracker List. The
//  first Tracker in the list is attached to one Product, the second (if it exists) to another and so on
//  until we get to the last Product, to which all other Trackers (if any) are attached.

//  The Products to which Trackers are attached are chosen at random from the Interaction Type's Products.
//  This ensures that all Cycle Types are represented during a run.  THIS DOESN'T SEEM TO HAVE BEEN IMPLEMENTED

{
    Assert ((Tracker_List_Ptr == NULL) or (Tracker_List_Ptr->empty() not_eq true),  "Reactor::Add_R_Int_Type_Products - Tracker list with no Trackers");

    CREATE_OBJECT(VCSList, VCSList* TList__Wkg_Int_Typ_Products_Ptr = new VCSList);   // Temporary List

//  Determine the Products for the Realisable_Interaction_Type

    Realisable_Interaction_Type__Ptr->Get_Interaction_Type_Ptr()
                                        ->Get_Int_Type_Products_With_Dups(TList__Wkg_Int_Typ_Products_Ptr);

    int Num_Products = TList__Wkg_Int_Typ_Products_Ptr->Count();

//  Loop over the Products

    for (int Product_Counter = 0;
         Product_Counter < Num_Products;
         Product_Counter++)
    BEGIN
        Molecule_Type*  MolType_Ptr;
        MolType_Ptr = static_cast<Molecule_Type*>(TList__Wkg_Int_Typ_Products_Ptr->Remove_Random_Value());

        if(Tracker_List_Ptr == NULL)  // No Tracker List provided
        THEN
            Add_Mol_Of_Type(MolType_Ptr,Tracker_List_Ptr);
        ELSE
            if (Tracker_List_Ptr->empty() not_eq true)
            THEN
                if ((Product_Counter < (Num_Products - 1)) and  Tracker_List_Ptr->size() > 1)
                THEN    
//                  Create a Tracker List for the new Molecule 
//                  and include one Tracker from the provided Tracker list in it. 
                    CREATE_OBJECT(list<Tracker*>, list<Tracker*>* List_Ptr = new list<Tracker*>);
                    List_Ptr->push_back(Tracker_List_Ptr->front());
                    Tracker_List_Ptr->pop_front();
                    Add_Mol_Of_Type(MolType_Ptr,List_Ptr);
                ELSE    // Add all remaining Trackers to the Product
                    Add_Mol_Of_Type(MolType_Ptr,Tracker_List_Ptr);

//                  Tracker List has been passed on, so set pointer to Null so that it is not passed on again
                    Tracker_List_Ptr = NULL;
                ENDIF
            ELSE   // No more Trackers in the specified Tracker List, so don't pass any list to it
                Add_Mol_Of_Type(MolType_Ptr);
            ENDIF
        ENDIF
    END

    Assert (TList__Wkg_Int_Typ_Products_Ptr->Count() == 0,
            "Reactor::Add_R_Int_Type_Products - Products not handled correctly");

    DELETE_OBJECT(VCSList,TList__Wkg_Int_Typ_Products_Ptr);
}

//----------------------------------------------------------------------------------------------------------
void     Reactor::Add_R_Int_Types_For_New_R_MolTyp(Realised_Molecule_Type*  Realised_Mol_Type__Ptr)

//  Determine the Realisable_Interaction_Types that
//  become possible as a result of the new Molecule_Type entering
//  the Reactor. Add each one to the Reactor.

//  Finally, record the appearance of the Molecule_Type in the Reactor
//  with a Mol_Type_Event    ***** THIS PART NOT IMPLEMENTED YET ****

{

//  Loop over Interaction_Types that are
//  possible in the Chemistry with the Realised_Molecule_Type as a Reactant
//  For each one, check whether it is Realisable; if yes, add a Realisable_Interaction_Type

    for (bool Continue_Flag = Realised_Mol_Type__Ptr->
                Molecule_Type__Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_First();
         Continue_Flag == true;
         Continue_Flag = Realised_Mol_Type__Ptr->
                Molecule_Type__Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_Next())

    BEGIN
        Interaction_Type*   Interaction_Type__Ptr =
            Realised_Mol_Type__Ptr->Molecule_Type__Ptr->List_Of_Int_Types_With_MolType_As_Input.Get_Current_Item();

//      Check whether Interaction_Type has become Realisable

        if (Is_Int_Type_Realisable(Interaction_Type__Ptr))
        THEN
            if(Interaction_Type__Ptr->Get_R_Int_Type_Ptr() == 0)
            THEN
                Add_R_Int_Type(Interaction_Type__Ptr);
            ELSE
            
//              Should never get here, because if an Interaction Type has just become realisable, it 
//              should not be linked to a R_Int_Type

                Assert (Interaction_Type__Ptr->Get_R_Int_Type_Ptr()->Get_Unrealisable() != true,
                "Reactor::Add_R_Int_Types_For_New_R_MolTyp - Interaction_Type is linked to an R_Int_Type that is Unrealisable");
            ENDIF
        END
    END
}
//----------------------------------------------------------------------------------------------------------------
   
Realised_Interaction_Type*    Reactor::Add_Realised_Int_Type(Interaction_Type* Int_Type_Ptr)

//  Create a Realised Interaction Type and add it to the Reactor's list

{
    string Ind_Ser_Name = Ind_Ser_Names_Prefixes_.IntRate 
                        + To_String(Int_Type_Ptr->Get_Identifier());
    string Index_Name = Ind_Ser_Idx_Names_.Time;
    string Series_List = Ind_Ser_List_Names_.Int_Counts;

    CREATE_OBJECT (Realised_Interaction_Type, Realised_Interaction_Type* Realised_Int_Type_Ptr 
                    = new Realised_Interaction_Type(Int_Type_Ptr,
                                                    Ind_Ser_Name,
                                                    Index_Name,
                                                    Ind_Ser_Name,   // Value name is same as Series Name
                                                    Series_List));

    List_Of_Realised_Int_Type_Ptrs.push_back(Realised_Int_Type_Ptr);
    return Realised_Int_Type_Ptr;
}
//----------------------------------------------------------------------------------------------------------

    double Reactor::Avoid_Interaction_Rate_Overflow(Realisable_Interaction_Type* R_Int_Type_Ptr,
                                                    double Basic_Int_Rate,
                                                    bool& Overflow_Close_Flag)

//  Check whether we are nearing arithmetic overflow due to a large basic Interaction Rate
//  (this can arise if both Reactants have a high concentration).
//  Limit the basic Interaction Rate, and raise an alert for the first occurrence

//  This limit is primarily applicable to Constructions in cases where both Reactants 
//  have high concentration. However, the design is general, and applies the rule 
//  to all Interaction Types

{
    double Int_Rate_No_Overflow;
    if (Basic_Int_Rate > (INT_MAX / (Timestep * 1000)))
    THEN
        Int_Rate_No_Overflow =  1000000;  // This is well below INT_MAX on any machine that SimSoup will run on

        if (Alert_Flags.Interaction_Rate_Overflow_Avoidance_Has_Started == false)
        THEN
            Alert_Flags.Interaction_Rate_Overflow_Avoidance_Has_Started = true;  

            Interaction_Type*  IntType_Ptr = R_Int_Type_Ptr->Get_Interaction_Type_Ptr();
    
            Interaction_Type_Enum  Interaction_Type_Enumr;
            int Interaction_Type_Identifier;
            int Reactant1_Identifier;
            int Reactant2_Identifier;
            int Product1_Identifier;
            int Product2_Identifier;
            int Catalyst_Identifier;
    
            IntType_Ptr->Get_Int_Type_Characteristics(Interaction_Type_Identifier,
                                                    Interaction_Type_Enumr,
                                                    Reactant1_Identifier,
                                                    Reactant2_Identifier,
                                                    Product1_Identifier,
                                                    Product2_Identifier,
                                                    Catalyst_Identifier);
        
//            string IntType_Categ = Interaction_Type::Support::Convert_Int_Type_Categ_To_Text(Interaction_Type_Enumr);
        
            string Error_Message =  string("Arithmetic Overflow Avoidance has started.")
                                + string(" SimSoup has limited the basic Undamped Interaction Rate to ")
                                + To_String(Basic_Int_Rate)
                                + string(" in order to avoid an arithmetic overflow. ")
                                + string(" This should not happen in normal circumstances. However, it is possible")
                                + string(" if a new Construction is introduced during a run and the Reactants already")
                                + string(" exist with high concentrations. In this case, the alert was")
                                + string(" triggered as a result of Interaction Type ") 
                                + To_String(Interaction_Type_Identifier)
                                + string(", which has Reactant 1 identifier = ")
                                + To_String(Reactant1_Identifier)
                                + string(". Future occurrences of Interaction Rate modification for this")
                                + string(" reason will not be reported.");
    
            Reactor_Alert(Error_Message);
        ENDIF
        Overflow_Close_Flag = true;  //  Tell caller that we have had to avoid overflow
    ELSE
        Int_Rate_No_Overflow = Basic_Int_Rate;
        Overflow_Close_Flag = false;
    ENDIF
    return Int_Rate_No_Overflow;
}
//----------------------------------------------------------------------------------------------------------------

int Reactor::Avoid_Interaction_Overshoot(Realisable_Interaction_Type* R_Int_Type_Ptr,
                                        int Num_Ints_Undamped,
                                        bool& Damping_Flag)

//  Make sure that we do not attempt to have more Interactions than are possible.
//  That is, the number of Interactions of a type cannot exceed the number of (either) Reactant 

//  If we need to limit the number, this is called Interaction Damping. The default is that
//  there is no damping, in which case the Undamped number is 'passed through'.

{       
    int Num_Act_Ints = Num_Ints_Undamped;  // Default is no damping
      
    int Num_Mols_Reactant1 
        = R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Reactant1_MolType_Ptr()
            ->Get_Realised_Molecule_Type__Ptr()->Get_Num_Of_Mols_In_Reactor();
    if(Num_Mols_Reactant1  < Num_Act_Ints)
    THEN
        if(Num_Mols_Reactant1 >10)
        THEN
            Num_Act_Ints = Num_Mols_Reactant1/10;
        ELSEIF(Num_Mols_Reactant1 > -1 )
        THEN
            if (Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(1000) < 100)
            THEN
                Num_Act_Ints = Num_Mols_Reactant1;
            ELSE
                Num_Act_Ints = 0;
            ENDIF
        ELSE 
            Assert (true == false, "Reactor::Single_Timestep - Attempting to remove negative number of Reactant1 Molecules");
        ENDIF
    ENDIF
        
    if(R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Reactant2_MolType_Ptr() not_eq 0)
    THEN    
        int Num_Mols_Reactant2
            = R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Reactant2_MolType_Ptr()
                ->Get_Realised_Molecule_Type__Ptr()->Get_Num_Of_Mols_In_Reactor();
        if(Num_Mols_Reactant2  < Num_Act_Ints)
        THEN
            if(Num_Mols_Reactant2 > 10)
            THEN
                Num_Act_Ints = Num_Mols_Reactant2/10;
            ELSEIF(Num_Mols_Reactant2 > -1 )
            THEN
                if (Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(1000) < 100)
                THEN
                    Num_Act_Ints = Num_Mols_Reactant2;   
                ELSE
                    Num_Act_Ints = 0;   
                ENDIF
            ELSE
                Assert (true == false, "Reactor::Single_Timestep - Attempting to remove negative number of Reactant2 Molecules");
            ENDIF
        ENDIF
    ENDIF   
    
    if ((Num_Ints_Undamped - Num_Act_Ints) not_eq 0) 
    THEN
        Damping_Flag = true;
        if (Alert_Flags.Interaction_Damping_Has_Occurred not_eq true)
        THEN
            Reactor_Alert(string("Interaction Damping has started. This means that")
                            + string(" applying the usual rate formula at this timestep would")
                            + string(" have led to an attempt to remove more Reactant Molecules than are present in the Reactor. ")
                            + string("To avoid this condition, try reducing the rate of input of Molecules."));
            Alert_Flags.Interaction_Damping_Has_Occurred = true;
        ENDIF
    ELSE
        Damping_Flag = false;
    ENDIF

    return Num_Act_Ints;
}
//----------------------------------------------------------------------------------------------------------------

int Reactor::Calc_Num_Ints_In_Timestep(Realisable_Interaction_Type* R_Int_Type_Ptr,
                                       int& Num_Ints_Undamped,
                                       bool& Alert_This_Timestep_Flag)

//      Calculate the (integer) number of Actual_Interactions in one timestep for the
//      specified Interaction_Rate, taking account of the timestep size.

//      Check whether we have had to avoid overflow when calculating the number of Interactions.
//      If we have, generate a Reactor Alert and limit the Interaction Rate to well within
//      the harware capabilities. Note: this is NOT the same thing as Interaction Damping.

//      Use random sampling to determine whether to round the (double) Interaction_Rate up or down.

//      Then damp the number of interactions if necessary to avoid overshoot

{
    int    Num_Act_Ints;  // Finalised Number of Interactions for the R Int Type
    double Int_Rate_Without_Overflow;
    bool Overflow_Avoidance_Flag = false;
    bool Damping_Flag = false;

//  Calculate the basic Undamped Interaction Rate

    double Basic_Int_Rate = R_Int_Type_Ptr->Update_Interaction_Rates(Reactor_Volume);


//  Check for Overflow risk. If we are close to overflow conditions then
//  Adjust Basic_Int_Rate if necessary and inform the caller

    Int_Rate_Without_Overflow = Avoid_Interaction_Rate_Overflow(R_Int_Type_Ptr, 
                                                                Basic_Int_Rate,
                                                                Overflow_Avoidance_Flag);

    int  Expected_Ints_In_Timestep_X_1000 
            = static_cast <int> (Int_Rate_Without_Overflow * Timestep * 1000);

    if ((Expected_Ints_In_Timestep_X_1000%1000) > Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(1000))
    THEN
        Num_Ints_Undamped = (Expected_Ints_In_Timestep_X_1000/1000) +1;
    ELSE
        Num_Ints_Undamped = (Expected_Ints_In_Timestep_X_1000/1000);
    END
    
//  Interaction Damping
//  Limit the number of interactions in cases where the number calculated
//  would exceed the number of Reactant Molecules available

    Num_Act_Ints= Avoid_Interaction_Overshoot(R_Int_Type_Ptr, 
                                              Num_Ints_Undamped,
                                              Damping_Flag);

    if (Overflow_Avoidance_Flag or Damping_Flag)
    THEN
        Alert_This_Timestep_Flag = true;
    ENDIF
    return Num_Act_Ints;
}
//----------------------------------------------------------------------------------------------------------------

void Reactor::Clear()

//  Clear the Reactor data structures 

{

//  Remove all Molecules from the Reactor  

    Empty();
    
//  Remove all the Maintained Molecule Type Specifiers, and clear the Reactor's list of specifiers
    list<Maint_MolType_Specifier*>::iterator Spec_Iter;
    for (Spec_Iter = List_Of_Maint_MolType_Specs.begin();
         Spec_Iter not_eq List_Of_Maint_MolType_Specs.end();
         Spec_Iter++)
    BEGIN    
        DELETE_OBJECT(Maint_MolType_Specifier, *Spec_Iter);
    END
    List_Of_Maint_MolType_Specs.clear();
    
//  Delete the Realised Molecule Types and Clear the Reactor's list of Realised Molecule Types. 
//  This will not have been done by Remove_Molecule_Of_Type()
   for (bool Continue_Flag = TList__Realised_Mol_Types_Ptr->Iter_To_First();
         Continue_Flag == true;
         Continue_Flag = TList__Realised_Mol_Types_Ptr->Iter_To_Next())
    BEGIN    
        Realised_Molecule_Type* R_MolType_Ptr 
            = static_cast <Realised_Molecule_Type*> (TList__Realised_Mol_Types_Ptr->Get_Current_Item());
        
//        DELETE_OBJECT(VCSList, R_MolType_Ptr->TList__Of_Molecules_Of_Type_Ptr);
        DELETE_OBJECT(Realised_Molecule_Type, R_MolType_Ptr);
    END
    
    TList__Realised_Mol_Types_Ptr->Clear();

//  Remove Realisable Interaction Types that have been marked Unrelisable        
    Remove_Unrealisable_R_Int_Types();
      
//  Delete the Realised Interaction Types and clear the list
    list<Realised_Interaction_Type*>::iterator Iter;
    for (Iter = List_Of_Realised_Int_Type_Ptrs.begin();
         Iter not_eq List_Of_Realised_Int_Type_Ptrs.end();
         Iter++)
    BEGIN
        DELETE_OBJECT(Realised_Interaction_Type, *Iter);                
    END
    
//  Clear the list of Interaction Types that have been Realised    
    List_Of_Realised_Int_Type_Ptrs.clear();

    Reactor_Time = 0;
        
    Total_Ints_In_Collection_Int = 0;
    Total_Constructions_In_Collection_Int = 0;
    Total_Fissions_In_Collection_Int = 0;
    Total_Transformations_In_Collection_Int = 0;
    Total_Ints_Damped_In_Collection_Int = 0;
    Total_Heat_Release_In_Collection_Int = 0.0;
    
    Assert (Is_Empty(), "Reactor::Clear() - Reactor not empty after being cleared");

//  Clear all the Manhattan Series`
    Simulation_Statistics::Get_Ptr()->Clear_Series_List(Ind_Ser_List_Names_.Manhattan_Distances);

//  Clear the Reactor Overview Indexed Series
 
    Num_Molecules_Series_Ptr->Clear();
    Num_R_Int_Types_Series_Ptr->Clear();
    Num_Ints_Per_Timestep_Series_Ptr->Clear();
    Num_MolTypes_Series_Ptr->Clear();                     
    Constructions_Per_Timestep_Series_Ptr->Clear(); 
    Fissions_Per_Timestep_Series_Ptr->Clear();  
    Transformations_Per_Timestep_Series_Ptr->Clear();     
    Int_Damping_Series_Ptr->Clear();
    Realised_Cycle_Type_Series_Ptr->Clear();
    Num_Active_Cycle_Type_Series_Ptr->Clear();
    Avg_Active_Cycle_Type_Length_Series_Ptr->Clear();
    Avg_Active_Cycle_Rate_Series_Ptr->Clear();
    Avg_Active_Cycle_Contrib_Int_Rate_Series_Ptr->Clear();
    Min_Active_Cycle_Length_Series_Ptr->Clear();
    Max_Active_Cycle_Length_Series_Ptr->Clear();
    Heat_Release_Series_Ptr->Clear();
}
//---------------------------------------------------------------------------------------------------------------

void     Reactor::Collect_Statistics(int Stats_Collection_Interval)

//  Collect Simulation Statistics

{
    TRACE("Reactor::Collect_Statistics");
    double Value;
        
//  Keep track of the number of times the 'Sigma' counters have been incremented

    if (Reactor_Time > Init_Transient_Time)
    THEN
        Run_Summary.Summation_Index += 1;  
    ENDIF

    int Num_Mols = Get_Num_Mols_In_Reactor();
    Num_Molecules_Series_Ptr->Add_Value(Reactor_Time, Num_Mols); 
    if (Reactor_Time > Init_Transient_Time)
    THEN
        Run_Summary.Max_Num_Molecules_P_I_Transient = max(Num_Mols, Run_Summary.Max_Num_Molecules_P_I_Transient);
        Run_Summary.Min_Num_Molecules_P_I_Transient = min(Num_Mols, Run_Summary.Min_Num_Molecules_P_I_Transient);
        Run_Summary.Sigma_Molecules_P_I_T += Num_Mols;
    ENDIF
    Run_Summary.Max_Num_Molecules = max(Num_Mols, Run_Summary.Max_Num_Molecules);
    Run_Summary.Min_Num_Molecules = min(Num_Mols, Run_Summary.Min_Num_Molecules);
    
    int Num_R_Int_Types = Get_Num_R_Int_Types_In_Reactor();
    Num_R_Int_Types_Series_Ptr->Add_Value(Reactor_Time, Num_R_Int_Types);
    if (Reactor_Time > Init_Transient_Time)
    THEN
        Run_Summary.Max_R_Int_Types_P_I_Transient = max(Num_R_Int_Types, Run_Summary.Max_R_Int_Types_P_I_Transient);
        Run_Summary.Min_R_Int_Types_P_I_Transient = min(Num_R_Int_Types, Run_Summary.Min_R_Int_Types_P_I_Transient);
        Run_Summary.Sigma_R_Int_Types_P_I_T += Num_R_Int_Types;
    ENDIF
    Run_Summary.Max_R_Int_Types = max(Num_R_Int_Types, Run_Summary.Max_R_Int_Types);
    Run_Summary.Min_R_Int_Types = min(Num_R_Int_Types, Run_Summary.Min_R_Int_Types);
    
    Value = double(Total_Ints_In_Collection_Int)/Stats_Collection_Interval;
    Num_Ints_Per_Timestep_Series_Ptr->Add_Value(Reactor_Time, Value); 
    Total_Ints_In_Collection_Int = 0;
    if (Reactor_Time > Init_Transient_Time)
    THEN
        Run_Summary.Max_Int_Rate_P_I_Transient = max(Value, double (Run_Summary.Max_Int_Rate_P_I_Transient));
        Run_Summary.Min_Int_Rate_P_I_Transient = min(Value, double (Run_Summary.Min_Int_Rate_P_I_Transient));
        Run_Summary.Sigma_Int_Rate_P_I_T += Value;
    ENDIF
    Run_Summary.Max_Int_Rate = max(Value, double(Run_Summary.Max_Int_Rate));
    Run_Summary.Min_Int_Rate = min(Value, double(Run_Summary.Min_Int_Rate));
    
    int Num_MolTypes = Get_Num_Mol_Types_In_Reactor();
    Num_MolTypes_Series_Ptr->Add_Value(Reactor_Time, Num_MolTypes);   
    if (Reactor_Time > Init_Transient_Time)
    THEN
        Run_Summary.Max_MolTypes_P_I_Transient = max(Num_MolTypes, Run_Summary.Max_MolTypes_P_I_Transient);
        Run_Summary.Min_MolTypes_P_I_Transient = min(Num_MolTypes, Run_Summary.Min_MolTypes_P_I_Transient);
        Run_Summary.Sigma_MolTypes_P_I_T += Num_MolTypes;
    ENDIF
    Run_Summary.Max_MolTypes = max(Num_MolTypes, Run_Summary.Max_MolTypes);
    Run_Summary.Min_MolTypes = min(Num_MolTypes, Run_Summary.Min_MolTypes);
    
    Value = double(Total_Constructions_In_Collection_Int)/Stats_Collection_Interval;
    Constructions_Per_Timestep_Series_Ptr->Add_Value(Reactor_Time, Value);
    Total_Constructions_In_Collection_Int = 0;
    
    Value = double(Total_Fissions_In_Collection_Int)/Stats_Collection_Interval;
    Fissions_Per_Timestep_Series_Ptr->Add_Value(Reactor_Time, Value);
    Total_Fissions_In_Collection_Int = 0;
    
    Value = double(Total_Transformations_In_Collection_Int)/Stats_Collection_Interval;
    Transformations_Per_Timestep_Series_Ptr->Add_Value(Reactor_Time, Value);
    Total_Transformations_In_Collection_Int = 0;
    
    Value = Total_Ints_Damped_In_Collection_Int/Stats_Collection_Interval;
    Int_Damping_Series_Ptr->Add_Value(Reactor_Time, Value);
    Run_Summary.Max_Interaction_Damping_Rate =  max(Value, Run_Summary.Max_Interaction_Damping_Rate);
    Total_Ints_Damped_In_Collection_Int = 0;

    Value = RCT_Manager_Ptr->Get_Num_Realised_Cycle_Types();
    Realised_Cycle_Type_Series_Ptr->Add_Value(Reactor_Time, Value);

    Heat_Release_Series_Ptr->Add_Value(Reactor_Time, 
                                       Total_Heat_Release_In_Collection_Int/Stats_Collection_Interval);
    Total_Heat_Release_In_Collection_Int = 0.0;

//    int Last_Time = Reactor_Time - Stats_Collection_Interval;
//    int Num_Cycle_Types;
//    double  Avg_Length;

    Realised_Cycle_Type_Manager::Active_Cycle_Summary Cycle_Stats;
    RCT_Manager_Ptr->Get_Active_Cycle_Type_Stats(Reactor_Time, Cycle_Stats);

    Num_Active_Cycle_Type_Series_Ptr->Add_Value(Reactor_Time, Cycle_Stats.Num_Cycle_Types);
    Avg_Active_Cycle_Type_Length_Series_Ptr->Add_Value(Reactor_Time, Cycle_Stats.Avg_Length);
    Avg_Active_Cycle_Rate_Series_Ptr->Add_Value(Reactor_Time, Cycle_Stats.Avg_Cycle_Rate);

    Num_Mols = Get_Num_Mols_In_Reactor();
    int Num_Trackers = Tracker_Manager::Get_Num_Trackers();
    double Adjusted_Cycle_Interaction_Rate = 0;
    if (Num_Trackers not_eq 0)
    THEN
        Adjusted_Cycle_Interaction_Rate = (Cycle_Stats.Avg_Interaction_Rate * Num_Mols) / Num_Trackers;
    ELSE
        Adjusted_Cycle_Interaction_Rate = 0;
    ENDIF
    Avg_Active_Cycle_Contrib_Int_Rate_Series_Ptr->Add_Value(Reactor_Time, Adjusted_Cycle_Interaction_Rate);
    Min_Active_Cycle_Length_Series_Ptr->Add_Value(Reactor_Time, Cycle_Stats.Min_Length);

    Max_Active_Cycle_Length_Series_Ptr->Add_Value(Reactor_Time, Cycle_Stats.Max_Length);

//    RCT_Manager_Ptr->Output_Cycle_Types();
   
//  Record the number of Molecules of each type 
      
    bool Continue = TList__Realised_Mol_Types_Ptr->Iter_To_First();
    Realised_Molecule_Type* R_Mol_Type_Ptr;
            
    while (Continue == true)   // Loop over realised Molecule Types
    BEGIN
        R_Mol_Type_Ptr =
            static_cast<Realised_Molecule_Type*> (TList__Realised_Mol_Types_Ptr->Get_Current_Item());

        int Num_Mols = R_Mol_Type_Ptr->Get_Num_Of_Mols_In_Reactor();
        
        R_Mol_Type_Ptr->Get_Series_Ptr()->Add_Value(Reactor_Time, Num_Mols);   
                    
        Continue = TList__Realised_Mol_Types_Ptr->Iter_To_Next();  
    END      
                
//  Record the number of Interactions of each type since the last time they were recorded 
//  and then reset the counts

    list<Realised_Interaction_Type*>::iterator Iter;
    for (Iter = List_Of_Realised_Int_Type_Ptrs.begin();
        Iter not_eq List_Of_Realised_Int_Type_Ptrs.end();
        Iter++)
    BEGIN
        Realised_Interaction_Type* Realised_Int_Ptr = *Iter;
        int Num_Ints = Realised_Int_Ptr->Get_Num_Act_Ints();
        Value = double(Num_Ints)/Stats_Collection_Interval;
        Realised_Int_Ptr->Get_Series_Ptr()->Add_Value(Reactor_Time, Value);    
        Realised_Int_Ptr->Set_Num_Act_Ints(0);
    END
        
//  Create and populate Manhattan_Distance_Series based on the current Reactor Time
    string Manhattan_Series_Name(Ind_Ser_Names_Prefixes_.Manhattan 
                                 + To_String(Reactor_Time));
    string Manhattan_Series_List_Name(Ind_Ser_List_Names_.Manhattan_Distances);
    string Base_Data_Series_List_Name(Ind_Ser_List_Names_.Mol_Counts);  
    double Last_Vector_Index = Reactor_Time;
    int Vector_Index_Increment = Stats_Collection_Interval;

    Simulation_Statistics::Get_Ptr()->Create_And_Populate_Manhattn_Dist_Ser
                                        (Manhattan_Series_Name,                                                               
                                         Manhattan_Series_List_Name, 
                                         Base_Data_Series_List_Name,                                                                 
                                         Last_Vector_Index,    
                                         Vector_Index_Increment);      
}
//----------------------------------------------------------------------------------------------------------------

void Reactor::Disable()

//  Disable the Reactor

{
    Enabled = false;
}
//----------------------------------------------------------------------------------------------------------------
void  Reactor::Empty()

//  Remove all the Molecules and associated Tracker data from the Reactor

{
    Chemistry* Chem_Ptr = Chemistry::Get_Chemistry__Ptr();
    for (bool Continue_Flag = Chem_Ptr->Iter_To_First_Mol_Type();
         Continue_Flag == true;
         Continue_Flag = Chem_Ptr->Iter_To_Next_Mol_Type())
    BEGIN
        Molecule_Type*  MolType_Ptr = Chem_Ptr->Get_Current_Mol_Type_Ptr();
        while (MolType_Is_In_Reactor(MolType_Ptr))
        BEGIN
            list<Tracker*>*  Trkr_Ls_Pt;
            Trkr_Ls_Pt = Remove_Molecule_Of_Type(MolType_Ptr);
            Remove_Trackers_And_List(Trkr_Ls_Pt);
        END
    END
}
//---------------------------------------------------------------------------------------------------------------

void  Reactor::Execute_Interaction(Realisable_Interaction_Type* R_Int_Type_Ptr)

//  Execute one Interaction of the specified type.
//  This entails creating the Products and removing the Reactants, and ensuring that 
//  Trackers are managed correctly

{
    list<Tracker*>*  Combined_Tracker_List_Ptr;

//  Remove the Reactants for the the Interaction and get a combined list of all Trackers that
//  were attached to them

    Combined_Tracker_List_Ptr = Remove_R_Int_Type_Reactants(R_Int_Type_Ptr);

//  Add the Products for the Interaction, and pass the Trackers on to them

    Add_R_Int_Type_Products(R_Int_Type_Ptr, Combined_Tracker_List_Ptr);

//  Increment the total Heat Release for the statistics collection interval
//  and for this timestep

    Total_Heat_Release_In_Collection_Int = Total_Heat_Release_In_Collection_Int 
        + R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Heat_Release();

    Heat_Release_In_Timestep = Heat_Release_In_Timestep 
        + R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Heat_Release();
}
//----------------------------------------------------------------------------------------------------------------
Realisable_Interaction_Type* Reactor::Iterators::Get_Current_R_Int_Type_Ptr() const

//  Return a pointer to the Realisable_Interaction_Type at which the Reactor's
//  Realisable_Interaction_Type iterator is currently located.

{
    return(Iter__R_Int_Type_Ptrs_Ptr->data());
}    
//----------------------------------------------------------------------------------------------------------
Realised_Molecule_Type* Reactor::Iterators::Get_Current_R_Mol_Type_Ptr() const

//  Return a pointer to the Realised_Molecule_Type at which the Reactor's
//  Realised_Molecule_Type iterator is currently located.

{
    return(static_cast <Realised_Molecule_Type*>
                    (Reactor__Ptr->TList__Realised_Mol_Types_Ptr->Get_Item(Iter_Realised_Mol_Types)));
}    
//----------------------------------------------------------------------------------------------------------
//Event_Log*  Reactor::Get_Event_Log_Ptr() const

//  Return a pointer to the Reactor's Event Log

//{
//    return(Event_Log__Ptr);
//}

//----------------------------------------------------------------------------------------------------------

// string  Reactor::Get_Name_For_Ind_Ser_List(Ind_Ser_List_Name_Codes Code)
// 
// //  Get the text name for the specifies Series Range List
// 
// {
//     if (Code ==  Reactor_Overview_Cd) return Ind_Ser_List_Names_.Reactor_Overview;
//     if (Code ==  Mol_Counts_Cd) return Ind_Ser_List_Names_.Mol_Counts;
//     if (Code ==  Int_Counts_Cd) return Ind_Ser_List_Names_.Int_Counts;
//     if (Code ==  Manhattan_Dist_Cd) return Ind_Ser_List_Names_.Manhattan_Distances;
// 
//     Assert (false, "Reactor::Get_Name_For_Ind_Ser_List - Invalid Series Range List Code");
// 
//     return string("Invalid Code");  // Prevents compiler warning
// }
//----------------------------------------------------------------------------------------------------------------


int Reactor::Get_Num_Mols_In_Reactor() const

{
    return(Number_Of_Molecules_In_Reactor);
}
//----------------------------------------------------------------------------------------------------------
int Reactor::Get_Num_Mol_Types_In_Reactor() const

//  Return the number of Molecule_Types CURRENTLY in the Reactor

{
    int Num_Mol_Types_In_Reactor = 0;
    bool Continue = TList__Realised_Mol_Types_Ptr->Iter_To_First();
    while(Continue)
    BEGIN
        Realised_Molecule_Type* R_Mol_Type_Ptr =
            static_cast<Realised_Molecule_Type*>  (TList__Realised_Mol_Types_Ptr->Get_Current_Item());
        if (R_Mol_Type_Ptr->Get_Num_Of_Mols_In_Reactor() not_eq 0)
        THEN
            Num_Mol_Types_In_Reactor++;
        END
        
        Continue = TList__Realised_Mol_Types_Ptr->Iter_To_Next();     
    END
    return(Num_Mol_Types_In_Reactor);
}
//----------------------------------------------------------------------------------------------------------
int     Reactor::Get_Num_R_Int_Types_In_Reactor() const

//  Return the number of Realisable_Interaction_Types in the Reactor

{
    return(List__Realisable_Int_Type_Ptrs.items());
}
//----------------------------------------------------------------------------------------------------------------

list<string> Reactor::Get_And_Clear_Reactor_Alerts()

//  Return the list of strings containing Reactor Alerts and clear the list

{
    list<string> Temp_List;
    Temp_List = Reactor__Ptr->Reactor_Alerts;
    Reactor__Ptr->Reactor_Alerts.clear();
    return Temp_List;
}
//----------------------------------------------------------------------------------------------------------------

double Reactor::Get_Fitness()

//  Return the value of the fitness function for the Reactor

{
    double Accumulator = 0.0;
    list<double>::iterator  Iter;
    for (Iter = Recent_Heat_Release.begin();
         Iter not_eq Recent_Heat_Release.end();
         Iter++)
    BEGIN
        Accumulator += *Iter;
    END
    return Accumulator;
}
//----------------------------------------------------------------------------------------------------------

Reactor*  Reactor::Get_Reactor__Ptr()

// Return a pointer to the Reactor object

{
    return(Reactor__Ptr);
}
//----------------------------------------------------------------------------------------------------------
int  Reactor::Get_Reactor_Time()  const

// Return the Reactor time

{
    return(Reactor_Time);
}
//----------------------------------------------------------------------------------------------------------
int  Reactor::Get_Reactor_Volume()  const

//  Return the Reactor Volume

{
    return(Reactor_Volume);
}
//-----------------------------------------------------------------------------------------------------------
void    Reactor::Get_Run_Summary_Statistics(double& Max_Interaction_Damping_Rate,
                                           int& Max_MolTypes,
                                           int& Max_Num_Molecules,
                                           int& Max_R_Int_Types,
                                           double& Max_Int_Rate,

                                           int& Min_MolTypes,
                                           int& Min_Num_Molecules,
                                           int& Min_R_Int_Types,
                                           double& Min_Int_Rate,

                                           int& Max_MolTypes_P_I_Transient,
                                           int& Max_Num_Molecules_P_I_Transient,
                                           int& Max_R_Int_Types_P_I_Transient,
                                           double& Max_Int_Rate_P_I_Transient,

                                           int& Min_MolTypes_P_I_Transient,
                                           int& Min_Num_Molecules_P_I_Transient,
                                           int& Min_R_Int_Types_P_I_Transient,
                                           double& Min_Int_Rate_P_I_Transient,

                                           double& Avg_Num_Molecules_P_I_T,
                                           double& Avg_Num_MolTypes_P_I_T,
                                           double& Avg_Num_R_Int_Types_P_I_T,
                                           double& Avg_Int_Rate_P_I_T,
                                           int&  Initial_Transient_Time)

{
    Max_Interaction_Damping_Rate = Run_Summary.Max_Interaction_Damping_Rate;
    Max_MolTypes = Run_Summary.Max_MolTypes;
    Max_Num_Molecules = Run_Summary.Max_Num_Molecules;
    Max_R_Int_Types = Run_Summary.Max_R_Int_Types;
    Max_Int_Rate = Run_Summary.Max_Int_Rate;

    Min_MolTypes = Run_Summary.Min_MolTypes;
    Min_Num_Molecules = Run_Summary.Min_Num_Molecules;
    Min_R_Int_Types = Run_Summary.Min_R_Int_Types;
    Min_Int_Rate = Run_Summary.Min_Int_Rate;

    Max_MolTypes_P_I_Transient = Run_Summary.Max_MolTypes_P_I_Transient;
    Max_Num_Molecules_P_I_Transient = Run_Summary.Max_Num_Molecules_P_I_Transient;
    Max_R_Int_Types_P_I_Transient = Run_Summary.Max_R_Int_Types_P_I_Transient;
    Max_Int_Rate_P_I_Transient = Run_Summary.Max_Int_Rate_P_I_Transient;

    Min_MolTypes_P_I_Transient = Run_Summary.Min_MolTypes_P_I_Transient;
    Min_Num_Molecules_P_I_Transient = Run_Summary.Min_Num_Molecules_P_I_Transient;
    Min_R_Int_Types_P_I_Transient = Run_Summary.Min_R_Int_Types_P_I_Transient;
    Min_Int_Rate_P_I_Transient = Run_Summary.Min_Int_Rate_P_I_Transient;

    
    if (Run_Summary.Summation_Index > 0)
    THEN
        Avg_Num_Molecules_P_I_T = Run_Summary.Sigma_Molecules_P_I_T / Run_Summary.Summation_Index;
        Avg_Num_MolTypes_P_I_T = Run_Summary.Sigma_MolTypes_P_I_T / Run_Summary.Summation_Index;
        Avg_Num_R_Int_Types_P_I_T = Run_Summary.Sigma_R_Int_Types_P_I_T / Run_Summary.Summation_Index;
        Avg_Int_Rate_P_I_T = Run_Summary.Sigma_Int_Rate_P_I_T / Run_Summary.Summation_Index;
    ELSE
        Avg_Num_Molecules_P_I_T     = 0;
        Avg_Num_MolTypes_P_I_T      = 0;
        Avg_Num_R_Int_Types_P_I_T   = 0;
        Avg_Int_Rate_P_I_T          = 0;
    ENDIF

    Initial_Transient_Time = this->Init_Transient_Time;
}
//----------------------------------------------------------------------------------------------------------
void  Reactor::Healthcheck()

//    Check the integrity of the Reactor data structures.
//    Fatal error if any discrepancy found

//    This function should be called immediately after all Unrealisable Interaction_Types have been removed.

{
    Realisable_Interaction_Type*   R_Int_Type_Ptr;

//  Check that the Reactants for each Realisable_Interaction_Type exist in the Reactor
//  and that note of the Realisable_Interaction_Types is Unrealisable

    DN::Iter<Realisable_Interaction_Type*>  Iterator(List__Realisable_Int_Type_Ptrs);        
    int Continue_Flag = Iterator.first();
    while (Continue_Flag == TR_OK)
    BEGIN
        R_Int_Type_Ptr = Iterator.data();
        Continue_Flag = Iterator.next();   
        Assert(MolType_Is_In_Reactor(R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Reactant1_MolType_Ptr()) == true,
               "Reactor::Healthcheck - Reactant 1 is not present in the Reactor for a Realisable Interaction Type");
        if(R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Reactant2_MolType_Ptr() not_eq 0)
        THEN
            Assert(MolType_Is_In_Reactor(R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Reactant2_MolType_Ptr()) == true,
                   "Reactor::Healthcheck - Reactant 2 is not present in the Reactor for a Realisable Interaction Type");
        ENDIF
        
        Assert(R_Int_Type_Ptr->Get_Unrealisable() == false,
           "Reactor::Healthcheck - A Realisable Interaction_Type has its Unrealisable flag set. Any such Realisable_Interaction_Types should be transient and should have been removed before the Healthcheck was called");       
    END 
   
//  Check that all of the Realisable Interaction Types linked to each Realised Molecule Type
//  are actually Realisable.  Ie, they do not have their Unrealisable flag set and all
//  of their Reactants are present in the Reactor

    bool Continue_1 = TList__Realised_Mol_Types_Ptr->Iter_To_First();
    while (Continue_1 == true)   // Loop over realised Molecule Types
    BEGIN
        Realised_Molecule_Type* R_Mol_Type_Ptr =
            static_cast<Realised_Molecule_Type*> (TList__Realised_Mol_Types_Ptr->Get_Current_Item());
                       
        Continue_1 = TList__Realised_Mol_Types_Ptr->Iter_To_Next();  
                
        DN::List<Realisable_Interaction_Type*>&  List_Of_Realisable_Int_Ptrs 
            = R_Mol_Type_Ptr->Get_Int_Type_List_Ref();         
        DN::Iter<Realisable_Interaction_Type*>  R_Int_Type_Iterator(List_Of_Realisable_Int_Ptrs);   
                      
        bool Continue_2 = R_Int_Type_Iterator.first();
        while(Continue_2 == TR_OK)  // Loop over the R Mol Type's Realisable Interaction Types
        BEGIN
            R_Int_Type_Ptr = R_Int_Type_Iterator.data();        
            Continue_2 = R_Int_Type_Iterator.next();
                     
            Assert(R_Int_Type_Ptr->Get_Unrealisable() == false,
               "Reactor::Healthcheck - A Realisable Interaction_Type has its Unrealisable flag set. Any such Realisable_Interaction_Types should be transient and should have been removed before the Healthcheck was called");       
        
            Assert(MolType_Is_In_Reactor(R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Reactant1_MolType_Ptr()) == true,
               "Reactor::Healthcheck - Reactant 1 is not present in the Reactor for a Realisable Interaction Type that is associated with a Realised Molecule Type");
            if(R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Reactant2_MolType_Ptr() not_eq 0)
            THEN
                Assert(MolType_Is_In_Reactor(R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Reactant2_MolType_Ptr()) == true,
                   "Reactor::Healthcheck - Reactant 2 is not present in the Reactor for a Realisable Interaction Type that is associated with a Realised Molecule Type");
            ENDIF      
        END 
    END   
}
//-----------------------------------------------------------------------------------------------------------------

void    Reactor::Initialise()

//  Initialise, or Re-initialise, the Reactor

{
    Reactor_Time = 0;
    Enabled = true;
    
//  Set Reactor parameters for an empty reactor

    Number_Of_Molecules_In_Reactor = 0;
    Reactor_Volume = 1000;
    Timestep = 1.0;
    Reactor_Leakage_Proportion = 0.001;

    Alert_Flags.Interaction_Damping_Has_Occurred = false;
    Alert_Flags.Interaction_Rate_Overflow_Avoidance_Has_Started = false;

    Simulation_Statistics*  SimStats_Ptr = Simulation_Statistics::Get_Ptr();

//  Setup the Reactor Overview list of Indexed Series, 
//  and create the Indexed Series themselves

    const string Overvw_Ls_Nm_Tx = Ind_Ser_List_Names_.Reactor_Overview;
    SimStats_Ptr->Create_Indexed_Series_List(Overvw_Ls_Nm_Tx, true);  // Is for user access
    SimStats_Ptr->Set_Default_SerList(Overvw_Ls_Nm_Tx); // Make this the default Series List 
    SimStats_Ptr->Iter_To_First_Ind_Series_List();

    Num_Molecules_Series_Ptr 
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Mols, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Mols,  //  In general, the Value Name is
                                                                    //  the same as the Series Name
                                              Overvw_Ls_Nm_Tx,
                                              true);
    
    Num_R_Int_Types_Series_Ptr 
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Realisable_IntTypes, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Realisable_IntTypes,
                                              Overvw_Ls_Nm_Tx,
                                              true);
    Num_Ints_Per_Timestep_Series_Ptr 
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.IntRate, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.IntRate, 
                                              Overvw_Ls_Nm_Tx,
                                              true);
    
    Num_MolTypes_Series_Ptr 
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.MolTypes, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.MolTypes, 
                                              Overvw_Ls_Nm_Tx,
                                              true);
                                                
    Constructions_Per_Timestep_Series_Ptr 
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Constr_Rate, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Constr_Rate, 
                                              Overvw_Ls_Nm_Tx ,
                                              true);
    Fissions_Per_Timestep_Series_Ptr  
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Fission_Rate, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Fission_Rate, 
                                              Overvw_Ls_Nm_Tx ,
                                              true);
                                                
    Transformations_Per_Timestep_Series_Ptr 
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Transfm_Rate, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Transfm_Rate, 
                                              Overvw_Ls_Nm_Tx ,
                                              true);
                                                
    Int_Damping_Series_Ptr
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Damped_IntRate, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Damped_IntRate, 
                                              Overvw_Ls_Nm_Tx,
                                              true);

    Realised_Cycle_Type_Series_Ptr
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Realsd_Cycle_Types, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Realsd_Cycle_Types, 
                                              Overvw_Ls_Nm_Tx,
                                              true);

    Num_Active_Cycle_Type_Series_Ptr
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Num_Actv_Cycl_Types, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Num_Actv_Cycl_Types, 
                                              Overvw_Ls_Nm_Tx,
                                              true);

    Avg_Active_Cycle_Type_Length_Series_Ptr
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Avg_Actv_Cycl_Type_Len, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Avg_Actv_Cycl_Type_Len, 
                                              Overvw_Ls_Nm_Tx ,
                                              true);

    Avg_Active_Cycle_Rate_Series_Ptr
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Avg_Actv_Cycl_Rate, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Avg_Actv_Cycl_Rate, 
                                              Overvw_Ls_Nm_Tx,
                                              true);

    Avg_Active_Cycle_Contrib_Int_Rate_Series_Ptr
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Avg_Actv_Cycl_Contrib_IntRate, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Avg_Actv_Cycl_Contrib_IntRate, 
                                              Overvw_Ls_Nm_Tx,
                                              true);

    Min_Active_Cycle_Length_Series_Ptr
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Actv_Cycl_Min_Len, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Actv_Cycl_Min_Len, 
                                              Overvw_Ls_Nm_Tx,
                                              true);

    Max_Active_Cycle_Length_Series_Ptr
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Actv_Cycl_Max_Len, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Actv_Cycl_Max_Len, 
                                              Overvw_Ls_Nm_Tx,
                                              true);

    Heat_Release_Series_Ptr
        = SimStats_Ptr->Create_Indexed_Series(Ind_Ser_Names_.Heat_Release, 
                                              Ind_Ser_Idx_Names_.Time,
                                              Ind_Ser_Names_.Heat_Release, 
                                              Overvw_Ls_Nm_Tx,
                                              true);

//      Create Indexed_Series Lists for Reactor 
//      (The series themselves will be created as the simulation runs)                           
        SimStats_Ptr->Create_Indexed_Series_List(Ind_Ser_List_Names_.Mol_Counts, true);   // Is for user access
        SimStats_Ptr->Create_Indexed_Series_List(Ind_Ser_List_Names_.Int_Counts, true);     
        SimStats_Ptr->Create_Indexed_Series_List(Ind_Ser_List_Names_.Manhattan_Distances, false); // Not for user access

//      Initialise Run_Summary_Statistics

        Run_Summary.Max_Num_Molecules = 0;
        Run_Summary.Max_MolTypes = 0;
        Run_Summary.Max_R_Int_Types = 0;
        Run_Summary.Max_Int_Rate = 0;

        Run_Summary.Max_Interaction_Damping_Rate = 0;

        Run_Summary.Min_Num_Molecules = INT_MAX;
        Run_Summary.Min_MolTypes = INT_MAX;
        Run_Summary.Min_R_Int_Types = INT_MAX;
        Run_Summary.Min_Int_Rate = DBL_MAX;

        Run_Summary.Max_Num_Molecules_P_I_Transient = 0;
        Run_Summary.Max_MolTypes_P_I_Transient = 0;
        Run_Summary.Max_R_Int_Types_P_I_Transient = 0;
        Run_Summary.Max_Int_Rate_P_I_Transient = 0;

        Run_Summary.Min_Num_Molecules_P_I_Transient = INT_MAX;
        Run_Summary.Min_MolTypes_P_I_Transient = INT_MAX;
        Run_Summary.Min_R_Int_Types_P_I_Transient = INT_MAX;
        Run_Summary.Min_Int_Rate_P_I_Transient = DBL_MAX;

        Run_Summary.Sigma_Molecules_P_I_T  = 0;  
        Run_Summary.Sigma_MolTypes_P_I_T  = 0;   
        Run_Summary.Sigma_R_Int_Types_P_I_T  = 0;
        Run_Summary.Sigma_Int_Rate_P_I_T  = 0;
        Run_Summary.Summation_Index = 0;

        Init_Transient_Time = 50000;  // Used for calculating Run Summary Statistics
}
//-----------------------------------------------------------------------------------------------------------------
bool     Reactor::Is_Int_Type_Realisable(Interaction_Type*  Int_Type_Ptr) const

//  Determine whether the specified Interaction_Type is Realisable in the Reactor

{
    CREATE_OBJECT(VCSList,VCSList* TList__Of_Interaction_Inputs_Ptr = new VCSList);
                                               //  List of Reactants and
                                               //  Catalysts in an Interaction_Type

    Int_Type_Ptr->Get_Int_Type_Inputs_No_Dups(TList__Of_Interaction_Inputs_Ptr);

//  Check whether the Interaction_Type's Inputs exist in the Reactor

    bool All_Inputs_In_Reactor_Flag = true;

    for (int Interaction_Input_Counter = 0;
         Interaction_Input_Counter < TList__Of_Interaction_Inputs_Ptr->Count();
         Interaction_Input_Counter++)

    BEGIN
        Molecule_Type* Molecule_Type__Ptr
            = static_cast<Molecule_Type*>
                (TList__Of_Interaction_Inputs_Ptr
                    ->Get_Item(Interaction_Input_Counter));

        if (MolType_Is_In_Reactor(Molecule_Type__Ptr) == false)
        THEN
            All_Inputs_In_Reactor_Flag = false;
            break;
        ENDIF
    END
    DELETE_OBJECT(VCSList,TList__Of_Interaction_Inputs_Ptr);

    if (All_Inputs_In_Reactor_Flag == true)
    THEN

//      The Interaction_Type is Realisable unless it has two identical Reactants
//      and there are less than two Molecules of this type in the Reactor,

        bool Same_MolTyp = false;
        int Num_Mols_Same_MolTyp;
        if (Int_Type_Ptr->Get_Reactant1_MolType_Ptr() == Int_Type_Ptr->Get_Reactant2_MolType_Ptr())
        THEN
            Same_MolTyp = true;
            Num_Mols_Same_MolTyp = Int_Type_Ptr->Get_Reactant1_MolType_Ptr()->
                                    Get_Realised_Molecule_Type__Ptr()->Get_Num_Of_Mols_In_Reactor();
        ENDIF

        if(Same_MolTyp && (Num_Mols_Same_MolTyp < 2))
        THEN
            return false;
        ELSE
            return true;
        ENDIF
    ELSE
        return false;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

bool    Reactor::Is_Empty()

//  Check whether the Reactor is Empty

{
    if(Iterators__Ptr->R_Int_Types_Exist() == true)
    THEN
        return  false;
    ENDIF
    
    if(Iterators__Ptr->Iter_To_First_R_Mol_Type() == true)
    THEN
        return false;
    ENDIF
    
    if (Number_Of_Molecules_In_Reactor not_eq 0)
    THEN
        return false;
    ENDIF
    
    return true;
}
//----------------------------------------------------------------------------------------------------------
Reactor::Iterators::Iterators(Reactor* Creator_Ptr) :  Reactor__Ptr(Creator_Ptr)

//  Constructor for Reactor's Iterator object.

{
    CREATE_OBJECT(DN::Iter<Realisable_Interaction_Type*>, Iter__R_Int_Type_Ptrs_Ptr = new DN::Iter<Realisable_Interaction_Type*> (Reactor__Ptr->List__Realisable_Int_Type_Ptrs);)
}
//----------------------------------------------------------------------------------------------------------
bool  Reactor::Iterators::Iter_To_First_R_Int_Type() const

//  Move the Iterator to the first Realisable_Interaction_Type

{
    if(Iter__R_Int_Type_Ptrs_Ptr->first() == TR_OK)
    THEN
        return(true);
    ELSE
        return(false);
    END
}
//----------------------------------------------------------------------------------------------------------
bool  Reactor::Iterators::Iter_To_First_R_Mol_Type()

//  Move the Iterator to the first Realised_Molecule_Type 

{
    if(Reactor__Ptr->TList__Realised_Mol_Types_Ptr->Count() > 0)
    THEN
        Iter_Realised_Mol_Types = 0;
        return(true);
    ELSE
        return(false);

    END
}
//----------------------------------------------------------------------------------------------------------
bool  Reactor::Iterators::Iter_To_Next_R_Int_Type() const

//  Move the Iterator to the next Realisable_Interaction_Type 

{
    if(Iter__R_Int_Type_Ptrs_Ptr->next() == TR_OK)
    THEN
        return(true);
    ELSE
        return(false);
    END
}
//----------------------------------------------------------------------------------------------------------
bool  Reactor::Iterators::Iter_To_Next_R_Mol_Type()

//  Move the Iterator to the next Realised_Molecule_Type

{
    if(Iter_Realised_Mol_Types < Reactor__Ptr->TList__Realised_Mol_Types_Ptr->Count() - 1)
    THEN
        Iter_Realised_Mol_Types++;
        return(true);
    ELSE
        return(false);
    END

}
//----------------------------------------------------------------------------------------------------------------

bool    Reactor::Iterators::R_Int_Types_Exist()

//  Return true if at least one Realisable Interaction Type pointer exists

{
    if(Iter__R_Int_Type_Ptrs_Ptr->first() not_eq TR_EMPTY)
    THEN
        return(true);
    ELSE
        return(false);
    END
}
//----------------------------------------------------------------------------------------------------------
void  Reactor::Link_R_IntType_To_Inp_R_MolTypes(Realisable_Interaction_Type*
                                                Realisable_Interaction_Type__Ptr)

//  Link the specified Realisable_Interaction_Type to the
//  Realised_Molecule_Types corresponding to the
//  Realisable_Interaction_Type's Inputs
//  The linkage is both ways. That is, the Realisable_Interaction_Type
//  is added to lists held by the Realisable_Molecule_Types, and in addition, the
//  Realisable_Interaction_Type holds a list of Nodes in the Realisable_Molecule_Types's lists
//  to enable quick unlinking.

{
    Molecule_Type*    Molecule_Type__Input_Ptr;
    Realised_Molecule_Type*     Realised_Molecule_Type__Ptr;

//  Get the Inputs for the Realisable_Interaction_Type in a working TList

    CREATE_OBJECT(VCSList, VCSList* TList__Wkg_Int_Type_Inputs_Ptr = new VCSList); // Temporary List

    Interaction_Type*  I_Ptr;

    I_Ptr = Realisable_Interaction_Type__Ptr->Get_Interaction_Type_Ptr();

    I_Ptr->Get_Int_Type_Inputs_No_Dups(TList__Wkg_Int_Type_Inputs_Ptr);

//  Loop over the Reactants to make the links

    for (int Interaction_Type_Input_Counter = 0;
         Interaction_Type_Input_Counter < TList__Wkg_Int_Type_Inputs_Ptr->Count();
         Interaction_Type_Input_Counter++)

    BEGIN

        Molecule_Type__Input_Ptr
            = static_cast<Molecule_Type*> (TList__Wkg_Int_Type_Inputs_Ptr->Get_Item(Interaction_Type_Input_Counter));

        Realised_Molecule_Type__Ptr = Molecule_Type__Input_Ptr->Get_Realised_Molecule_Type__Ptr();

//      Link  (both ways). DN::List's add() method creates a Node object
//      First add the R Int Type to the R Mool type's list

        DN::Node<Realisable_Interaction_Type*>* Node_Ptr;
        CREATE_OBJECT(DN::Node<Realisable_Interaction_Type*>,
                      Node_Ptr = Realised_Molecule_Type__Ptr->List__Of_Realisable_Int_Ptrs.add(Realisable_Interaction_Type__Ptr));

//      Now add the created list item's node to the R Int type's list
                      
        CREATE_OBJECT(DN::Node<DN::Node<Realisable_Interaction_Type*>*>, 
                      Realisable_Interaction_Type__Ptr->List__Nodes_In_R_Mol_Types_Lists.add(Node_Ptr));
    END
    DELETE_OBJECT(VCSList, TList__Wkg_Int_Type_Inputs_Ptr);
}
//----------------------------------------------------------------------------------------------------------

void  Reactor::Make()

{
    TRACE("Reactor::Make");
    Assert(Reactor__Ptr == 0,"Reactor::Make - The instance pointer for the singleton instance is not zero");
    CREATE_OBJECT(Reactor, Reactor__Ptr = new Reactor);
}
//----------------------------------------------------------------------------------------------------------------

void   Reactor::Link_R_IntType_To_Reactor_Objs(Realisable_Interaction_Type*
                                               Realisable_Int_Type__Ptr)

//  Link the specified Realisable Interaction Type to the Reactor
//  and to its Input Realisable Molecule Types

{

//  Link to the Reactor (Both ways)

//  First add the R Int Type top the Reactor's list of R Int Types. 
//  This will create a Node for the list item

    DN::Node<Realisable_Interaction_Type*>* Node_Ptr;
    CREATE_OBJECT(DN::Node<Realisable_Interaction_Type*>,
                  Node_Ptr = List__Realisable_Int_Type_Ptrs.add(Realisable_Int_Type__Ptr));
                  
//  Now store a pointer to the Node in the R Int type

    Realisable_Int_Type__Ptr->Set_Node_In_Reactors_List_Ptr(Node_Ptr);                     

//  Link to the Realised_Molecule_Types that are Inputs to the Realisable_Interaction_Type

    Link_R_IntType_To_Inp_R_MolTypes(Realisable_Int_Type__Ptr);
}
//----------------------------------------------------------------------------------------------------------------

bool    Reactor::MolType_Is_In_Reactor(Molecule_Type*  MolType_Ptr)  const

//  Return true if there is at least one Molecule of the specified type in the Reactor

{
    Realised_Molecule_Type* R_MolType_Ptr = MolType_Ptr->Get_Realised_Molecule_Type__Ptr();

    if (R_MolType_Ptr == NULL)
    THEN
        return(false);
    ELSEIF (R_MolType_Ptr->Get_Num_Of_Mols_In_Reactor() == 0)
    THEN
        return (false);
    ELSE
        return(true);     
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
void    Reactor::Notify_New_Int_Type(Interaction_Type*  Int_Type_Ptr)

//  This function enables an external caller to notify the Reactor of
//  a new Interaction_Type. When called, it takes appropriate action.

//  Specifically, it checks whether the Interaction_Type is Realisable, and if yes,
//  adds the appropriate Realisable_Interaction_Type to the Reactor. 

{
    if (Is_Int_Type_Realisable(Int_Type_Ptr))
    THEN
        Add_R_Int_Type(Int_Type_Ptr);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
void    Reactor::Notify_Removed_Int_Type(Interaction_Type*  Int_Type_Ptr)

//  Inform the Reactor that an Interaction_Type has been removed from
//  the Chemistry.  The Reactor can then take appropriate action. 

{
    Realisable_Interaction_Type*  R_Int_Type_Ptr = Int_Type_Ptr->Get_R_Int_Type_Ptr();

    Assert(R_Int_Type_Ptr != 0, "Reactor::Notify_Removed_Int_Type - Invalid Realisable Interaction Type Pointer");


    R_Int_Type_Ptr->Set_Unrealisable();  // Indicates the Realisable_Interaction_Type
                                          // has become Unrealisable and hence eligible for deletion
                                          // Also unlinks from Interaction_type

//  Unlink the Realisable Interaction Type from its Input Realised Molecule Types

    Unlink_R_IntType_From_Inp_R_MolTypes(R_Int_Type_Ptr);
              
//  Unlink from the Reactor and delete
              
    Remove_R_Int_Type(R_Int_Type_Ptr);
    
//  Remove from the Reactor's list of Realised Interaction Types
        
 //   List_Of_Realised_Int_Type_Ptrs.remove(Int_Type_Ptr);  //  *******  NEED TO REMOVE THE REALISED INT TYPE
}
//----------------------------------------------------------------------------------------------------------

Reactor::Reactor()

//  Create an empty Reactor and its Event Log (if logging enabled)
//  Also setup the Iterators object

{

//  First Check whether or not a Reactor already exists in memory.

    Assert(Reactor__Ptr == 0, "Reactor::Reactor - A Reactor is already open.");

    Reactor__Ptr = this;

    CREATE_OBJECT(VCSList,TList__Realised_Mol_Types_Ptr = new VCSList);

//  Create the Reactor's Iterators object
    CREATE_OBJECT(Reactor::Iterators,Iterators__Ptr = new Reactor::Iterators(this));

//  Create the Reactor's Tracker Manager and Cycle Manager

    CREATE_OBJECT(Realised_Cycle_Type_Manager, RCT_Manager_Ptr = new Realised_Cycle_Type_Manager);
    CREATE_OBJECT(Tracker_Manager, Tracker_Manager_Ptr = new Tracker_Manager(RCT_Manager_Ptr));


    Number_Of_Molecules_In_Reactor = 0;

//  Setup the standard Indexed Series Lists and their associated Indexed Series
//  Lists first
    Ind_Ser_List_Names_.Reactor_Overview = string("Reactor Overview");
    Ind_Ser_List_Names_.Mol_Counts = string("Molecule Counts");
    Ind_Ser_List_Names_.Int_Counts =  string("Interaction Counts");
    Ind_Ser_List_Names_.Manhattan_Distances = string("Manhattan Distances");

//  Now the Indexed Series Names
    Ind_Ser_Names_.Mols = string("Molecules");
    Ind_Ser_Names_.Realisable_IntTypes = string ("Realisable Int. Types");
    Ind_Ser_Names_.IntRate = string ("Interaction Rate");
    Ind_Ser_Names_.MolTypes = string ("Molecule Types");
    Ind_Ser_Names_.Constr_Rate = string ("Construction Rate");
    Ind_Ser_Names_.Fission_Rate = string ("Fission Rate");
    Ind_Ser_Names_.Transfm_Rate = string ("Transformation Rate");
    Ind_Ser_Names_.Damped_IntRate = string ("Damped Interaction Rate");
    Ind_Ser_Names_.Realsd_Cycle_Types = string ("Realised Cycle Types");
    Ind_Ser_Names_.Num_Actv_Cycl_Types = string ("Num. Active Cycle Types");
    Ind_Ser_Names_.Avg_Actv_Cycl_Type_Len = string ("Avg. Active Cycle Type Length");
    Ind_Ser_Names_.Avg_Actv_Cycl_Rate = string ("Avg. Active Cycle Rate");
    Ind_Ser_Names_.Avg_Actv_Cycl_Contrib_IntRate = string ("Avg. Active Cycle Contrib. Int. Rate");
    Ind_Ser_Names_.Actv_Cycl_Min_Len = string ("Active Cycle Min. Length");
    Ind_Ser_Names_.Actv_Cycl_Max_Len = string ("Active Cycle Max. Length"); 
    Ind_Ser_Names_.Heat_Release = string ("Heat Release Rate");

    // Names for Indexes in Indexed Series
    Ind_Ser_Idx_Names_.Time = string("Time");

//  Name prefixes for automatically generated series
    Ind_Ser_Names_Prefixes_.Manhattan = string("Manhattan");
    Ind_Ser_Names_Prefixes_.IntRate = string("Rate: Int ");        
    Ind_Ser_Names_Prefixes_.MolCount = string("Count: Mol ");        
 
    Initialise();
}

//----------------------------------------------------------------------------------------------------------------

void Reactor::Reactor_Alert(string Alert_Text)

//   Record a Reactor Alert

{                 
    Reactor_Alerts.push_back(string("***** WARNING ***** ") + Alert_Text);
}
//---------------------------------------------------------------------------------------------------------------

bool Reactor::Remove_Maintained_MolType_Spec(int& MolType_Index)

//  Remove the first Maintained Molecule Type Specifier

{    
    if (List_Of_Maint_MolType_Specs.empty())
    THEN
        return false;
    ELSE
        MolType_Index = (List_Of_Maint_MolType_Specs.front())->MolType_Ptr->Get_Identifier();
        List_Of_Maint_MolType_Specs.pop_front();
        return true;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------

list<Tracker*>*    Reactor::Remove_Molecule_Of_Type(Molecule_Type*  Mol_Type_Ptr)

//  Remove a Molecule of the specified type from the Reactor.
//  Return the pointer to any attched Tracker as returned by Remove_Molecule

//  The Realised_Molecule_Type is NOT removed. It remains, but with its number of Molecules as Zero.

{
    Realised_Molecule_Type*  R_Mol_Type_Ptr = Mol_Type_Ptr->Get_Realised_Molecule_Type__Ptr();

    Assert(R_Mol_Type_Ptr->Get_Num_Of_Mols_In_Reactor() > 0, "Reactor::Remove_Molecule_Of_Type attempted to remove a Molecule of a Molecule_Type that is not realised in the Reactor.");

    Number_Of_Molecules_In_Reactor--;
        
    int Before_Num_Mols = R_Mol_Type_Ptr->Get_Num_Of_Mols_In_Reactor();  // Used for assertion            
    
    list<Tracker*>*  Attached_Tracker_List_Ptr = R_Mol_Type_Ptr->Remove_Molecule();
    
//  Remove Realisable Interaction Types if need be        
    if (R_Mol_Type_Ptr->Get_Num_Of_Mols_In_Reactor() < 3)  // Actual value not critical
    THEN
        Rem_R_Int_Types_For_Old_R_MolTyp(R_Mol_Type_Ptr);

//      The Realised_Molecule_Type is NOT removed
    ENDIF
      
    Assert (R_Mol_Type_Ptr->Get_Num_Of_Mols_In_Reactor() == (Before_Num_Mols-1), " Reactor::Remove_Molecule_Of_Type - Incorrect number of molecules removed");

    return Attached_Tracker_List_Ptr;
}
//----------------------------------------------------------------------------------------------------------
void    Reactor::Remove_Outflow_Molecules()

//  Remove Molecules at Random from the Reactor
//  Also delete any associated Trackers and the Tracker List that contains them
//  ****  Need to finalise and control the outflow rate

{
//    double Proportion_To_Remove = 0.0001;
    unsigned int Num_Mols_Of_Type_To_Remove;


    for (int R_Mol_Type_Counter = 0;
         R_Mol_Type_Counter < TList__Realised_Mol_Types_Ptr->Count();

         R_Mol_Type_Counter++)
    BEGIN
        Molecule_Type*  Molecule_Type__Ptr;
        Realised_Molecule_Type*  Realised_Molecule_Type__Ptr;

        Realised_Molecule_Type__Ptr
            = static_cast<Realised_Molecule_Type*> (TList__Realised_Mol_Types_Ptr->Get_Item(R_Mol_Type_Counter));

        unsigned int Num_Mols_Of_Type_To_Remove_X100000
            = static_cast <int> (Realised_Molecule_Type__Ptr->Get_Num_Of_Mols_In_Reactor()*Reactor_Leakage_Proportion*100000);
        
//      Example:  Num in Reactor = 2, then  int Num_Mols_Of_Type_To_Remove = 2/100000 = 0
        Num_Mols_Of_Type_To_Remove = Num_Mols_Of_Type_To_Remove_X100000/100000;

//      Example  Excess = 2 - 0 = 2        
        int Excess =  Num_Mols_Of_Type_To_Remove_X100000 - (Num_Mols_Of_Type_To_Remove*100000);
        if (Excess > Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(100000)) // Remove one extra molecule, based on random sample.
        THEN
            Num_Mols_Of_Type_To_Remove++;
        ENDIF

//      Remove Molecule and any Trackers and Tracker List

        if (Num_Mols_Of_Type_To_Remove > 0)
        THEN
            Molecule_Type__Ptr = Realised_Molecule_Type__Ptr->Get_Molecule_Type_Ptr();
            for (unsigned int Molecule_Counter = 0;
                 Molecule_Counter < Num_Mols_Of_Type_To_Remove;
                 Molecule_Counter++)
            BEGIN
                list<Tracker*>*  Tracker_List_Ptr;
                Tracker_List_Ptr = Remove_Molecule_Of_Type(Molecule_Type__Ptr);
                Remove_Trackers_And_List(Tracker_List_Ptr);
/*
                list<Tracker*>::iterator Iter;
                if (Tracker_List_Ptr not_eq NULL)
                THEN
                    for (Iter = Tracker_List_Ptr->begin();
                         Iter not_eq Tracker_List_Ptr->end();
                         ++Iter)
                    BEGIN
                         Get_Tracker_Manager_Ptr()->Remove_Tracker(*Iter);
                    END
                    DELETE_OBJECT(list<Tracker*>, Tracker_List_Ptr);
                END
*/
            END
        ENDIF
    END 
}

//----------------------------------------------------------------------------------------------------------
void    Reactor::Remove_R_Int_Type(Realisable_Interaction_Type*  Realisable_Interaction_Type__Ptr)

//  Unlink the Realisable_Interaction_Type from the Reactor's list of Realisable Interaction Types
//  Then delete the Realisable_Interaction_Type
//  (Set_Unrealisable unlinks from the Interaction Type)

{

//  Delete the Node in the list. The Node's destructor will unlink from the list and relink the remaining 
//  Nodes in the list.

    DELETE_OBJECT(DN::Node<Realisable_Interaction_Type*>, Realisable_Interaction_Type__Ptr->Get_Node_In_Reactors_List_Ptr());

//    delete Realisable_Interaction_Type__Ptr->Get_Node_In_Reactors_List_Ptr();  //  This will also unlink from
                                                                               //  Reactor List
    DELETE_OBJECT(Realisable_Interaction_Type, Realisable_Interaction_Type__Ptr);
}
//----------------------------------------------------------------------------------------------------------
list<Tracker*>*   Reactor::Remove_R_Int_Type_Reactants(Realisable_Interaction_Type*
                                                       Realisable_Interaction_Type__Ptr)

//  Remove one Molecule of each of the Realisable_Interaction_Type's Reactants from
//  the Reactor. If more than one of the Reactants is of the same Molecule_Type, then
//  remove one Molecule for each Reactant.

//  If any of the Reactants being removed has a Tracker list, then return a pointer to a combined list

{
    CREATE_OBJECT(VCSList, VCSList* TList__Wkg_Int_Typ_Reactants_Ptr = new VCSList);   // Temporary List

    Realisable_Interaction_Type__Ptr->Get_Interaction_Type_Ptr()->Get_Int_Type_Reactants_With_Dups(TList__Wkg_Int_Typ_Reactants_Ptr);

    list<Tracker*>* First_Tracker_List_Ptr = NULL;
    list<Tracker*>* Second_Tracker_List_Ptr = NULL;
    list<Tracker*>* Combined_Tracker_List_Ptr = NULL;

    Molecule_Type*  First_MolType_Ptr;
    Molecule_Type*  Second_MolType_Ptr;

//  Determine the Reactants for the Realisable_Interaction_Type, remove them, and get their Tracker Lists

    int Num_Reactants = TList__Wkg_Int_Typ_Reactants_Ptr->Count();

    Assert ((Num_Reactants == 1) or (Num_Reactants == 2), "Reactor::Remove_R_Int_Type_Reactants - Invalid number of Reactants");

    First_MolType_Ptr = static_cast<Molecule_Type*> (TList__Wkg_Int_Typ_Reactants_Ptr->Get_Item(0));
    First_Tracker_List_Ptr = Remove_Molecule_Of_Type(First_MolType_Ptr); 

    if (Num_Reactants > 1)
    THEN
        Second_MolType_Ptr = static_cast<Molecule_Type*> (TList__Wkg_Int_Typ_Reactants_Ptr->Get_Item(1));
        Second_Tracker_List_Ptr = Remove_Molecule_Of_Type(Second_MolType_Ptr); 
    ENDIF
 
//  Return a pointer to a combined Tracker list, or null pointer if neither Reactant has a Tracker List
//  Base the combined list on the second Reactant's list if it had one. Otherwise use the 
//  first Reactant's list

    if (First_Tracker_List_Ptr not_eq NULL)
    THEN
        if (Second_Tracker_List_Ptr not_eq NULL)
        THEN
            Combined_Tracker_List_Ptr = Second_Tracker_List_Ptr;
            while(First_Tracker_List_Ptr->empty() not_eq true)
            BEGIN
                Combined_Tracker_List_Ptr->push_back(First_Tracker_List_Ptr->front());
                First_Tracker_List_Ptr->pop_front();
            END

//          Delete the first Reactant's Tracker List. We only keep the second one.
            DELETE_OBJECT(list<Tracker*>, First_Tracker_List_Ptr);
        ELSE
            Combined_Tracker_List_Ptr = First_Tracker_List_Ptr;
        ENDIF
    ELSE
        Combined_Tracker_List_Ptr = Second_Tracker_List_Ptr;  // might be null
    ENDIF

    DELETE_OBJECT(VCSList,TList__Wkg_Int_Typ_Reactants_Ptr);  

    return Combined_Tracker_List_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

void   Reactor::Remove_Trackers_And_List(list<Tracker*>* Tracker_List_Ptr)

{
    list<Tracker*>::iterator Iter;
    if (Tracker_List_Ptr not_eq NULL)
    THEN
        for (Iter = Tracker_List_Ptr->begin();
                Iter not_eq Tracker_List_Ptr->end();
                ++Iter)
        BEGIN
            Get_Tracker_Manager_Ptr()->Remove_Tracker(*Iter);
        END
        DELETE_OBJECT(list<Tracker*>, Tracker_List_Ptr);
    END
}
//----------------------------------------------------------------------------------------------------------------

void   Reactor::Remove_Unrealisable_R_Int_Types()

//  Remove Realisable_Interaction_Types that have become Unrealisable

{
    DN::Iter<Realisable_Interaction_Type*>  Iterator(List__Realisable_Int_Type_Ptrs);
    Realisable_Interaction_Type* R_Int_Type_Ptr;
    
    bool Continue_Flag = Iterator.first();  
    while (Continue_Flag == TR_OK)
    BEGIN
        R_Int_Type_Ptr = Iterator.data();
        Continue_Flag = Iterator.next();   // Must move iterator forward prior to unlinking
        if (R_Int_Type_Ptr->Get_Unrealisable() == true)
        THEN
            Remove_R_Int_Type(R_Int_Type_Ptr);     // Unlinks the current node
        ENDIF
    END
}
//----------------------------------------------------------------------------------------------------------
void  Reactor::Rem_R_Int_Types_For_Old_R_MolTyp(Realised_Molecule_Type*  Realised_Molecule_Type__Ptr)

//  Check the Realised_Molecule_Type's Realisable_Interaction_Types to see
//  if they have become unrealisable (see below for rules)
//  Mark those that have as Unrealisable.
//  Then unlink them from the the Realised_Molecule_Types that are Inputs to the

//  Realisable_Interaction_Types.
//  Add an entry to the Event Log for each Realisable_Interactio_Type that has
//  become Unrealisable.

//  Finally, record the disappearance of the Molecule_Type in the Reactor
//  with a Mol_Type_Event  *****  THIS PART NOT IMPLEMENTED YET ******

{
//  Iterator for the Realised_Molecule_Type's List of Realisable Interaction Types
    DN::Iter<Realisable_Interaction_Type*> Iter__R_Int_Type_Ptr (Realised_Molecule_Type__Ptr->List__Of_Realisable_Int_Ptrs);

//  Vector to hold temporary list of Realisable Interaction Types to be unlinked from Input Realised Molecule Types    
    
    vector<Realisable_Interaction_Type*> List_Of_R_Int_Types_To_Unlink; 
        
    for (int Continue_Flag = Iter__R_Int_Type_Ptr.first();
         Continue_Flag == TR_OK;
         Continue_Flag = Iter__R_Int_Type_Ptr.next())
    BEGIN
        Realisable_Interaction_Type*  R_Int_Type__Ptr = Iter__R_Int_Type_Ptr.data();
                        
        Assert (R_Int_Type__Ptr != 0, "Reactor::Rem_R_Int_Type_For_Old_R_MolType - Invalid Realisable Interaction Type Pointer");
        int Num_Mols_Old_MolTyp = Realised_Molecule_Type__Ptr->Get_Num_Of_Mols_In_Reactor();

        Interaction_Type*  Interaction_Type__Ptr = R_Int_Type__Ptr->Get_Interaction_Type_Ptr();

        Assert (Interaction_Type__Ptr != 0, "Reactor::Rem_R_Int_Type_For_Old_R_MolType - Invalid Interaction Type Pointer");

        Molecule_Type*  Mol_Type1_Ptr =  Interaction_Type__Ptr->Get_Reactant1_MolType_Ptr();

        Assert (Mol_Type1_Ptr != 0, "Reactor::Rem_R_Int_Type_For_Old_R_MolType - Invalid Molecule Type 1 Pointer");

        Molecule_Type*  Mol_Type2_Ptr =  Interaction_Type__Ptr->Get_Reactant2_MolType_Ptr();

        bool Same_MolTyp = false;

        if (Mol_Type1_Ptr == Mol_Type2_Ptr)
        THEN
            Same_MolTyp = true;
        ENDIF

        Assert (Same_MolTyp == false, "Reactor::Rem_R_Int_Types_For_Old_R_MolTyp - Interaction Type has Reactants of same type. This is currently not supported by SimSoup");

        if(Num_Mols_Old_MolTyp == 0)
        THEN
            R_Int_Type__Ptr->Set_Unrealisable();  // Indicates the Realisable_Interaction_Type
                                                  // has become Unrealisable and hence eligible for deletion
                                                  // Also unlinks from Interaction_type
    
//          Store pointer to R Int Type for unlinking later. Its not safe to unlink it here
//          because this function is called from another function that iterates over a list of
//          R Int Types for an R Mol Type, and unlinking here would remove an item from the list 
//          that is being iterated.

            List_Of_R_Int_Types_To_Unlink.push_back(R_Int_Type__Ptr);
            
            Assert(List_Of_R_Int_Types_To_Unlink.size() < 5000, 
            "Reactor::Rem_R_Int_Types_For_Old_R_MolTyp - Too many Int Types to unlink");
        END

        if (MolType_Is_In_Reactor(Mol_Type1_Ptr) == false)
        THEN
            Assert (R_Int_Type__Ptr->Get_Unrealisable() == true, 
                "Reactor::Rem_R_Int_Types_For_Old_R_MolTyp - Int Type is not Unrealisable but no Reactant 1 Mols exist.");
 //                      + string (To_String(Realised_Molecule_Type__Ptr->Get_Molecule_Type_Ptr()->Get_Identifier()))
 //                      + string (".  Reactant 1 Mol Type = ")
 //                      + string (To_String(Mol_Type1_Ptr->Get_Identifier()))).c_str());
       ENDIF
     
        if ((Mol_Type2_Ptr not_eq 0) and (MolType_Is_In_Reactor(Mol_Type1_Ptr) == false))
        THEN
            Assert (R_Int_Type__Ptr->Get_Unrealisable() == true, 
                "Reactor::Rem_R_Int_Types_For_Old_R_MolTyp - Int Type is not Unrealisable but no Reactant 2 Mols");
        ENDIF    
     
     END
     
//  Now execute the deferred unlinking activity

    for (vector<Realisable_Interaction_Type*>::iterator Iter = List_Of_R_Int_Types_To_Unlink.begin();
         Iter not_eq List_Of_R_Int_Types_To_Unlink.end();
         ++Iter)
    BEGIN
        Unlink_R_IntType_From_Inp_R_MolTypes(*Iter);
    END
}
//-----------------------------------------------------------------------------------------------------------------
/*
void    Reactor::Set_Num_Maint_MolTypes(int Num_MolTypes)

//  Set the number of Maintained Molecule Types

{
    Assert (Num_MolTypes >= 0, "Reactor::Set_Num_Maint_MolTypes - Attempt to set a negative value for number of Maintained Molecule Types");
    Number_Of_Maintained_Mol_Types = Num_MolTypes;
}
//-----------------------------------------------------------------------------------------------------------------

void    Reactor::Set_Inflow_Rate(int Reactor_Inflow_Rate)

//  Set the Reactor Inflow Rate

{
    Assert (Reactor_Inflow_Rate >= 0, "Reactor::Set_Inflow_Rate - Attempt to set a negative value for the Reactor Inflow Rate");
    Maintained_Mol_Types_Inflow_Rate = Reactor_Inflow_Rate;
}  */
//----------------------------------------------------------------------------------------------------------
bool  Reactor::Single_Timestep(int Stats_Collection_Interval)

//  Carry out the activity for a single timestep in the Reactor

//  FOR each Realisable_Interaction_Type (that has not been logically deleted)
//  BEGIN

//      Update_Interaction_Rate:-
//      Calculate and update the Interaction_Rate for the current Realisable_Interaction_Type
//      based on the Num_Of_Mols_In_Reactors of the Inputs.

//      Calc_Num_Ints_In_Timestep:-
//      Calculate the number of Actual_Interactions in this timestep
//      taking account of the timestep size.  Use random sampling to determine the integer number
//      of interactions based on the calculated Interaction_Rate and the timestep size.

//      FOR the number of Actual_Interactions
//      BEGIN
//          Add_Interaction_Products
//          Remove_Interaction_Reactants (and Realisable_Interaction Types if need be)
//      END
//  END

//  Physically remove Realisable_Interaction_Types that were logically deleted

// Add_Maintained_Molecules
// Remove_Outflow_Molecules

// Collect Simulation Statistics
// Increment Reactor Time


{
    TRACE("Reactor::Single_Timestep");
    DN::Iter<Realisable_Interaction_Type*>  Iterator(List__Realisable_Int_Type_Ptrs);
    Realisable_Interaction_Type*   R_Int_Type_Ptr;
    bool Alert_This_Timestep_Flag = false;
    bool Alert_For_R_Int_Type_Flag = false;

    Heat_Release_In_Timestep = 0.0;  //  Used to calculate fitness

    if (!Enabled) return Alert_This_Timestep_Flag;  // If Reactor is disabled then do nothing

//  Main Simulation loop over all Realisable_Interaction_Types in the Reactor

    bool Continue_Flag;
    for(Continue_Flag = Iterator.first();
        Continue_Flag == TR_OK;
        Continue_Flag = Iterator.next()
       )
    BEGIN
        R_Int_Type_Ptr = Iterator.data();
        if (R_Int_Type_Ptr->Get_Unrealisable() == false)
        THEN
            int Num_Ints_Undamped;
            int Num_Act_Ints = Calc_Num_Ints_In_Timestep(R_Int_Type_Ptr, 
                                                         Num_Ints_Undamped, 
                                                         Alert_For_R_Int_Type_Flag);

            Assert (Num_Act_Ints >=0, "Reactor::Single_Timestep - Calculated number of Interactions is negative!");
            if (Alert_For_R_Int_Type_Flag == true)
            THEN
                Alert_This_Timestep_Flag = true;
            ENDIF

            Assert (Num_Ints_Undamped >= Num_Act_Ints, "Reactor::Single_Timestep() - Number of damped Interactions is negative");
                            
            Total_Ints_Damped_In_Collection_Int += Num_Ints_Undamped - Num_Act_Ints;
            Total_Ints_In_Collection_Int += Num_Act_Ints;  // Remember for Statistics collection    
            
//          Count number of Interactions of each Realised Interaction Type
            
            Interaction_Type* Int_Type_Ptr = R_Int_Type_Ptr->Get_Interaction_Type_Ptr();
            int Previous_Ints = Int_Type_Ptr->Get_Realised_Int_Type_Ptr()->Get_Num_Act_Ints();
            Int_Type_Ptr->Get_Realised_Int_Type_Ptr()->Set_Num_Act_Ints(Num_Act_Ints + Previous_Ints);
            
//          Count Constructions, Fissions and Transformations            
           
            Interaction_Type_Enum Int_Categ = Int_Type_Ptr->Get_Category();
            if(Int_Categ == Interaction_Type_Enumr__Constr)
            THEN
                Total_Constructions_In_Collection_Int += Num_Act_Ints;
            ELSEIF (Int_Categ == Interaction_Type_Enumr__Fission)
            THEN
                Total_Fissions_In_Collection_Int += Num_Act_Ints;
            ELSEIF (Int_Categ == Interaction_Type_Enumr__Transfm)
            THEN
                Total_Transformations_In_Collection_Int += Num_Act_Ints;
            ENDIF

//          Create the Products and remove the Reactants
    
            for(int Act_Int_Counter = 0;
                Act_Int_Counter < Num_Act_Ints;
                Act_Int_Counter++)
            BEGIN
                Execute_Interaction(R_Int_Type_Ptr);
            END
        ENDIF
    END
  
//  Loop over Reactor's Realisable_Interaction_Types again to remove
//  those that became Unrealisable.  Its safer to use a separate loop than
//  to try to do this within the main simulation loop.

    Remove_Unrealisable_R_Int_Types();
   
//  Check Reactor data structures    
    
    Healthcheck();
                    
    Add_Maintained_Molecules();
   
//  Remove_Outflow_Molecules

    Remove_Outflow_Molecules();   
        
//  Record Simulation Statistics    

    if(Reactor_Time % Stats_Collection_Interval == 0)
    THEN
        Collect_Statistics(Stats_Collection_Interval);
        Chemistry::Get_Chemistry__Ptr()->Collect_Statistics(Reactor_Time);
    ENDIF

    Update_Fitness();

    Reactor_Time++;

    return Alert_This_Timestep_Flag;
 }
//----------------------------------------------------------------------------------------------------------


void    Reactor::Unlink_R_IntType_From_Inp_R_MolTypes(Realisable_Interaction_Type*
                                                      R_Int_Type__Ptr)
                                                     
//  Unlink the specified Realisable_Interaction_Type from the
//  Realised_Molecule_Types corresponding to the Realisable_Interaction_Type's Inputs
//  The linkage is both ways. That is, the Realisable_Interaction_Type
//  is removed from lists held by the Realisable_Molecule_Types, and in addition, the
//  Realisable_Interaction_Type holds a list of Nodes in the Realisable_Molecule_Types's lists
//  to enable quick unlinking - entries are also removed from this list.


{

//  Use the Realisable_Interaction_Type's List of Nodes in the Realised Molecule Type lists
//  to build the temporary list of nodes for unlinking.
//  Also remove items from the Realisable_Interaction_Type's list

    DN::Iter <DN::Node<Realisable_Interaction_Type*>* >
        Iter__Node_Ptr (R_Int_Type__Ptr->List__Nodes_In_R_Mol_Types_Lists);

//  Temporary list of pointers to nodes in lists attached to Realised_Molecule_Types. Each node
//  identifies a Realisable_Interaction_Type whose Interaction_Type has become Unrealisable.
//  Pointers to nodes are placed in this temporary list to enable the nodes themselves to be
//  unlinked from the lists attached to the Realised_Molecule_Types.

    vector<DN::Node<Realisable_Interaction_Type*>* >  Nodes_To_Unlink_From_R_MolTyp_Lists;

    while(Iter__Node_Ptr.first() == TR_OK) //  Nodes are removed in the body of the loop


    BEGIN
    
//      Store a pointer to a Node that is currently in a Realised_Molecule_Type's
//      List of Realisable_Interaction_Types. The Node will be unlinked later;
//      its not safe to unlink it here.

        Nodes_To_Unlink_From_R_MolTyp_Lists.push_back(Iter__Node_Ptr.data());

//      Now unlink and delete the Node in the Realisable_Interaction_Type's List
//      of Nodes that are in Lists attached to Realised_Molecule_Types.
//      Do this by deleting the Node itself. The destructor invokes the unlink() method
//      which reconnects the two ends of the linked list. Iterator will now point to
//      the new first Node (if one exists).

        DELETE_OBJECT(DN::Node<DN::Node<Realisable_Interaction_Type*>*>, Iter__Node_Ptr.atNode());                                                                                  
    END

//  Now use the temporary list of nodes to remove references to the Realisable_Interaction_Type
//  from the lists of R_Int_Types that are maintained with the Realised_Molecule_Types
//  that provide inputs to the R_Int_Type.

    vector<DN::Node<Realisable_Interaction_Type*>* >::const_iterator Iterator__Nodes_To_Unlink;
    for (Iterator__Nodes_To_Unlink = Nodes_To_Unlink_From_R_MolTyp_Lists.begin();
        Iterator__Nodes_To_Unlink != Nodes_To_Unlink_From_R_MolTyp_Lists.end();
        ++Iterator__Nodes_To_Unlink)
    BEGIN
        DELETE_OBJECT(DN::Node<Realisable_Interaction_Type*>, *Iterator__Nodes_To_Unlink);
    END
}
//----------------------------------------------------------------------------------------------------------------

void Reactor::Update_Fitness()

//  Update the Reactor Fitness function
//  Currently implemented as the average Heat Release Rate

{
    Recent_Heat_Release.push_back(Heat_Release_In_Timestep);
    if(Recent_Heat_Release.size() > 500)
    THEN
        Recent_Heat_Release.pop_front();
    ENDIF
}



