/****************************************************************************
*
*	  Module Filename:  Action_Request.cpp
*	  Module Name:      Action_Request
*	  Description:      Specifies and controls a particular action required
*                       as part of a simulation Scenario
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/
//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

#include "globldef.h"
#include "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

#include  <gsl/gsl_rng.h>    // GNU Scientific library
//---------------- Utilities Subsystem ----------------------------------------------------------------------------
#include  "Random_Sequence.h"
//----------------------------------------------------------------------------------------------------------

//------------- Kernel_Codes  ------------------------------------------------------------------------------------
#include  "Kernel_Codes.h"
//----------------------------------------------------------------------------------------------------------------

//--------------  Simulation  Subsystem  -----------------------------------------------------------------------
#include "Action_Request.h"
#include "ActReq_Requestor.h"
#include "Attribute.h"
//--------------------------------------------------------------------------------------------------------------

//--------------  Chemistry  Subsystem  -----------------------------------------------------------------------
#include "Chemistry.h"
#include "Molecule_Type.h"
#include "Interaction_Type.h"
//--------------------------------------------------------------------------------------------------------------

//--------------  Reactor  Subsystem  ---------------------------------------------------------------------------
#include "Reactor.h"
#include "Realisable_Interaction_Type.h"
//--------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------

//  Initialise static variables

//const string Action_Request::Warning_String = string("**** WARNING ****");
//const string Action_Request::Equals_Tx = string(" = ");
const string Action_Request::Repeat_Req_Tx = string(" First in a repeat sequence. Successful repeats will not be reported.");
        

//---------------------------------------------------------------------------------------------------------------

Action_Request::~Action_Request()

{

} 
//----------------------------------------------------------------------------------------------------------------

Action_Request::Action_Request(int Time, int Repeat_Interval):

    Time_(Time),
    Repeat_Req_(false),
    Repeat_Interval_(Repeat_Interval),
    Fail_Fl_(false),
    Prev_Exec_Fail_Fl_(false)

{
    TRACE("Action_Request::Action_Request(int Time, int Repeat_Interval, bool Repeat_Req)");
}
//----------------------------------------------------------------------------------------------------------------
bool Action_Request::Compare_Time(const Action_Request* x, const Action_Request* y)

//  Compare times of Action Requests. Return true if time of x less than y, otherwise false

{
    TRACE("Action_Request::Compare_Time");
    return x->Get_Time() < y->Get_Time();
}
//-----------------------------------------------------------------------------------------------------------------

void Action_Request::Exec_Fail(const char*  Fail_Tx_Pt) 

//  Record execution failure and setup specified failure reason
//  Only do this if its the first execution for this Action Request (not a repeat)
//  or if its the first in a repeat sequence that has given an error

{
    if (not Repeat_Req_
                     or
        (Fail_Fl_ and (not Prev_Exec_Fail_Fl_)))
 
    THEN
        if (Fail_Fl_) Prev_Exec_Fail_Fl_ = true;
        Fail_Fl_ = true;
        Fail_Reason_Tx_ = string(Fail_Tx_Pt);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void Action_Request::Exec_Fail(const string*  Fail_Tx_Pt) 

//  Record execution failure and setup specified failure reason
//  Only do this if its the first execution for this Action Request (not a repeat)
//  or if its the first in a repeat sequence that has given an error

{
    TRACE("Action_Request::Exec_Fail(const string*  Fail_Tx_Pt)");
    if (not Repeat_Req_
                    or
        (Fail_Fl_ and (not Prev_Exec_Fail_Fl_)))

    THEN
        if (Fail_Fl_) Prev_Exec_Fail_Fl_ = true;
        Fail_Fl_ = true;
        Fail_Reason_Tx_ = string(*Fail_Tx_Pt);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

string  Action_Request::Get_Act_Rslt_Tx() const

//  Get a text string describing the result of the Action Request execution

{
    TRACE("Action_Request::Get_Act_Rslt_Tx");
    return Fail_Fl_ ? Fail_Msg() : Succ_Msg();
}
//----------------------------------------------------------------------------------------------------------------

bool Action_Request::Get_Fail_Fl()  const
 
//  Return whether the latest execution failed
 
{
     TRACE("Action_Request::Get_Fail_Fl");
     return  Fail_Fl_;
}
//----------------------------------------------------------------------------------------------------------------
bool Action_Request::Get_Prev_Fail_Fl()  const
 
//  Return whether or not there has been a previous failure
 
{
     TRACE("Action_Request::Get_Prev_Fail_Fl");
     return  Prev_Exec_Fail_Fl_;
}
//----------------------------------------------------------------------------------------------------------------

int  Action_Request::Get_Time() const

//  Return the time for the Action Request

{
    TRACE("Action_Request::Get_Time");
    return Time_;
}
//------------------------------------------------------------------------------------------------------------------

int  Action_Request::Get_Repeat_Interval() const

//  Return the Repeat Interval 

{
    TRACE("Action_Request::Get_Repeat_Interval");
    return  Repeat_Interval_;
}
//----------------------------------------------------------------------------------------------------------------

bool Action_Request::Get_Repeat_Req_Fl()  const
 
//  Return the Repeat Request Flag
 
{
     TRACE("Action_Request::Get_Repeat_Req_Fl");
     return  Repeat_Req_;
}
//-----------------------------------------------------------------------------------------------------------------

void Action_Request::Exec_Success(const char*  Tx_Pt)

//  Record execution success 
//  Only do this if its the first execution for this Action Request (not a repeat)
 
{
    if (not Repeat_Req_)

    THEN
        Fail_Fl_ = false;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

bool  Action_Request::Setup_Repeat_If_Reqd()

//  Check whether a repeat action is required
//  If so schedule it using the same Action Request

{
    TRACE("Action_Request::Sched_Repeat_If_Reqd");
    Assert (Reactor::Time() == Time_, "Action_Request::Setup_Repeat_If_Reqd - Attempt to schedule a repeat of an action request at a time other than Reactor time for the previous execution. If a repeat execution is required, then it must be scheduled immediately after an execution before advancing Reactor time.");

    if(Repeat_Interval_)
    THEN
        Time_ += Repeat_Interval_;  //  Setup the repeat
        Repeat_Req_ = true;
        return true;
    ELSE
        Repeat_Req_ = false;
        return false;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

bool ActReq_Help::MolType_Spec_2_Id(const MolType_Spec*  const MT_Spec_Pt, 
                                    int& MT_Id, 
                                    string& Rslt_Tx)

//  Determine an actual MolType Id based on the MolType_Spec.
//  If the MolType_Spec indicates a random Id, then a random Id
//  is chosen, taking account of the number of MolTypes currently in the Chemistry.
//  Otherwise the Id is as in the MolType_Spec.

//  Return false and a message if no appropriate MolType exists

{
    int N_MolTypes = Chemistry::Get_Num_MolTypes();
 
    if (MT_Spec_Pt->Get_Rnd_Fl())
    THEN
        if (N_MolTypes < 1)
        THEN
            Rslt_Tx += string("No Molecule Types exist in Chemistry");
            return false;
        ELSE
            MT_Id = RndSeq::Get_Ptr(RndSeq::One)->Uniform(N_MolTypes);   
            return true;
        ENDIF
    ELSE
        MT_Id = MT_Spec_Pt->Get_MolType_Id();
        Assert (MT_Id >= 0, "ActReq_Help::MolType_Spec_2_Id - Negative MolType Id");

        if (N_MolTypes < (MT_Id + 1))
        THEN
            Rslt_Tx += string("Molecule_Type ") 
                    + To_String(MT_Id) 
                    + string(" does not exist");
            return false;
        ELSE
            return true;
        ENDIF
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------
// bool Action_Request::Validate_Activated_Complex_Energy_Spec(string  Activated__Complex_Energy_Spec)
// 
// //  Validate that the specification for Activated Complex Energy is either a non negative number 
// //  or the text 'Random'
// 
// {
//     TRACE("Action_Request::Validate_Activated_Complex_Energy_Spec");
//     if ((Activated__Complex_Energy_Spec not_eq "Random")
//             and
//        (Validate_Text_As_Non_Neg_Int(Activated__Complex_Energy_Spec) not_eq true))
//     THEN
//         return false;
//     ELSE
//         return true;
//     ENDIF
// }

//----------------------------------------------------------------------------------------------------------------
// bool Action_Request::Validate_Reactant_Product_Spec(string  MolType_Spec)
// 
// //  Validate that the specification for a Reactant or Product is either a non negative number 
// //  or the text 'Random'
// 
// {
//     TRACE("Action_Request::Validate_Reactant_Product_Spec");
//     if ((MolType_Spec not_eq "Random")
//             and
//        (Validate_Text_As_Non_Neg_Int(MolType_Spec) not_eq true))
//     THEN
//         return false;
//     ELSE
//         return true;
//     ENDIF
// }

//===============================================================================================================

Add_Cat_Trans_Req::Add_Cat_Trans_Req(Add_Cat_Trans_ReqRqstr*  Rqstr_Pt):
    Action_Request(Rqstr_Pt->Get_Action_Time(), 
    Rqstr_Pt->Get_Repeat_Interval())


//  Constructor

{
    Time_ = Rqstr_Pt->Get_Action_Time();  //  ******  Need to also validate this
    Num_To_Add = Rqstr_Pt->Num_To_Add;
    Repeat_Interval_ = Rqstr_Pt->Get_Repeat_Interval();
//    Repeat_Req = false;
    
//     if (Validate_Reactant_Product_Spec(Requestor_Ptr->Reactant_Spec) not_eq true)
//     THEN
//         Assert(true == false, "Add_Cat_Trans_Req::Add_Cat_Trans_Req - Invalid Reactant Spec: ");
//     ENDIF

/*//     if (Validate_Reactant_Product_Spec(Requestor_Ptr->Catalyst_Spec) not_eq true)
//     THEN
//         Assert(true == false, "Add_Cat_Trans_Req::Add_Cat_Trans_Req - Invalid Catalyst Spec: ");
//     ENDIF

    if (Validate_Reactant_Product_Spec(Requestor_Ptr->Product_Spec) not_eq true)
    THEN
        Assert(true == false, "Add_Cat_Trans_Req::Add_Cat_Trans_Req - Invalid Product Spec: ");
    ENDIF*/
	
    Reactant_Spec = Rqstr_Pt->Reactant_Spec;
    Catalyst_Spec = Rqstr_Pt->Catalyst_Spec;
    Product_Spec =  Rqstr_Pt->Product_Spec;
}
//-----------------------------------------------------------------------------------------------------------------

string Add_Cat_Trans_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the Add Catalytic Transformation Action Request

{
    string Action_Result;
    if (Chemistry::Get_Chemistry__Ptr()->Get_Number_Of_Interaction_Types() + 2 * Num_To_Add > 40000)
    THEN 
        Action_Result = Get_Warn_Tx() 
                        + string(" Cannot add ") 
                        + To_String(Num_To_Add) 
                        + string(" Catalytic Transformations. SimSoup has not yet been tested with more than 40000 Interaction Types");
    ELSE
        try
        BEGIN    
            Action_Result = To_String(Num_To_Add) + string(" Catalytic Transformations added");
            for (int ILoop = 0; ILoop < Num_To_Add; ILoop++)
            BEGIN
                Make_And_Notify_Int_Types_For_Cat_Trans();
            END
        END

        catch (Create_Error  Excep)
        BEGIN
            Action_Result =   Get_Warn_Tx()
                            + string(" Failed to add all of the ")  
                            + To_String(Num_To_Add) 
                            + string(" Catalytic Transformations requested. ")
                            + Excep.Get_Error_Text();
        END
    ENDIF

    return  Action_Result;
}
//-----------------------------------------------------------------------------------------------------------------

int  Add_Cat_Trans_Req::Get_Number()

//  Return the number of Catalytic Transformations to be added

{
    return Num_To_Add;
}
//-----------------------------------------------------------------------------------------------------------------
void  Add_Cat_Trans_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Add Catalytic Transformation action request

{
    Param_List.push_back(string("Random"));
//    Param_List.push_back(string("Catalyst = ") + Catalyst_Spec);   // **** CURRENTLY NOT SUPPORTED
//    Param_List.push_back(string("Product = ") + Product_Spec);     // **** CURRENTLY NOT SUPPORTED
}
//-----------------------------------------------------------------------------------------------------------------
string Add_Cat_Trans_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Catalytic Transformation");
}
//-----------------------------------------------------------------------------------------------------------------        
string Add_Cat_Trans_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Add");   
}
//----------------------------------------------------------------------------------------------------------------
        
void    Add_Cat_Trans_Req::Make_And_Notify_Int_Types_For_Cat_Trans()

//  Setup unique characteristics for this Catalytic_Transformation
//  and then add the Construction and Fission that make up the Catalytic Construction to the Chemistry
//  Include reverse reactions, so that we have two pairs of Interaction Types, one consisting of 
//  a Construction and a Fission making up the forward Catalytic Transformation, and another
//  pair making up the reverse Catalytic Transformation.

{
    int     Cat_Trans_Reactant_Index;
    int     Cat_Trans_Product_Index;
    int     Cat_Trans_Catalyst_Index;
    bool    Cat_Trans_Reactant_Random_Flag = false;
    bool    Cat_Trans_Product_Random_Flag = false;
    bool    Cat_Trans_Catalyst_Random_Flag = false;
    int     Intermediate_Product_Index; 
    double  Construction_Actv_Cplx_Energy;
    double  Fission_Actv_Cplx_Energy;

    if (Reactant_Spec == "Random")
    THEN
        Cat_Trans_Reactant_Random_Flag = true;
    ELSE
        Text_To_Int(Reactant_Spec, Cat_Trans_Reactant_Index);
    ENDIF

    if (Product_Spec == "Random")
    THEN
        Cat_Trans_Product_Random_Flag = true;
    ELSE
        Text_To_Int(Product_Spec, Cat_Trans_Product_Index);
    ENDIF

    if (Catalyst_Spec == "Random")
    THEN
        Cat_Trans_Catalyst_Random_Flag = true;
    ELSE
        Text_To_Int(Catalyst_Spec, Cat_Trans_Catalyst_Index);      
    ENDIF

    int Ret_Code = Specify_Cat_Trans(Cat_Trans_Reactant_Random_Flag, Cat_Trans_Reactant_Index,
                                     Cat_Trans_Product_Random_Flag, Cat_Trans_Product_Index,
                                     Cat_Trans_Catalyst_Random_Flag, Cat_Trans_Catalyst_Index,
                                     Intermediate_Product_Index, 
				                     Construction_Actv_Cplx_Energy,
				                     Fission_Actv_Cplx_Energy);

    if (Ret_Code not_eq 0)
    THEN
        if (Ret_Code == -1)
        THEN
            throw Create_Error(string("Insufficient possibilities for new Constructions."));
        ELSEIF (Ret_Code == -2)
        THEN
            throw Create_Error(string("Unable to find suitable Fission for any Construction."));
        ELSE
            Assert(true == false, 
		   "Catalytic_Transformation::Catalytic_Transformation - Invalid return code from Specify_Cat_Trans");
        ENDIF
    ENDIF
                             
//    Chemistry* Chemistry_Ptr = Chemistry::Get_Chemistry__Ptr();
//    Molecule_Type* Const_Reactant1_Ptr  =  Chemistry::Get_MolType_Pt(Cat_Trans_Reactant_Index);
//    Molecule_Type* Const_Reactant2_Ptr  =  Chemistry::Get_MolType_Pt(Cat_Trans_Catalyst_Index);
//    Molecule_Type* Const_Product1_Ptr   =  Chemistry::Get_MolType_Pt(Intermediate_Product_Index);
//    Molecule_Type* Fission_Product1_Ptr =  Chemistry::Get_MolType_Pt(Cat_Trans_Product_Index);

//----------------------------------------------------------------------------------------------------------------

//  Setup the forward Catalytic Transformation

        
//    Construction* Constr_Ptr;
//    CREATE_OBJECT(Construction, Constr_Ptr = new Construction(Const_Reactant1_Ptr, 
//                                                              Const_Reactant2_Ptr, 
//                                                              Const_Product1_Ptr, 
//                                                              Construction_Actv_Cplx_Energy));

//    Fission*  Fission_Ptr;
//    CREATE_OBJECT(Fission, Fission_Ptr = new Fission(Const_Product1_Ptr, 
//						                             Const_Reactant2_Ptr, 
//						                             Fission_Product1_Ptr, 
//						                             Fission_Actv_Cplx_Energy));

//----------------------------------------------------------------------------------------------------------------
/*
//  Now setup the reverse Catalytic Transformation

//  Reactant 1 for the Construction is the Product for the forward Catalytic transformation
//  Product 1 for the Fission is the Reactant for the forward Catalytic Transformation 
//  The Catalyst is still Reactant 2 for the Construction and the Intermediate is 
//  still the Product for the Construction.
//  The Activated Complex Energy for the Construction in the reverse Catalytic Transformation
//  is the Activated Complex Energy for the Fission in the forward Catalytic Transformation,
//  and vice versa

    Molecule_Type* Rev_CT_Const_Reactant1_Ptr  =  Chemistry_Ptr->Get_Molecule_Type_Ptr(Cat_Trans_Product_Index);
    Molecule_Type* Rev_CT_Const_Reactant2_Ptr  =  Chemistry_Ptr->Get_Molecule_Type_Ptr(Cat_Trans_Catalyst_Index);
    Molecule_Type* Rev_CT_Const_Product1_Ptr   =  Chemistry_Ptr->Get_Molecule_Type_Ptr(Intermediate_Product_Index);
    Molecule_Type* Rev_CT_Fission_Product1_Ptr =  Chemistry_Ptr->Get_Molecule_Type_Ptr(Cat_Trans_Reactant_Index);
    double  Rev_CT_Construction_Actv_Cplx_Energy = Fission_Actv_Cplx_Energy;
    double  Rev_CT_Fission_Actv_Cplx_Energy      = Construction_Actv_Cplx_Energy;

    CREATE_OBJECT(Construction, Constr_Ptr = new Construction(Rev_CT_Const_Reactant1_Ptr, 
                                                              Rev_CT_Const_Reactant2_Ptr, 
                                                              Rev_CT_Const_Product1_Ptr, 
                                                              Rev_CT_Construction_Actv_Cplx_Energy));


    CREATE_OBJECT(Fission, Fission_Ptr = new Fission(Rev_CT_Const_Product1_Ptr, 
                                                     Rev_CT_Const_Reactant2_Ptr, 
                                                     Rev_CT_Fission_Product1_Ptr, 
                                                     Rev_CT_Fission_Actv_Cplx_Energy));

*/
}
//---------------------------------------------------------------------------------------------------------------

int  Add_Cat_Trans_Req::Specify_Cat_Trans(bool Cat_Trans_Reactant_Random_Flag, int& Cat_Trans_Reactant_Index,
                                          bool Cat_Trans_Product_Random_Flag,  int& Cat_Trans_Product_Index,
                                          bool Cat_Trans_Catalyst_Random_Flag, int& Cat_Trans_Catalyst_Index,
                                          int&     Intermediate_Product_Index,   //  Compound Interaction Intermediate 
					                      double&  Construction_Actv_Cplx_Energy,
					                      double&  Fission_Actv_Cplx_Energy)


//   Generate characteristics for a Catalytic_Transformation 
//   Use a unique Contsruction and Fission
//   If the Reactant_Random_Flag is not set, then Cat_Trans_Reactant_Index specifies the required Reactant
//   A similar principle applies for Product_Random_Flag and Catalyst_Random_Flag

{
//     int Num_Cat_Trans_Attempts = 0;
//     while(true)
//     BEGIN
//         bool No_Fission_For_Construction_Flag = false;
// //         if (not Construction::Specify_Unique_Construction(Cat_Trans_Reactant_Random_Flag,        
// //                                                         Cat_Trans_Reactant_Index,   
// //                                                             // Reactant1 for the Construction
// //                                                         true,   
// //                                                             //  Product 1  for the Construction is an 
// //                                                             //  intermediate for the Cat Trans
// //                                                             //  and is allowed to be chosen randomly
// //                                                         Intermediate_Product_Index,
// //                                                         Cat_Trans_Catalyst_Random_Flag,
// //                                                         Cat_Trans_Catalyst_Index,    
// //                                                             // Reactant2 for the Construction
// // 						                              true, Construction_Actv_Cplx_Energy))
// //         THEN
// //             return -1;
// //         ENDIF  
// //................................................................................................................
// 
// //      We have found a Construction. Now look for a Fission to go with it.           
//             
//         bool Prod_And_Energy_OK;
//         int Num_Fission_Attempts = 0;                                  
//         while(No_Fission_For_Construction_Flag == false)
//         BEGIN
//             if (Cat_Trans_Product_Random_Flag == true)
//             THEN
//                 Prod_And_Energy_OK =
// //                Fission::Find_Candidate_Product_And_Energy(Intermediate_Product_Index, // Reactant for Fission   
// //                                                        Cat_Trans_Catalyst_Index,    // Product 1 for Fission
// //                                                        Fission_Actv_Cplx_Energy,
// //                                                        Cat_Trans_Product_Index);
//             
//             ELSE
//                 Assert (true == false, "Add_Cat_Trans_Req::Specify_Cat_Trans - Non random Product for Fission not currently supported");
//             ENDIF 
// 
//             if ((Cat_Trans_Product_Index not_eq Cat_Trans_Reactant_Index) and Prod_And_Energy_OK) 
//             THEN
//                 return 0;
//             ENDIF
//             if (Num_Fission_Attempts++ > 1000)
//             THEN            
//                 No_Fission_For_Construction_Flag = true;
// //                return -2;
//             ENDIF
//         END     
// 
// //      We didn't find a Fission
// 
//         if (Num_Cat_Trans_Attempts++ > 1000)
//         THEN
//             return -2;
//         ENDIF
//     END
    return 0;
}
//================================================================================================================
Add_Constr_Req::Add_Constr_Req(Add_Constr_ReqRqstr*  Rqstr_Pt):

//  Initialise the Add Construction Request

    Action_Request(Rqstr_Pt->Get_Action_Time(), 
                 Rqstr_Pt->Get_Repeat_Interval()),  
    Num_To_Add_ (Rqstr_Pt->Get_Num_To_Add())

{
    TRACE("Add_Constr_Req::Add_Constr_Req(Add_Constr_ReqRqstr*  Rqstr_Ptr)");
    Assert (Rqstr_Pt->Get_Action_Time() >= 0,
            "Add_Construction_Req::Add_Construction_Req - Invalid Action Time");
    Assert (Rqstr_Pt->Get_Num_To_Add() >= 0, "Add_Construction_Req::Add_Construction_Req - Invalid Number To Add");
    Assert (Rqstr_Pt->Get_Repeat_Interval() >= 0, 
            "Add_Construction_Req::Add_Construction_Req - Invalid Repeat_Interval");

    const MolType_Spec& R1_Spec = *(Rqstr_Pt->Get_R1_Spec_Pt());
    CREATE_OBJECT(MolType_Spec, R1_Spec_Pt_ = new MolType_Spec(R1_Spec));

    const MolType_Spec& R2_Spec = *(Rqstr_Pt->Get_R2_Spec_Pt());
    CREATE_OBJECT(MolType_Spec, R2_Spec_Pt_ = new MolType_Spec(R2_Spec));

    const MolType_Spec& P_Spec = *(Rqstr_Pt->Get_P_Spec_Pt());
    CREATE_OBJECT(MolType_Spec, P_Spec_Pt_ = new MolType_Spec(P_Spec));

    const ACE_Spec& ACE_Spc = *(Rqstr_Pt->Get_ACE_Spec_Pt());
    CREATE_OBJECT(ACE_Spec, ACE_Spec_Pt_ = new ACE_Spec(ACE_Spc));
}
//----------------------------------------------------------------------------------------------------------------

Add_Constr_Req::~Add_Constr_Req()

{
    TRACE("Add_Constr_Req::~Add_Constr_Req");
    DELETE_OBJECT(MolType_Spec,  R1_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec,  R2_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec,  P_Spec_Pt_);
    DELETE_OBJECT(ACE_Spec,  ACE_Spec_Pt_);
}
//-----------------------------------------------------------------------------------------------------------------

void   Add_Constr_Req::Execute()

//  Execute the Add Construction Action Request

{
    TRACE("Add_Constr_Req::Execute");

    bool R1_Rnd_Fl = R1_Spec_Pt_->Get_Rnd_Fl();
    int  R1_Id = R1_Spec_Pt_->Get_MolType_Id();
    bool R2_Rnd_Fl = R2_Spec_Pt_->Get_Rnd_Fl();
    int  R2_Id = R2_Spec_Pt_->Get_MolType_Id();
    bool P_Rnd_Fl =  P_Spec_Pt_->Get_Rnd_Fl();
    int  P_Id = P_Spec_Pt_->Get_MolType_Id();

    bool ACE_Rnd_Fl = ACE_Spec_Pt_->Get_Rnd_Fl();
    double Actvtd_Cplex_Enrgy = ACE_Spec_Pt_->Get_Energy();

    string*  Fail_Tx_Pt = 0;
    for (int ILoop = 0; ILoop < Num_To_Add_; ++ILoop)
    BEGIN
        if (not Construction::Specify_Unique_Construction
                                                (R1_Rnd_Fl, R1_Id,
                                                 P_Rnd_Fl,  P_Id,
                                                 R2_Rnd_Fl, R2_Id,
                                                 ACE_Rnd_Fl, Actvtd_Cplex_Enrgy,
                                                 Fail_Tx_Pt))
        THEN
            Exec_Fail(Fail_Tx_Pt);
            DELETE_OBJECT(string, Fail_Tx_Pt);
            return;
        ELSE
            CREATE_OBJECT(Construction, new Construction(R1_Id, R2_Id, P_Id, Actvtd_Cplex_Enrgy));
        ENDIF
    END
    Exec_Success(0);
}
//----------------------------------------------------------------------------------------------------------------

string  Add_Constr_Req::Fail_Msg()  const

//  Setup Action fail message

{
    TRACE("Add_Constr_Req::Fail_Msg");
    return   Get_Warn_Tx()
           + string(" Cannot add ")
           + To_String(Num_To_Add_)
           + string(" Construction(s).")
           + Fail_Reason_Tx_
           + string(" Reactant 1 = ")
           + R1_Spec_Pt_->Get_Spec_As_Text()
           + string(", Reactant 2 = ")
           + R2_Spec_Pt_->Get_Spec_As_Text()
           + string(", Product = ")
           + P_Spec_Pt_->Get_Spec_As_Text()
           + string(", Activated Complex Energy = ")
           + ACE_Spec_Pt_->Get_Spec_As_Tx();
}
//-----------------------------------------------------------------------------------------------------------------       

string  Add_Constr_Req::Succ_Msg()  const

{
    TRACE("Add_Constr_Req::Succ_Msg");
    return  To_String(Num_To_Add_) 
          + string(" Construction(s) Added. Reactant 1 = ") 
          + R1_Spec_Pt_->Get_Spec_As_Text()
          + string(", Reactant 2 = ")
          + R2_Spec_Pt_->Get_Spec_As_Text()
          + string(", Product = ")
          + P_Spec_Pt_->Get_Spec_As_Text()
          + string(", Activated Complex Energy = ")
          + ACE_Spec_Pt_->Get_Spec_As_Tx();
}
//-----------------------------------------------------------------------------------------------------------------

int  Add_Constr_Req::Get_Number()  const

//  Return the number of Constructions to be added

{
    return Num_To_Add_;
}
//-----------------------------------------------------------------------------------------------------------------

void  Add_Constr_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Add Construction action request

{
     using namespace  Attribute;
     Param_List.push_back(AttrNames::Cd_To_Name(Num_To_Add_Cd) 
                          + Get_Equals_Tx()
                          + To_String(Num_To_Add_));

     Param_List.push_back(AttrNames::Cd_To_Name(Reactant1_Cd) 
                          + Get_Equals_Tx()
                          + R1_Spec_Pt_->Get_Spec_As_Text());

     Param_List.push_back(AttrNames::Cd_To_Name(Reactant2_Cd) 
                          + Get_Equals_Tx()
                          + R2_Spec_Pt_->Get_Spec_As_Text());

     Param_List.push_back(AttrNames::Cd_To_Name(Product_Cd) 
                          + Get_Equals_Tx()
                          + P_Spec_Pt_->Get_Spec_As_Text());

     Param_List.push_back(AttrNames::Cd_To_Name(ACE_Cd) 
                          + Get_Equals_Tx()
                          + ACE_Spec_Pt_->Get_Spec_As_Tx());
}
//-----------------------------------------------------------------------------------------------------------------

string Add_Constr_Req::Get_Subject() const 

//  Return the subject for the Action Request

{
    return string("Construction");
}

//-----------------------------------------------------------------------------------------------------------------
string Add_Constr_Req::Get_Type()  const

//  Return the Type of the Action Request

{
    return string("Add");   
}
//=================================================================================================================


string  Add_Fission_Req::Fail_Msg()  const

//  Setup Action fail message

{
    TRACE("Add_Fission_Req::Fail_Msg");
    
    return  Get_Warn_Tx()
          + string(" Cannot add ")
          + To_String(Num_To_Add_)
          + string(" Fissions(s),")
          + Fail_Reason_Tx_
          + string(" Reactant = ")
          + R_Spec_Pt_->Get_Spec_As_Text()
          + string(", Product 1 = ")
          + P1_Spec_Pt_->Get_Spec_As_Text()
          + string(", Product 2 = ")
          + P2_Spec_Pt_->Get_Spec_As_Text()
          + string(", Activated Complex Energy = ")
          + ACE_Spec_Pt_->Get_Spec_As_Tx();
}
//----------------------------------------------------------------------------------------------------------------

void  Add_Fission_Req::Execute()

//  Execute the Add Fission Action Request

{
    TRACE("Add_Fission_Req::Execute");

    bool R_Rnd_Fl  = R_Spec_Pt_->Get_Rnd_Fl();
    int  R_Id      = R_Spec_Pt_->Get_MolType_Id();
    bool P1_Rnd_Fl = P1_Spec_Pt_->Get_Rnd_Fl();
    int  P1_Id     = P1_Spec_Pt_->Get_MolType_Id();
    bool P2_Rnd_Fl = P2_Spec_Pt_->Get_Rnd_Fl();
    int  P2_Id     = P2_Spec_Pt_->Get_MolType_Id();

    bool ACE_Rnd_Fl = ACE_Spec_Pt_->Get_Rnd_Fl();
    double Actvtd_Cplex_Enrgy = ACE_Spec_Pt_->Get_Energy();

    string*  Fail_Tx_Pt = 0;
    for (int ILoop = 0; ILoop < Num_To_Add_; ++ILoop)
    BEGIN
        if (not Fission::Specify_Unique_Fission
                                           (R_Rnd_Fl,  R_Id,
                                            P1_Rnd_Fl, P1_Id,
                                            P2_Rnd_Fl, P2_Id,
                                            ACE_Rnd_Fl, Actvtd_Cplex_Enrgy,
                                            Fail_Tx_Pt))
        THEN
            Exec_Fail(Fail_Tx_Pt);
            DELETE_OBJECT(string, Fail_Tx_Pt);
            return;
        ELSE
            CREATE_OBJECT(Fission, new Fission(R_Id, P1_Id, P2_Id, Actvtd_Cplex_Enrgy));
        ENDIF
    END
    Exec_Success(0);
}
//----------------------------------------------------------------------------------------------------------------

string  Add_Fission_Req::Succ_Msg()  const

//  Generate an execution success message for the request 

{
    TRACE("Add_Fission_Req::Succ_Msg");
    return   To_String(Num_To_Add_) 
           + string(" Fission(s) Added. Reactant = ")
           + R_Spec_Pt_->Get_Spec_As_Text()
           + string(", Product 1 = ")
           + P1_Spec_Pt_->Get_Spec_As_Text()
           + string(", Product 2 = ")
           + P2_Spec_Pt_->Get_Spec_As_Text()
           + string(", Activated Complex Energy = ")
           + ACE_Spec_Pt_->Get_Spec_As_Tx();
}
//================================================================================================================
Add_Molecule_Req::Add_Molecule_Req(const  Add_Molecule_ReqRqstr*  ReqRqstr_Pt)

: Action_Request(ReqRqstr_Pt->Get_Action_Time(), 
                 ReqRqstr_Pt->Get_Repeat_Interval()), 
  Num_To_Add_ (ReqRqstr_Pt->Get_Num_To_Add())
{
    TRACE("Add_Molecule_Req::Add_Molecule_Req(Add_Molecule_ReqRqstr*  ReqRqstr_Ptr)");

    const MolType_Spec& MT_Spec = *(ReqRqstr_Pt->Get_MolType_Spec_Pt());
    CREATE_OBJECT(MolType_Spec, MolType_Spec_Pt_ = new MolType_Spec(MT_Spec));
}
//----------------------------------------------------------------------------------------------------------------

Add_Molecule_Req::~Add_Molecule_Req()

{
    TRACE("Add_Molecule_Req::~Add_Molecule_Req");
    DELETE_OBJECT(MolType_Spec, MolType_Spec_Pt_);
}
//-----------------------------------------------------------------------------------------------------------------

void Add_Molecule_Req::Execute() 

//  Execute the Add Molecule Action Request

{    
    TRACE("Add_Molecule_Req::Execute");

    int Num_MolTypes = Chemistry::Get_Num_MolTypes();
    Molecule_Type* MolType_Ptr;

    if (MolType_Spec_Pt_->Get_Rnd_Fl())  // Random MolType Id
    THEN  
//      Add the required number of Molecules. The type of each Molecule is selected at random
        for (int ILoop = 0; ILoop < Num_To_Add_; ILoop++)
        BEGIN
            int MolType_Index = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Num_MolTypes);   
            MolType_Ptr =  Chemistry::Get_MolType_Pt(MolType_Index); 
            Reactor::Add_Mol_Of_Type(MolType_Ptr, 0.01); // 1% Tracker Prob
        END
        Exec_Success(0);
    ELSE        

//      Molecules of a specific type are to be added. Validate first
        int MolType_Index = MolType_Spec_Pt_->Get_MolType_Id();
        if (MolType_Index >= Num_MolTypes)
        THEN
            Exec_Fail("Molecule Type does not exist");
        ELSE            
//          Add the Molecules
            MolType_Ptr = Chemistry::Get_MolType_Pt(MolType_Index);
            for (int ILoop = 0; ILoop < Num_To_Add_; ILoop++)
            BEGIN
                Reactor::Add_Mol_Of_Type(MolType_Ptr, 0.01);
            END
            Exec_Success(0);
        ENDIF
    END
}
//----------------------------------------------------------------------------------------------------------------
string  Add_Molecule_Req::Fail_Msg() const

//  Setup failure message

{
    TRACE("Add_Molecule_Req::Fail_Msg");

    string  Msg  =  Get_Warn_Tx()
                  + string(" Cannot add ") 
                  + To_String(Num_To_Add_)
                  + string(" Molecules of type ") 
                  + MolType_Spec_Pt_->Get_Spec_As_Text()
                  + string(". ")
                  + Fail_Reason_Tx_;
    return Msg;
}
//-----------------------------------------------------------------------------------------------------------------
  
int  Add_Molecule_Req::Get_Number() const

//  Return the number of Molecules to be added

{
    TRACE("Add_Molecule_Req::Get_Number");
    return Num_To_Add_;
}
//-----------------------------------------------------------------------------------------------------------------
void  Add_Molecule_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Add Molecule action request

{
    TRACE("Add_Molecule_Req::Get_Params");

    using namespace Attribute;

    Param_List.push_back(AttrNames::Cd_To_Name(Num_To_Add_Cd)
                         + Get_Equals_Tx()
                         + To_String(Num_To_Add_));

    Param_List.push_back(AttrNames::Cd_To_Name(MolType_Cd)
                         + Get_Equals_Tx()
                         + MolType_Spec_Pt_->Get_Spec_As_Text()); 
}
//-----------------------------------------------------------------------------------------------------------------
string Add_Molecule_Req::Get_Subject() const

//  Return the subject for the Action Request

{
    TRACE("Add_Molecule_Req::Get_Subject");
    return string("Molecule");
}
//-----------------------------------------------------------------------------------------------------------------        
string  Add_Molecule_Req::Get_Type() const

//  Return the Type of the Action Request

{
    TRACE("Add_Molecule_Req::Get_Type");
    return string("Add");   
}
//----------------------------------------------------------------------------------------------------------------

string  Add_Molecule_Req::Succ_Msg() const

//  Generate a success message for the Action Request

{
    TRACE("Add_Molecule_Req::Succ_Msg");


    string Msg_Tx  =  To_String(Num_To_Add_)
                    + string(" Molecules of type ")
                    + MolType_Spec_Pt_->Get_Spec_As_Text()
                    + string(" added.");
//                    + Succ_Qual_Tx_ ;
    if (Repeat_Interval_) Msg_Tx +=  Repeat_Req_Tx;

    return Msg_Tx;
}

//=================================================================================================================

Add_MolType_Req::Add_MolType_Req(const Add_MolType_ReqRqstr*  ReqRqstr_Pt): 

//  Initialise a Add_MolType_Req based on the specified Req Requestor

    Action_Request (ReqRqstr_Pt->Get_Action_Time(), 
                    ReqRqstr_Pt->Get_Repeat_Interval()), 
    Num_To_Add_ (ReqRqstr_Pt->Get_Num_To_Add())    
{
    TRACE("Add_MolType_Req::Add_MolType_Req(Add_MolType_ReqRqstr*  ReqRqstr_Ptr)");

    const Mass_Spec& M_Spec = *(ReqRqstr_Pt->Get_Mass_Spec_Pt());
    CREATE_OBJECT(Mass_Spec, Mass_Spec_Pt_ = new Mass_Spec(M_Spec));

    const PotEnergy_Spec& PE_Spec = *(ReqRqstr_Pt->Get_PotEnergy_Spec_Pt());
    CREATE_OBJECT(PotEnergy_Spec, PotEnergy_Spec_Pt_ = new PotEnergy_Spec(PE_Spec));
}
//-----------------------------------------------------------------------------------------------------------------

Add_MolType_Req::~Add_MolType_Req()

{
    TRACE("Add_MolType_Req::~Add_MolType_Req");
    DELETE_OBJECT(Mass_Spec, Mass_Spec_Pt_);
    DELETE_OBJECT(PotEnergy_Spec, PotEnergy_Spec_Pt_);
}
//----------------------------------------------------------------------------------------------------------------

void Add_MolType_Req::Execute()

//  Execute the Add Molecule Type Action Request
//  Call different Molecule Type constructors depending on requirement

{    
    TRACE("Add_MolType_Req::Execute(");
    Molecule_Type* MolType_Pt;
  
//  Add the required number of Molecule Types.
    for (int ILoop = 0; ILoop < Num_To_Add_; ILoop++)
    BEGIN
        if(Mass_Spec_Pt_->Get_Rand_Flag())
        THEN
            if (PotEnergy_Spec_Pt_->Get_Rand_Flag())
            THEN
                CREATE_OBJECT(Molecule_Type, MolType_Pt = new Molecule_Type);
            ELSE
                Molecule_Type::Potential_Energy_Tp  Pot_Energy(PotEnergy_Spec_Pt_->Get_Pot_Energy());
                CREATE_OBJECT(Molecule_Type, MolType_Pt = new Molecule_Type(Pot_Energy));
            ENDIF
        ELSE
            Molecule_Type::Mass_Tp  MT_Mass(Mass_Spec_Pt_->Get_Mass());
            if (PotEnergy_Spec_Pt_->Get_Rand_Flag())
            THEN
                CREATE_OBJECT(Molecule_Type, MolType_Pt  = new Molecule_Type(MT_Mass));
            ELSE
                Molecule_Type::Potential_Energy_Tp  Pot_Energy(PotEnergy_Spec_Pt_->Get_Pot_Energy());
                CREATE_OBJECT(Molecule_Type, MolType_Pt = new Molecule_Type(MT_Mass, Pot_Energy));                                                                                
            ENDIF                                                                       
        ENDIF                                                                             
        MolType_Pt->Add_To_Chemistry();
    END
    Exec_Success(0); 
}
//-----------------------------------------------------------------------------------------------------------------
  
int  Add_MolType_Req::Get_Number() const

//  Return the number of Molecule Types to be added

{
    return Num_To_Add_;
}
//-----------------------------------------------------------------------------------------------------------------
void  Add_MolType_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Add Molecule Type action request

{
    TRACE("Add_MolType_Req::Get_Params");

    using namespace Attribute;
    
    Param_List.push_back(AttrNames::Cd_To_Name(Num_To_Add_Cd)
                         + Get_Equals_Tx()
                         + To_String(Num_To_Add_));
    Param_List.push_back(AttrNames::Cd_To_Name(Mass_Cd) 
                         + Get_Equals_Tx()
                         + Mass_Spec_Pt_->Get_Spec_As_Text()); 
    Param_List.push_back(AttrNames::Cd_To_Name(Pot_Energy_Cd) 
                         + Get_Equals_Tx()
                         + PotEnergy_Spec_Pt_->Get_Spec_As_Text()); 
}

//-----------------------------------------------------------------------------------------------------------------
string Add_MolType_Req::Get_Subject()  const

//  Return the subject for the Action Request

{
    return string("Molecule Type");
}
//-----------------------------------------------------------------------------------------------------------------        
string Add_MolType_Req::Get_Type()  const

//  Return the Type of the Action Request

{
    return string("Add");   
}
//----------------------------------------------------------------------------------------------------------------

string Add_MolType_Req::Succ_Msg()  const

//  Success message for Add MolType request execution

{
    TRACE("Add_MolType_Req::Succ_Msg");

    string  Msg_Tx =  To_String(Num_To_Add_)
                    + string(" Molecule Type(s) added. Mass = ")
                    + Mass_Spec_Pt_->Get_Spec_As_Text()
                    + string(", Potential Energy = ") 
                    + PotEnergy_Spec_Pt_->Get_Spec_As_Text()
                    + string(". ");
//                    + Succ_Qual_Tx_ ;
            if (Repeat_Interval_) Msg_Tx +=  Repeat_Req_Tx;
    return Msg_Tx;
}
//===============================================================================================================

Remove_Construction_Req::Remove_Construction_Req(Remove_Construction_ReqRqstr*  ReqRqstr_Ptr):


    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                    ReqRqstr_Ptr->Get_Repeat_Interval())

{
    Time_ = ReqRqstr_Ptr->Get_Action_Time()  ;
    Num_To_Remove = ReqRqstr_Ptr->Num_To_Remove;
    Repeat_Interval_ = ReqRqstr_Ptr->Get_Repeat_Interval();
    
    Reactant1_Spec = ReqRqstr_Ptr->Reactant1_Spec;
    Reactant2_Spec = ReqRqstr_Ptr->Reactant2_Spec;
    Product_Spec = ReqRqstr_Ptr->Product_Spec;
}
//-----------------------------------------------------------------------------------------------------------------

string  Remove_Construction_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

{
    return string("");   //  ****  NEEDS FINALIISATION
}
//-----------------------------------------------------------------------------------------------------------------
int  Remove_Construction_Req::Get_Number()

//  Return the number of Constructions to remove 

{
    return Num_To_Remove;
}

//-----------------------------------------------------------------------------------------------------------------      
string Remove_Construction_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Construction");
}
//-----------------------------------------------------------------------------------------------------------------

string Remove_Construction_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Remove");   
}
//-----------------------------------------------------------------------------------------------------------------       

void  Remove_Construction_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Remove Construction Action Request

{
    Param_List.push_back(string("Reactant 1 = ") + Reactant1_Spec);
    Param_List.push_back(string("Reactant 2 = ") + Reactant2_Spec);   
    Param_List.push_back(string("Product = ") + Product_Spec);    
}
//================================================================================================================

Add_Fission_Req::Add_Fission_Req(Add_Fission_ReqRqstr*  Rqstr_Pt):

//  Initialise the Add Fission Request

    Action_Request(Rqstr_Pt->Get_Action_Time(), 
                   Rqstr_Pt->Get_Repeat_Interval()),  
    Num_To_Add_ (Rqstr_Pt->Get_Num_To_Add())

{
    TRACE("Add_Fission_Req::Add_Fission_Req(Add_Fission_ReqRqstr*  Rqstr_Ptr)");
    Assert (Rqstr_Pt->Get_Action_Time() >= 0,
            "Add_Fission_Req::Add_Fission_Req- Invalid Action Time");
    Assert (Rqstr_Pt->Get_Num_To_Add() >= 0, "Add_Fission_Req::Add_Fission_Req- Invalid Number To Add");
    Assert (Rqstr_Pt->Get_Repeat_Interval() >= 0, 
            "Add_Fission_Req::Add_Fission_Req- Invalid Repeat_Interval");

    const MolType_Spec& R_Spec = *(Rqstr_Pt->Get_R_Spec_Pt());
    CREATE_OBJECT(MolType_Spec, R_Spec_Pt_ = new MolType_Spec(R_Spec));

    const MolType_Spec& P1_Spec = *(Rqstr_Pt->Get_P1_Spec_Pt());
    CREATE_OBJECT(MolType_Spec, P1_Spec_Pt_ = new MolType_Spec(P1_Spec));

    const MolType_Spec& P2_Spec = *(Rqstr_Pt->Get_P2_Spec_Pt());
    CREATE_OBJECT(MolType_Spec, P2_Spec_Pt_ = new MolType_Spec(P2_Spec));

    const ACE_Spec& ACE_Spc = *(Rqstr_Pt->Get_ACE_Spec_Pt());
    CREATE_OBJECT(ACE_Spec, ACE_Spec_Pt_ = new ACE_Spec(ACE_Spc));
}
//----------------------------------------------------------------------------------------------------------------

Add_Fission_Req::~Add_Fission_Req()

{
    TRACE("Add_Fission_Req::~Add_Fission_Req");
    DELETE_OBJECT(MolType_Spec, R_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec, P1_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec, P2_Spec_Pt_);
    DELETE_OBJECT(ACE_Spec, ACE_Spec_Pt_);
}
//-----------------------------------------------------------------------------------------------------------------

int  Add_Fission_Req::Get_Number()  const

//  Return the number of Fissions to add 

{
    return Num_To_Add_;
}
//-----------------------------------------------------------------------------------------------------------------
        
void Add_Fission_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Add Fission Action Request

{
     using namespace  Attribute;
     Param_List.push_back(AttrNames::Cd_To_Name(Num_To_Add_Cd) 
                          + Get_Equals_Tx()
                          + To_String(Num_To_Add_));

     Param_List.push_back(AttrNames::Cd_To_Name(Reactant_Cd) 
                          + Get_Equals_Tx()
                          + R_Spec_Pt_->Get_Spec_As_Text());

     Param_List.push_back(AttrNames::Cd_To_Name(Product1_Cd) 
                          + Get_Equals_Tx()
                          + P1_Spec_Pt_->Get_Spec_As_Text());

     Param_List.push_back(AttrNames::Cd_To_Name(Product2_Cd) 
                          + Get_Equals_Tx()
                          + P2_Spec_Pt_->Get_Spec_As_Text());

     Param_List.push_back(AttrNames::Cd_To_Name(ACE_Cd) 
                          + Get_Equals_Tx()
                          + ACE_Spec_Pt_->Get_Spec_As_Tx());
}
//-----------------------------------------------------------------------------------------------------------------

string Add_Fission_Req::Get_Subject()   const

//  Return the subject for the Action Request

{
    return string("Fission");
}
//-----------------------------------------------------------------------------------------------------------------
string Add_Fission_Req::Get_Type()  const

//  Return the Type of the Action Request

{
    return string("Add");   
}


//=================================================================================================================
        
//Add_Maint_MolType_Spec_Req::Add_Maint_MolType_Spec_Req(Add_Maint_MolType_Spec_ReqRqstr*  ReqRqstr_Ptr)

//  Constructor

//{        
//    int MolType_Index = Random_Sequence::Get_Ptr(Random_Sequence::One)->
//                         Uniform(Chemistry::Get_Chemistry__Ptr()->Get_Number_Of_Molecule_Types());
//    double  Tracker_Probability = 0.0;
//    Add_Maint_MolType_Spec_Req(ReqRqstr_Ptr->Action_Time, 
//                               ReqRqstr_Ptr->Num_To_Add, 
//                               ReqRqstr_Ptr->Num_Mols_Fixed,
//                               ReqRqstr_Ptr->Tracker_Probability, 
//                               ReqRqstr_Ptr->Molecule_Type_Index, 
//                               ReqRqstr_Ptr->Repeat_Interval, 
//                               ReqRqstr_Ptr->Repeat_Flag);
//}
//---------------------------------------------------------------------------------------------------------------

Add_Maint_MolType_Spec_Req::Add_Maint_MolType_Spec_Req(Add_Maint_MolType_Spec_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor 

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())


{
    Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Repeat_Interval_ = ReqRqstr_Ptr->Get_Repeat_Interval();
//    Repeat_Req = ReqRqstr_Ptr->Repeat_Flag;
    Num_To_Add = ReqRqstr_Ptr->Num_To_Add;
    Num_Mols_Fixed = ReqRqstr_Ptr->Num_Mols_Fixed;
    Tracker_Probability = ReqRqstr_Ptr->Tracker_Probability;
    Molecule_Type_Index = ReqRqstr_Ptr->Molecule_Type_Index;
}
//-----------------------------------------------------------------------------------------------------------------

string Add_Maint_MolType_Spec_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Add the required number of Maintained Molecule Type Specifications. 

{
    bool  Success = true;
//    int MolType_Index;
    string  MolType_List;
    for (int ILoop = 0; ILoop < Num_To_Add; ILoop++)
    BEGIN
        if (not Reactor::Get_Reactor__Ptr()->Add_Maintained_MolType_Spec(Molecule_Type_Index, 
                                                                         Num_Mols_Fixed,
                                                                         Tracker_Probability))
        THEN
            Success = false;
            break;
        ELSE
            if (!MolType_List.empty()) MolType_List += string(", ");
            MolType_List += To_String(Molecule_Type_Index);
        ENDIF
    END
    
    string Action_Result;  
    if (Success)
    THEN     
        if(not Repeat_Interval_)
        THEN   // No repeats for this Action Request
            Action_Result = To_String(Num_To_Add) + string(" Maintained Molecule Type Specifications added. Molecule Type(s): ")
                            + MolType_List + string(".");
        ELSEIF (not Repeat_Req_)  // First in a repeat sequence
        THEN
            Action_Result = To_String(Num_To_Add) + string(" Maintained Molecule Type Specifications added. Molecule Type(s): ")
                            + MolType_List + string(".") 
                            + string(" First in sequence of repeat Action Requests. Successful repeats will not be reported.");
        ELSE
            Action_Result = string("No_Rep");  // Its a repeat so don't report
        ENDIF
    
//      If repeat required, then create another Action Request and pass back pointer. Mark the Action Request
//      as an automatically generated repeat (last param)
    
        if(Repeat_Interval_)
        THEN
//			Add_Maint_MolType_Spec_ReqRqstr  Rqstr;
//			Rqstr.Get_Action_Time = Time + Repeat_Interval;
//			Rqstr.Num_To_Add = Num_To_Add;
//			Rqstr.Num_Mols_Fixed = Num_Mols_Fixed;
//			Rqstr.Tracker_Probability = Tracker_Probability;
//			Rqstr.Molecule_Type_Index = Molecule_Type_Index;
//			Rqstr.Repeat_Interval = Repeat_Interval;
//			Rqstr.Repeat_Flag = true;

//            CREATE_OBJECT(Add_Maint_MolType_Spec_Req, Repeat_Act_Req_Ptr = new Add_Maint_MolType_Spec_Req(&Rqstr));
        ELSE
            Repeat_Act_Req_Ptr = 0;
        ENDIF
    ELSE
        Action_Result = Get_Warn_Tx()
                        + string(" Cannot add ") 
                        + To_String(Num_To_Add) 
                        + string(" Maintained Molecule Type Specifications");
    ENDIF
 
    return Action_Result; 
}
//-----------------------------------------------------------------------------------------------------------------

int  Add_Maint_MolType_Spec_Req::Get_Number() const

//  Return the number of Maintained Molecule Type Specifications to be added

{
    return Num_To_Add;
}
//----------------------------------------------------------------------------------------------------------------

void  Add_Maint_MolType_Spec_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Add Maintained Molecule Type Spec. action request

{
    Param_List.push_back(string("Mol. Type = ") + To_String(Molecule_Type_Index)); // No Parameters 
}
//-----------------------------------------------------------------------------------------------------------------

string Add_Maint_MolType_Spec_Req::Get_Subject() const

//  Return the subject for the Action Request

{
    return string("Maint. Mol. Type Spec.");
}
//-----------------------------------------------------------------------------------------------------------------

string Add_Maint_MolType_Spec_Req::Get_Type() const

//  Return the Type of the Action Request

{
    return string("Add");   
}

//=================================================================================================================
     
Remove_Fission_Req::Remove_Fission_Req(Remove_Fission_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())


{
	Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Num_To_Remove = ReqRqstr_Ptr->Num_To_Remove;
    Repeat_Interval_ = ReqRqstr_Ptr->Get_Repeat_Interval();
    
    Reactant_Spec = ReqRqstr_Ptr->Reactant_Spec;
    Product1_Spec = ReqRqstr_Ptr->Product1_Spec;
    Product2_Spec = ReqRqstr_Ptr->Product2_Spec;
}
//-----------------------------------------------------------------------------------------------------------------

string  Remove_Fission_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

{
    return string("");   // **** NEEDS FINALISATION
}
//-----------------------------------------------------------------------------------------------------------------
int  Remove_Fission_Req::Get_Number()

//  Return the number of Fissions to remove 

{
    return Num_To_Remove;
}
//----------------------------------------------------------------------------------------------------------------   
void  Remove_Fission_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Remove Fission Action Request

{
    Param_List.push_back(string("Reactant = ") + Reactant_Spec);
    Param_List.push_back(string("Product 1 = ") + Product1_Spec);   
    Param_List.push_back(string("Product 2 = ") + Product2_Spec);    
}
//-----------------------------------------------------------------------------------------------------------------

string Remove_Fission_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Fission");
}

//-----------------------------------------------------------------------------------------------------------------
string Remove_Fission_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Remove");   
}
//=================================================================================================================

Add_Transformation_Req::Add_Transformation_Req(Add_Transfm_ReqRqstr*  Rqstr_Pt):

//  Initialise the Add Transformation Request

    Action_Request(Rqstr_Pt->Get_Action_Time(), 
                   Rqstr_Pt->Get_Repeat_Interval()),  
    Num_To_Add_ (Rqstr_Pt->Get_Num_To_Add())

{
    TRACE("Add_Transformation_Req::Add_Transformation_Req(Add_Transfm_ReqRqstr*  ReqRqstr_Ptr)");
    Assert (Rqstr_Pt->Get_Action_Time() >= 0,
            "Add_Transformation_Req::Add_Transformation_Req - Invalid Action Time");
    Assert (Rqstr_Pt->Get_Num_To_Add() >= 0, "Add_Transformation_Req::Add_Transformation_Req- Invalid Number To Add");
    Assert (Rqstr_Pt->Get_Repeat_Interval() >= 0, 
            "Add_Transformation_Req::Add_Transformation_Req - Invalid Repeat_Interval");

    const MolType_Spec& R_Spec = *(Rqstr_Pt->Get_R_Spec_Pt());
    CREATE_OBJECT(MolType_Spec, R_Spec_Pt_ = new MolType_Spec(R_Spec));

    const MolType_Spec& P_Spec = *(Rqstr_Pt->Get_P_Spec_Pt());
    CREATE_OBJECT(MolType_Spec, P_Spec_Pt_ = new MolType_Spec(P_Spec));

    const ACE_Spec& ACE_Spc = *(Rqstr_Pt->Get_ACE_Spec_Pt());
    CREATE_OBJECT(ACE_Spec, ACE_Spec_Pt_ = new ACE_Spec(ACE_Spc));
}
//-----------------------------------------------------------------------------------------------------------------

Add_Transformation_Req::~Add_Transformation_Req()

{
    TRACE("Add_Transformation_Req::~Add_Transformation_Req");
    DELETE_OBJECT(MolType_Spec, R_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec, P_Spec_Pt_);
    DELETE_OBJECT(ACE_Spec, ACE_Spec_Pt_);
}
//----------------------------------------------------------------------------------------------------------------

void  Add_Transformation_Req::Execute()

//  Execute the Add Transformation Action Request

{
    TRACE("Add_Transformation_Req::Execute");

    bool R_Rnd_Fl = R_Spec_Pt_->Get_Rnd_Fl();
    int  R_Id = R_Spec_Pt_->Get_MolType_Id();
    bool P_Rnd_Fl =  P_Spec_Pt_->Get_Rnd_Fl();
    int  P_Id = P_Spec_Pt_->Get_MolType_Id();

    bool ACE_Rnd_Fl = ACE_Spec_Pt_->Get_Rnd_Fl();
    double Actvtd_Cplex_Enrgy = ACE_Spec_Pt_->Get_Energy();

    string*  Fail_Tx_Pt = 0;
    for (int ILoop = 0; ILoop < Num_To_Add_; ++ILoop)
    BEGIN
        if (not Transformation::Specify_Unique_Transformation
                                                (R_Rnd_Fl, R_Id,
                                                 P_Rnd_Fl,  P_Id,
                                                 ACE_Rnd_Fl, Actvtd_Cplex_Enrgy,
                                                 Fail_Tx_Pt))
        THEN
            Exec_Fail(Fail_Tx_Pt);
            DELETE_OBJECT(string, Fail_Tx_Pt);
            return;
        ELSE
            CREATE_OBJECT(Transformation, new Transformation(R_Id, P_Id, Actvtd_Cplex_Enrgy));
        ENDIF
    END
    Exec_Success(0);
}
//-----------------------------------------------------------------------------------------------------------------
int  Add_Transformation_Req::Get_Number() const

//  Return the number of Transfoormations to add 

{
    return Num_To_Add_;
}
//-----------------------------------------------------------------------------------------------------------------

string Add_Transformation_Req::Get_Subject() const

//  Return the subject for the Action Request

{
    return string("Transformation");
}

//-----------------------------------------------------------------------------------------------------------------
string Add_Transformation_Req::Get_Type() const

//  Return the Type of the Action Request

{
    return (string("Add"));
}
//-----------------------------------------------------------------------------------------------------------------       

void  Add_Transformation_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Add Transformation Action Request

{
     using namespace  Attribute;
     Param_List.push_back(AttrNames::Cd_To_Name(Num_To_Add_Cd) 
                          + Get_Equals_Tx()
                          + To_String(Num_To_Add_));

     Param_List.push_back(AttrNames::Cd_To_Name(Reactant_Cd) 
                          + Get_Equals_Tx()
                          + R_Spec_Pt_->Get_Spec_As_Text());

     Param_List.push_back(AttrNames::Cd_To_Name(Product_Cd) 
                          + Get_Equals_Tx()
                          + P_Spec_Pt_->Get_Spec_As_Text());

     Param_List.push_back(AttrNames::Cd_To_Name(ACE_Cd) 
                          + Get_Equals_Tx()
                          + ACE_Spec_Pt_->Get_Spec_As_Tx());
}
//----------------------------------------------------------------------------------------------------------------

string  Add_Transformation_Req::Fail_Msg()  const

//  Setup Action fail message

{
    TRACE("Add_Transformation_Req::Fail_Msg");
    return   Get_Warn_Tx()
           + string(" Cannot add ")
           + To_String(Num_To_Add_)
           + string(" Transformation(s).")
           + Fail_Reason_Tx_
           + string(" Reactant = ")
           + R_Spec_Pt_->Get_Spec_As_Text()
           + string(", Product = ")
           + P_Spec_Pt_->Get_Spec_As_Text()
           + string(", Activated Complex Energy = ")
           + ACE_Spec_Pt_->Get_Spec_As_Tx();
}
//----------------------------------------------------------------------------------------------------------------

string  Add_Transformation_Req::Succ_Msg()  const

{
    TRACE("Add_Transformation_Req::Succ_Msg");
    return  To_String(Num_To_Add_) 
          + string(" Transformation(s) Added. Reactant = ") 
          + R_Spec_Pt_->Get_Spec_As_Text()
          + string(", Product = ")
          + P_Spec_Pt_->Get_Spec_As_Text()
          + string(", Activated Complex Energy = ")
          + ACE_Spec_Pt_->Get_Spec_As_Tx();
}

//===============================================================================================================

ChemEvol_Req::ChemEvol_Req(const ChemEvol_ReqRqstr*  ReqRqstr_Pt): 

//  Initialise a Chemistry Evolution requet based on the specified Req Requestor

    Action_Request (ReqRqstr_Pt->Get_Action_Time(), 
                    ReqRqstr_Pt->Get_Repeat_Interval()),
    Last_Fitness_  (DBL_MIN)   // Last_Fitness is only relevant for a repeat
{
    TRACE("ChemEvol_Req::ChemEvol_Req(ChemEvol_ReqRqstr*  ReqRqstr_Ptr)");
}

//----------------------------------------------------------------------------------------------------------------

ChemEvol_Req::~ChemEvol_Req()

{
    TRACE("ChemEvol_Req::~ChemEvol_Req");
}
//----------------------------------------------------------------------------------------------------------------

void   ChemEvol_Req::Execute()

//  Execute the Chemistry Evolution request

{
    TRACE("ChemEvol_Req::Execute");

    string ActRes_Last("");
    string ActRes_New("");
    string ActRes_Fitness("");

    double Current_Fitness = Reactor::Get_Reactor__Ptr()->Get_Fitness();
    Interaction_Type*  IntType_Pt;

//  If there was a previous Evolution Step, then report and evaluate the outcome

    if (Repeat_Req_)
    THEN
        Assert((
                (Last_Mutation_.Last_IntType_Added_Pt == 0) or (Last_Mutation_.Last_IntType_Disabled_Pt == 0)
               ), 
               "ChemEvol_Req::Execute - Invalid Last Mutation Specifier: appears to specify both addition and removal of an Interaction Type"
              );

        if ((Last_Mutation_.Last_IntType_Added_Pt == 0) 
                and 
            (Last_Mutation_.Last_IntType_Disabled_Pt == 0))
        THEN
            ActRes_Last += string("Last Mutation: None. ");
        ELSE

//          If fitness has reduced, then reverse the last mutation

            if (Current_Fitness < Last_Fitness_)
            THEN
                ActRes_Last += string("Last Mutation: BAD. ");
                if (Last_Mutation_.Last_IntType_Added_Pt not_eq 0)
                THEN
//                  -----------Log Entry -----
                    IntType_Pt = Last_Mutation_.Last_IntType_Added_Pt;
                    ActRes_Last += string("Removing ")
                    + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Pt);
//                  --------------------------
    
                    Last_Mutation_.Last_IntType_Added_Pt->Remove_From_Chemistry();
                ELSEIF (Last_Mutation_.Last_IntType_Disabled_Pt not_eq 0)
                THEN
                    IntType_Pt = Last_Mutation_.Last_IntType_Disabled_Pt;

//                  Check whether the R Int Type is still Realisable for the Interaction Type.
//                  If it is, then it should still exist, and it should be re-enabled. If it
//                  was set Unrealisable then it should have been deleted, and it will be 
//                  re-created if it becomes Realisable again.

                    if (IntType_Pt->Is_Realisable())
                    THEN
//                      -----------Log Entry -----
                        ActRes_Last += string("Re-enabling ")
                        + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Pt);
//                      --------------------------
                        IntType_Pt->Get_R_Int_Type_Ptr()->Enable();
                    ELSE
//                      -----------Log Entry -----
                        ActRes_Last += string("Not re-enabling ")
                        + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Pt)
                        + string (" because it is no longer Realisable");
//                      --------------------------
                    ENDIF
                ENDIF   
    
//          Otherwise - report the outcome, and if the mutation was to disable a 
//          Realisable Interaction Type then permanently remove the Interaction Type
    
            ELSE
                if(Current_Fitness == Last_Fitness_)
                THEN
                    ActRes_Last += string("Last Mutation: NEUTRAL. ");
                ELSE
                    ActRes_Last += string("Last Mutation: GOOD. ");
                ENDIF
    
                if (Last_Mutation_.Last_IntType_Disabled_Pt not_eq 0)
                THEN
                    Interaction_Type*  IntType_Ptr;
                    IntType_Ptr = Last_Mutation_.Last_IntType_Disabled_Pt;
    
//                  -----------Log Entry -----
                    ActRes_Last += string("Permanently removing ")
                    + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr);
//                  --------------------------
    
                    IntType_Ptr->Remove_From_Chemistry();
                ENDIF
            ENDIF
        ENDIF
    ENDIF

//  Now add an Interaction Type or disable a Realisable Interaction Type randomly

    Current_Mutation_.Last_IntType_Added_Pt = 0;
    Current_Mutation_.Last_IntType_Disabled_Pt = 0;

    ActRes_New += string("New mutation: ");      
    if (Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(double(1.0)) < 0.5) 
    THEN

//      Add an Interaction Type
  
        Interaction_Type*  IntType_Ptr;
        if (Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(double(1.0)) < 0.5)
        THEN    
            string*  Fail_Tx_Pt = 0;

            bool R1_Rnd_Fl = true;
            int  R1_Id;
            bool R2_Rnd_Fl = true;
            int  R2_Id;
            bool P_Rnd_Fl = true;
            int  P_Id;

            bool ACE_Rnd_Fl = true;
            double Actvtd_Cplex_Enrgy;

            if (not Construction::Specify_Unique_Construction
                                            (R1_Rnd_Fl, R1_Id,
                                             P_Rnd_Fl,  P_Id,
                                             R2_Rnd_Fl, R2_Id,
                                             ACE_Rnd_Fl, Actvtd_Cplex_Enrgy,
                                             Fail_Tx_Pt))
            THEN
                ActRes_New += string("None. Cannot create Construction: ") + (*Fail_Tx_Pt);
                DELETE_OBJECT(string, Fail_Tx_Pt);
//                return;
            ELSE
                CREATE_OBJECT(Construction, IntType_Ptr = new Construction(R1_Id, R2_Id, P_Id, Actvtd_Cplex_Enrgy));
                ActRes_New += string("Adding ")
                 + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr);
                Current_Mutation_.Last_IntType_Added_Pt = IntType_Ptr;
            ENDIF
        ELSE   // Create a Fission

            string*  Fail_Tx_Pt = 0;

            bool R_Rnd_Fl = true;
            int  R_Id;
            bool P1_Rnd_Fl = true;
            int  P1_Id;
            bool P2_Rnd_Fl = true;
            int  P2_Id;

            bool ACE_Rnd_Fl = true;
            double Actvtd_Cplex_Enrgy;

            if (not Fission::Specify_Unique_Fission
                                            (R_Rnd_Fl,  R_Id,
                                             P1_Rnd_Fl, P1_Id,
                                             P2_Rnd_Fl, P2_Id,
                                             ACE_Rnd_Fl, Actvtd_Cplex_Enrgy,
                                             Fail_Tx_Pt))
            THEN
                ActRes_New += string("None. Cannot create Fission: ") + (*Fail_Tx_Pt);
                DELETE_OBJECT(string, Fail_Tx_Pt);
//                return;
            ELSE
                CREATE_OBJECT(Fission, IntType_Ptr = new Fission(R_Id, P1_Id, P2_Id, Actvtd_Cplex_Enrgy));
                ActRes_New += string("Adding ")
                 + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr);
                Current_Mutation_.Last_IntType_Added_Pt = IntType_Ptr;
            ENDIF
        ENDIF
    ELSE

//      Disable a Realisable Interaction Type at random

        if (Chemistry::Get_Chemistry__Ptr()->Get_Number_Of_Interaction_Types() > 0)
        THEN
            IntType_Pt = Chemistry::Get_Chemistry__Ptr()->Get_Random_IntType_Ptr();
            Assert(IntType_Pt != 0, "ChemEvol_Req::Execute() - Invalid Interaction Type pointer");
            Realisable_Interaction_Type*  R_Int_Type_Ptr;
            R_Int_Type_Ptr = IntType_Pt->Get_R_Int_Type_Ptr();
            if (R_Int_Type_Ptr)
            THEN
//              -----------Log Entry -----
                ActRes_New += string("Disabling ")
                + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Pt);
//              --------------------------
    
                R_Int_Type_Ptr->Disable();
                Current_Mutation_.Last_IntType_Disabled_Pt = IntType_Pt;  // Remember this rather than the R_IntType
                                                                        // which could become unrealisable before 
                                                                        // the next evolution step
            ELSE
                ActRes_New += string("None. Cannot disable Realisable Interaction Type - selected Interaction Type is not realisable.");         
            ENDIF
        ELSE
            ActRes_New += string("None. Cannot disable Realisable Interaction Type - no Interaction Types exist in Chemistry");         
        ENDIF
    ENDIF

//  -----------Log Entry -----
    ActRes_Fitness += string("Current Fitness = ") + To_String(Current_Fitness)
                  +  string("   Last Fitness = ") + To_String(Last_Fitness_) + string(". ");
//  --------------------------

    Last_Mutation_ = Current_Mutation_;
    Last_Fitness_ = Current_Fitness;

    Exec_Success(ActRes_New.c_str());
    cout  << "Reactor Time = " << Reactor::Time() << endl 
          << ActRes_Fitness << endl  
          << ActRes_Last << endl  
          << ActRes_New  <<  endl 
          << endl << endl;

//  If a repeat action is required, then setup the request
/*
    if(Repeat_Interval_)
    THEN
//        CREATE_OBJECT(Evolution_Step_Req, 
//            Repeat_Act_Req_Ptr = new Evolution_Step_Req(Time + Repeat_Interval, 
//                                                        Current_Fitness, // Will be last fitness when step executed
//                                                        Current_Mutation,   
//                                                        Repeat_Interval, 
//                                                        true));  // Repeat
    ELSE
////        Repeat_Act_Req_Ptr = 0;
    ENDIF

    cout << Action_Result  <<  endl;
////    return string("No_Rep");

//    return Action_Result;

*/
}
//----------------------------------------------------------------------------------------------------------------

string  ChemEvol_Req::Fail_Msg()  const

//  Setup Action fail message

{
    TRACE("ChemEvol_Req::Fail_Msg");
    Assert(false, "ChemEvol_Req::Fail_Msg - Should never be called");
    return string ("");
}
//-----------------------------------------------------------------------------------------------------------------       

string  ChemEvol_Req::Succ_Msg()  const

{
    TRACE("ChemEvol_Req::Succ_Msg");
    return  string(" Chemistry Evolution step: ") 
          + Success_Qualifier_Tx_;
}
//-----------------------------------------------------------------------------------------------------------------

void  ChemEvol_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Chemistry Evolution action request
//  None for this Action Request

{
    Param_List.push_back(string(""));
}
//-----------------------------------------------------------------------------------------------------------------

string ChemEvol_Req::Get_Subject() const 

//  Return the subject for the Action Request

{
    return string("Chemistry");
}

//-----------------------------------------------------------------------------------------------------------------
string ChemEvol_Req::Get_Type()  const

//  Return the Type of the Action Request

{
    return string("Evolve");   
}
//=================================================================================================================

Empty_Reactor_Req::Empty_Reactor_Req(Empty_Reactor_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())
{
    Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Repeat_Req_ = false;
    Repeat_Interval_ = 0;
}
//----------------------------------------------------------------------------------------------------------------

string Empty_Reactor_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the action request to remove all the Molecules from the Reactor

{
    Reactor::Get_Reactor__Ptr()->Empty();
    return string("Reactor Emptied - All Molecules removed");
}
//----------------------------------------------------------------------------------------------------------------

int Empty_Reactor_Req::Get_Number()

//  Number associated with the action request. Always 1

{
    return 1;
}
//----------------------------------------------------------------------------------------------------------------

void Empty_Reactor_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Empty Reactor Action Request. There aren't
//  any, so just blank

{
    Param_List.push_back(string(""));
}
//-----------------------------------------------------------------------------------------------------------------

string Empty_Reactor_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Reactor");
}

//-----------------------------------------------------------------------------------------------------------------
string Empty_Reactor_Req::Get_Type()

//  Return the Type of the Action Request

{
    return (string("Empty"));
}
//================================================================================================================

/*
Evolution_Step_Req::Evolution_Step_Req (Evolution_Step_ReqRqstr*  ReqRqstr_Ptr)

//  Initialise an Evolution_Step_Req based on the specified Req Requestor

    Action_Request (ReqRqstr_Pt->Get_Action_Time(), 
                    ReqRqstr_Pt->Get_Repeat_Interval())

{
//    Time = ReqRqstr_Ptr->Action_Time;
//    Repeat_Req = ReqRqstr_Ptr->Repeat_Flag;
//    Repeat_Interval = ReqRqstr_Ptr->Repeat_Interval;
    Last_Fitness = ReqRqstr_Ptr->Last_Fitness;
///////    Last_Mutation = ReqRqstr_Ptr->Last_Mutation_Spec;
} 
//-----------------------------------------------------------------------------------------------------------------

string Evolution_Step_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the action request to execute an Evolution Step

{
    string  Action_Result = string("Evolution Step: ");
    double Current_Fitness = Reactor::Get_Reactor__Ptr()->Get_Fitness();
    Interaction_Type*  IntType_Ptr;

//  If there is a previous Evolution Step, then report and evaluate the outcome

    if (Repeat_Req)
    THEN
        Assert((
                (Last_Mutation.Last_IntType_Add<< ActRes_Fitness << endl  ed_Ptr == 0) or (Last_Mutation.Last_IntType_Disabled_Ptr == 0)
               ), 
               "Evolution_Step_Req::Evolution_Step_Req - Invalid Last Mutation Specifier"
              );

        if ((Last_Mutation.Last_IntType_Added_Ptr == 0) 
                and 
            (Last_Mutation.Last_IntType_Disabled_Ptr == 0))
        THEN
            Action_Result += string("Last Mutation NULL. ");
        ELSE

//          If fitness has reduced, then reverse the last mutation

            if (Current_Fitness < Last_Fitness)
            THEN
                Action_Result += string("Last Mutation BAD. ");
                if (Last_Mutation.Last_IntType_Added_Ptr not_eq 0)
                THEN
//                  -----------Log Entry -----
                    IntType_Ptr = Last_Mutation.Last_IntType_Added_Ptr;
                    Action_Result += string("Removing ")
                    + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr);
//                  --------------------------
    
                    Last_Mutation.Last_IntType_Added_Ptr->Remove_From_Chemistry();
                ELSEIF (Last_Mutation.Last_IntType_Disabled_Ptr not_eq 0)
                THEN
                    IntType_Ptr = Last_Mutation.Last_IntType_Disabled_Ptr;

//                  Check whether the R Int Type is still Realisable for the Interaction Type.
//                  If it is, then it should still exist, and it should be re-enabled. If it
//                  was set Unrealisable then it should have been deleted, and it will be 
//                  re-created if it becomes Realisable again.

                    if (IntType_Ptr->Is_Realisable())
                    THEN
//                      -----------Log Entry -----
                        Action_Result += string("Re-enabling ")
                        + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr);
//                      --------------------------
                        IntType_Ptr->Get_R_Int_Type_Ptr()->Enable();
                    ELSE
//                      -----------Log Entry -----
                        Action_Result += string("Not re-enabling ")
                        + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr)
                        + string (" because it is no longer Realisable");
//                      --------------------------
                    ENDIF
                ENDIF   
    
//          Otherwise - report the outcome, and if the mutation was to disable a 
//          Realisable Interaction Type then permanently remove the Interaction Type
    
            ELSE
                if(Current_Fitness == Last_Fitness)
                THEN
                    Action_Result += string("Last Mutation NEUTRAL. ");
                ELSE
                    Action_Result += string("Last Mutation GOOD. ");
                ENDIF
    
                if (Last_Mutation.Last_IntType_Disabled_Ptr not_eq 0)
                THEN
                    Interaction_Type*  IntType_Ptr;
                    IntType_Ptr = Last_Mutation.Last_IntType_Disabled_Ptr;
    
//                  -----------Log Entry -----
                    Action_Result += string("Permanently removing ")
                    + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr);
//                  --------------------------
    
                    IntType_Ptr->Remove_From_Chemistry();
                ENDIF
            ENDIF
        ENDIF
    ENDIF

//  Now add an Interaction Type or disable a Realisable Interaction Type randomly

    Current_Mutation.Last_IntType_Added_Ptr = 0;
    Current_Mutation.Last_IntType_Disabled_Ptr = 0;

    Action_Result += string(" New mutation: ");      
    if (Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(double(1.0)) < 0.5) 
    THEN

//      Add an Interaction Type
  
        Interaction_Type*  IntType_Ptr;
        try
        BEGIN
            if (Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(double(1.0)) < 0.5)
            THEN
                CREATE_OBJECT(Construction, IntType_Ptr = new Construction);

//              -----------Log Entry -----
                Action_Result += string("Adding ")
                 + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr);
//              --------------------------
            ELSE
                CREATE_OBJECT(Fission, IntType_Ptr = new Fission);

//              -----------Log Entry -----
                Action_Result += string("Adding ")
                 + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr);;
//              --------------------------
            ENDIF
            Current_Mutation.Last_IntType_Added_Ptr = IntType_Ptr;
        END

        catch (Create_Error  Excep)
        BEGIN
            Action_Result = Warning_String + string(" Evolution step failed. ")  
                                        + Excep.Get_Error_Text();
        END
    ELSE

//      Disable a Realisable Interaction Type at random

        IntType_Ptr = Chemistry::Get_Chemistry__Ptr()->Get_Random_IntType_Ptr();
        Realisable_Interaction_Type*  R_Int_Type_Ptr;
        R_Int_Type_Ptr = IntType_Ptr->Get_R_Int_Type_Ptr();
        if (R_Int_Type_Ptr)
        THEN
//          -----------Log Entry -----
            Action_Result += string("Disabling ")
            + Interaction_Type::Support::Get_IntType_Summary_As_String(IntType_Ptr);
//          --------------------------

            R_Int_Type_Ptr->Disable();
            Current_Mutation.Last_IntType_Disabled_Ptr = IntType_Ptr; // Remember this rather than the R_IntType
                                                                      // which could become unrealisable before 
                                                                      // the next evolution step
        ELSE
            Action_Result += string("No mutation this step.");         
        ENDIF
    ENDIF

//  -----------Log Entry -----
    Action_Result += string(" Current/Last Fitness = ") + To_String(Current_Fitness)
                  +  string("/") + To_String(Last_Fitness) + string(". ");
//  --------------------------

//  If a repeat action is required, then setup the request

    if(Repeat_Interval)
    THEN
        CREATE_OBJECT(Evolution_Step_Req, 
            Repeat_Act_Req_Ptr = new Evolution_Step_Req(Time + Repeat_Interval, 
                                                        Current_Fitness, // Will be last fitness when step executed
                                                        Current_Mutation,   
                                                        Repeat_Interval, 
                                                        true));  // Repeat
    ELSE
        Repeat_Act_Req_Ptr = 0;
    ENDIF

    cout << Action_Result  <<  endl;
    return string("No_Rep");

//    return Action_Result;
}
//----------------------------------------------------------------------------------------------------------------

int Evolution_Step_Req::Get_Number()

//  Number associated with the action request. Always 1

{
    return 1;
}
//----------------------------------------------------------------------------------------------------------------

void Evolution_Step_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Evolution StepAction Request. There aren't
//  any, so just blank

{
    Param_List.push_back(string(""));
}
//----------------------------------------------------------------------------------------------------------------

string Evolution_Step_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Reactor");
}
//----------------------------------------------------------------------------------------------------------------

string Evolution_Step_Req::Get_Type()

//  Return the Type of the Action Request

{
    return (string("Execute"));
}  */
//=================================================================================================================

Remove_Maint_MolType_Spec_Req::Remove_Maint_MolType_Spec_Req(Remove_Maint_MolType_Spec_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())


{
    Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Repeat_Interval_ = ReqRqstr_Ptr->Get_Repeat_Interval();
//    Repeat_Req = ReqRqstr_Ptr->Repeat_Flag;
}
//-----------------------------------------------------------------------------------------------------------------        
        
string Remove_Maint_MolType_Spec_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the Action Request

{
    bool  Success = true;
    int MolType_Index;
    if (not Reactor::Get_Reactor__Ptr()->Remove_Maintained_MolType_Spec(MolType_Index))
    THEN
        Success = false;
    ENDIF
    
    
    string Action_Result;  
    if (Success)
    THEN     
        if(not Repeat_Interval_)
        THEN   // No repeats for this Action Request
            Action_Result = To_String(1) + string(" Maintained Molecule Type Specification removed. ")
                            + string("Molecule Type = ") + To_String(MolType_Index);
        ELSEIF (not Repeat_Req_)  // First in a repeat sequence
        THEN
            Action_Result = To_String(1) + string(" Maintained Molecule Type Specification removed. ")
                            + string("Molecule Type = ") + To_String(MolType_Index)
                            + string(". First in sequence of repeat Action Requests. Successful repeats will not be reported.");
        ELSE
            Action_Result = string("No_Rep");  // Its a repeat so don't report
        ENDIF
    
//      If repeat required, then create another Action Request and pass back pointer. Mark the Action Request
//      as an automatically generated repeat (last param)
    
        if(Repeat_Interval_)
        THEN
//			Remove_Maint_MolType_Spec_ReqRqstr  Rqstr;
//			Rqstr.Action_Time = Time + Repeat_Interval;
//			Rqstr.Repeat_Interval = Repeat_Interval;
//   			Rqstr.Repeat_Flag = true;

//            CREATE_OBJECT(Remove_Maint_MolType_Spec_Req, Repeat_Act_Req_Ptr = new Remove_Maint_MolType_Spec_Req(&Rqstr));
        ELSE
            Repeat_Act_Req_Ptr = 0;
        ENDIF
    ELSE
        Action_Result = Get_Warn_Tx()
                        + string(" Cannot remove ") 
                        + To_String(1) 
                        + string(" Maintained Molecule Type Specification");
    ENDIF
 
    return Action_Result; 
}
//-----------------------------------------------------------------------------------------------------------------        
        
int  Remove_Maint_MolType_Spec_Req::Get_Number() const

{
    return 1;
}        
//-----------------------------------------------------------------------------------------------------------------        
        
string Remove_Maint_MolType_Spec_Req::Get_Subject() const

{
    return string("Maint. Mol. Type Spec.");
}             
//----------------------------------------------------------------------------------------------------------------

string Remove_Maint_MolType_Spec_Req::Get_Type() const

{
    return string("Remove");
}             
//----------------------------------------------------------------------------------------------------------------

void  Remove_Maint_MolType_Spec_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Remove Maintained Molecule Type Spec. action request

{
    Param_List.push_back(string("")); // No Parameters 
}

//=================================================================================================================


Remove_Transformation_Req::Remove_Transformation_Req(Remove_Transformation_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())


{
    Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Num_To_Remove = ReqRqstr_Ptr->Num_To_Remove;
    Repeat_Interval_ = ReqRqstr_Ptr->Get_Repeat_Interval();
    Reactant_Spec = ReqRqstr_Ptr->Reactant_Spec;
    Product_Spec = ReqRqstr_Ptr->Product_Spec;
}
//-----------------------------------------------------------------------------------------------------------------

string  Remove_Transformation_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

{
    return string("");    //  ****  NEEDS FINALISATION
}
//-----------------------------------------------------------------------------------------------------------------
int  Remove_Transformation_Req::Get_Number()

//  Return the number of Transfoormations to remove 

{
    return Num_To_Remove;
}
//-----------------------------------------------------------------------------------------------------------------

string Remove_Transformation_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Transformation");
}

//-----------------------------------------------------------------------------------------------------------------
string Remove_Transformation_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Remove");   
}
//-----------------------------------------------------------------------------------------------------------------       

void  Remove_Transformation_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Remove Transformation Action Request

{
    Param_List.push_back(string("Reactant = ") + Reactant_Spec);
    Param_List.push_back(string("Product  = ") + Product_Spec);   
}
//=================================================================================================================
/*
Set_Inflow_Rate_Req::Set_Inflow_Rate_Req(int Action_Time, int Inflow_Rate)

//  Constructor for the Action Request

{
    Time = Action_Time;
    Reactor_Inflow_Rate = Inflow_Rate;
    Repeat_Interval = 0;
    Repeat_Req = false;
}
//------------------------------------------------------------------------------------------------------------------

string  Set_Inflow_Rate_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the Action request to set the Reactor Inflow Rate
{
    Reactor::Get_Reactor__Ptr()->Set_Inflow_Rate(Reactor_Inflow_Rate);    
    string Action_Result = string("Reactor Inflow Rate set to ") + To_String(Reactor_Inflow_Rate)
                            + string(" for each food Molecule Type");    
    return  Action_Result;
}
//-----------------------------------------------------------------------------------------------------------------

int  Set_Inflow_Rate_Req::Get_Number()

//  Return the number of times to set the Reactor Inflow Rate. Must always be 1

{
    return 1;
}
//-----------------------------------------------------------------------------------------------------------------

void  Set_Inflow_Rate_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Set Reactor Inflow Rate Action Request

{
    Param_List.push_back(string("Value = ") + To_String(Reactor_Inflow_Rate));
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Inflow_Rate_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Inflow Rate/Food Type");
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Inflow_Rate_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Set");   
}  */
//===============================================================================================================

Set_Reactor_Vol_Req::Set_Reactor_Vol_Req(Set_Reactor_Vol_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())


{
    Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Reactor_Volume = ReqRqstr_Ptr->Reactor_Volume;
    Repeat_Interval_ = 0;
}
//-----------------------------------------------------------------------------------------------------------------

string  Set_Reactor_Vol_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

{
    return string("");
}
//-----------------------------------------------------------------------------------------------------------------

int  Set_Reactor_Vol_Req::Get_Number()

//  Return the number of times to set volume. Must always be 1

{
    return 1;
}
//-----------------------------------------------------------------------------------------------------------------

string Set_Reactor_Vol_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Reactor Volume");
}

//-----------------------------------------------------------------------------------------------------------------
string Set_Reactor_Vol_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Set");   
}
//-----------------------------------------------------------------------------------------------------------------       

void  Set_Reactor_Vol_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Set Reactor Volume Action Request

{
    Param_List.push_back(string("Reactor Volume = ") + To_String(Reactor_Volume));
}
//=================================================================================================================

Set_Max_Rand_Rate_Const_Constr_Req::Set_Max_Rand_Rate_Const_Constr_Req
												(Set_Max_Rand_Rate_Const_Constr_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor for the Action Request

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())


{
    Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Max_Rate_Constant = ReqRqstr_Ptr->Max_Rate_Constant;
    Repeat_Interval_ = 0;
    Repeat_Req_ = false;
}
//------------------------------------------------------------------------------------------------------------------

string  Set_Max_Rand_Rate_Const_Constr_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the Action request to set the Maximum Rate Constant for Constructions

{
    string Action_Result;
    if (Chemistry::Get_Rate_Constant_Model() not_eq Chemistry::Random)
    THEN
        Action_Result = Get_Warn_Tx()
                        + string(" Max Rate Constant for Constructions not set. ")
                        + string("Incorrect Rate Constant Model.");
    ELSE
        Construction::Set_Max_Rand_Rate_Constant(Max_Rate_Constant);    
        Action_Result = string("Maximum Rate Constant for Random Model Constructions set to ") 
                        + To_String(Max_Rate_Constant);    
    ENDIF
    return  Action_Result;
}
//-----------------------------------------------------------------------------------------------------------------

int  Set_Max_Rand_Rate_Const_Constr_Req::Get_Number()

//  Return the number of times to set the Maximum Rate Constant for Constructions. Must always be 1

{
    return 1;
}
//-----------------------------------------------------------------------------------------------------------------

void  Set_Max_Rand_Rate_Const_Constr_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Set Maximum Rate Constant for Constructions Action Request

{
    Param_List.push_back(string("Value = ") + To_String(Max_Rate_Constant));
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Max_Rand_Rate_Const_Constr_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Max. Rate for Random Model Constructions");
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Max_Rand_Rate_Const_Constr_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Set");   
}
//=================================================================================================================


Set_Max_Rand_Rate_Const_Fission_Req::Set_Max_Rand_Rate_Const_Fission_Req
										(Set_Max_Rand_Rate_Const_Fission_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor for the Action Request

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())


{
    Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Max_Rate_Constant = ReqRqstr_Ptr->Max_Rate_Constant;
    Repeat_Interval_ = 0;
    Repeat_Req_ = false;
}
//------------------------------------------------------------------------------------------------------------------

string  Set_Max_Rand_Rate_Const_Fission_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the Action request to set the Maximum Rate Constant for Fission
//  with the Random rate constant model

{
    string Action_Result;
    if (Chemistry::Get_Rate_Constant_Model() not_eq Chemistry::Random)
    THEN
        Action_Result = Get_Warn_Tx()
                        + string(" Max Rate Constant for Fissions not set. ")
                        + string("Incorrect Rate Constant Model.");
    ELSE
        Fission::Set_Max_Rand_Rate_Constant(Max_Rate_Constant);    
        Action_Result = string("Maximum Rate Constant for Random Model Fissions set to ") 
                        + To_String(Max_Rate_Constant);    
    ENDIF
    return  Action_Result;
}
//-----------------------------------------------------------------------------------------------------------------

int  Set_Max_Rand_Rate_Const_Fission_Req::Get_Number()

//  Return the number of times to set the Maximum Rate Constant for Fission (random Rate Const model). 
//  Must always be 1

{
    return 1;
}
//-----------------------------------------------------------------------------------------------------------------

void  Set_Max_Rand_Rate_Const_Fission_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Set Maximum Rate Constant for Fissions Action Request

{
    Param_List.push_back(string("Value = ") + To_String(Max_Rate_Constant));
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Max_Rand_Rate_Const_Fission_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Max. Rate for Fissions");
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Max_Rand_Rate_Const_Fission_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Set");  
}

//=================================================================================================================


Set_Max_Rate_Const_Transformation_Req::Set_Max_Rate_Const_Transformation_Req
									(Set_Max_Rate_Const_Transformation_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor for the Action Request

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())

{
    Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Max_Rate_Constant = ReqRqstr_Ptr->Max_Rate_Constant;
    Repeat_Interval_ = 0;
    Repeat_Req_ = false;
}
//------------------------------------------------------------------------------------------------------------------

string  Set_Max_Rate_Const_Transformation_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the Action request to set the Maximum Rate Constant for Transformation

{
    Transformation::Set_Max_Rand_Rate_Constant(Max_Rate_Constant);    
    string Action_Result = string("Maximum Rate Constant for Transformation set to ") + To_String(Max_Rate_Constant);    
    return  Action_Result;
}
//-----------------------------------------------------------------------------------------------------------------

int  Set_Max_Rate_Const_Transformation_Req::Get_Number()

//  Return the number of times to set the Maximum Rate Constant for Transformation. Must always be 1

{
    return 1;
}
//-----------------------------------------------------------------------------------------------------------------

void  Set_Max_Rate_Const_Transformation_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Set Maximum Rate Constant for Transformations Action Request

{
    Param_List.push_back(string("Value = ") + To_String(Max_Rate_Constant));
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Max_Rate_Const_Transformation_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Max. Rate for Transformations");
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Max_Rate_Const_Transformation_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Set");   
}
//===================================================================================================

Set_Rate_Const_Model_Req::Set_Rate_Const_Model_Req(Set_Rate_Const_Model_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor for the Action Request

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())

{
    if (ReqRqstr_Ptr->Rate_Const_Model_Spec not_eq string("Random")
	   and
	ReqRqstr_Ptr->Rate_Const_Model_Spec not_eq string("Arrenhius"))
    THEN
	Assert(true == false, 
	       "Set_Rate_Const_Model_Req::Set_Rate_Const_Model_Req - Invalid Rate Constant Model Specified");
    ENDIF

    Time_ = ReqRqstr_Ptr->Get_Action_Time();
    Rate_Const_Model_Spec = ReqRqstr_Ptr->Rate_Const_Model_Spec;
    Repeat_Interval_ = 0;
    Repeat_Req_ = false;
}
//-----------------------------------------------------------------------------------------------------------------

string  Set_Rate_Const_Model_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the Action request to set the Rate Constant Model

{   
    Chemistry::Rate_Const_Model_Tp  Model;
    string Action_Result;

    if (Rate_Const_Model_Spec == "Random")
    THEN
	   Model = Chemistry::Random;
    ELSEIF (Rate_Const_Model_Spec == "Arrenhius")
    THEN
	   Model = Chemistry::Arrenhius;
    ELSE
	Assert (true == false, 
		"Set_Rate_Const_Model_Req::Execute - Invalid Rate Constant Model requested");
    ENDIF

    if (Chemistry::Set_Rate_Contant_Model(Model) == true)
    THEN
	   Action_Result = string("Rate Contant Model set to: ") + Rate_Const_Model_Spec;
    ELSE
	   Action_Result = Get_Warn_Tx() 
	       + string(" Cannot set Rate Contant Model - it has already been set.");
    ENDIF

    return  Action_Result;
}
//-----------------------------------------------------------------------------------------------------------------

int  Set_Rate_Const_Model_Req::Get_Number()

//  Return the number of times to set the Rate Constant Model. Must always be 1

{
    return 1;
}
//-----------------------------------------------------------------------------------------------------------------

void  Set_Rate_Const_Model_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Set Rate Constant Model Action Request

{
    Param_List.push_back(string("Value = ") + Rate_Const_Model_Spec);
}
//-----------------------------------------------------------------------------------------------------------------

string Set_Rate_Const_Model_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Rate Constant Model");
}
//-----------------------------------------------------------------------------------------------------------------

string Set_Rate_Const_Model_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Set");   
}
//=================================================================================================================

Terminate_Scenario_Req::Terminate_Scenario_Req(Terminate_Scenario_ReqRqstr*  ReqRqstr_Ptr):

//  Constructor

    Action_Request(ReqRqstr_Ptr->Get_Action_Time(), 
                   ReqRqstr_Ptr->Get_Repeat_Interval())

{
    Time_ =ReqRqstr_Ptr-> Get_Action_Time();
    Repeat_Req_ = false;
    Repeat_Interval_ = 0;
}
//----------------------------------------------------------------------------------------------------------------

string Terminate_Scenario_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the action request to Disable (inactivate) the Reactor. 
//  This is used to terminate the scenario

{
    Reactor::Get_Reactor__Ptr()->Disable();
    return string("Scenario run terminated");
}
//----------------------------------------------------------------------------------------------------------------

int Terminate_Scenario_Req::Get_Number()

//  Number associated with the action request. Always 1

{
    return 1;
}
//----------------------------------------------------------------------------------------------------------------

void Terminate_Scenario_Req::Get_Params(list<string>& Param_List) const

//  Return the parameters for the Terminate_Scenario Action Request. There aren't
//  any, so just blank

{
    Param_List.push_back(string(""));
}
//-----------------------------------------------------------------------------------------------------------------


string Terminate_Scenario_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Scenario");
}

//-----------------------------------------------------------------------------------------------------------------
string Terminate_Scenario_Req::Get_Type()

//  Return the Type of the Action Request

{
    return (string("Terminate"));
}

//=================================================================================================================



//=================================================================================================================
/*
Set_Num_Maint_MolTypes_Req::Set_Num_Maint_MolTypes_Req(int Action_Time, int Num_Maint_M_Types)

//  Constructor for the Action Request

{
    Time = Action_Time;
    Num_Maint_MolTypes = Num_Maint_M_Types;
    Repeat_Interval = 0;
    Repeat_Req = false;
}
//------------------------------------------------------------------------------------------------------------------

string  Set_Num_Maint_MolTypes_Req::Execute(Action_Request*& Repeat_Act_Req_Ptr)

//  Execute the Action request to set the Number of Maintained Molecule types

{
    Reactor::Get_Reactor__Ptr()->Set_Num_Maint_MolTypes(Num_Maint_MolTypes);    
    string Action_Result = string("Number of Food Molecule Types set to ") + To_String(Num_Maint_MolTypes);    
    return  Action_Result;
}
//-----------------------------------------------------------------------------------------------------------------

int  Set_Num_Maint_MolTypes_Req::Get_Number()

//  Return the number of times to set the Number of Maintained Mol Types. Must always be 1

{
    return 1;
}
//-----------------------------------------------------------------------------------------------------------------

void  Set_Num_Maint_MolTypes_Req::Get_Params(list<string>& Param_List) const

// Return the parameters for the Set Maximum Rate Constant Action Request

{
    Param_List.push_back(string("Value = ") + To_String(Num_Maint_MolTypes));
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Num_Maint_MolTypes_Req::Get_Subject()

//  Return the subject for the Action Request

{
    return string("Num. Food Mol. Types");
}
//-----------------------------------------------------------------------------------------------------------------
string Set_Num_Maint_MolTypes_Req::Get_Type()

//  Return the Type of the Action Request

{
    return string("Set");   
}  */
//-----------------------------------------------------------------------------------------------------------------
