/****************************************************************************
*
*	  Module Filename:  Scenario.cpp
*	  Module Name:      Scenario
*	  Description:      Specifies the conditions for a simulation run, including
*                       initial setup and Actions to be triggered externally during the run                  
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

#include "globldef.h"
#include "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

#include  <gsl/gsl_rng.h>    // GNU Scientific library
//---------------- Utilities Subsystem ----------------------------------------------------------------------------
#include  "Random_Sequence.h"
//----------------------------------------------------------------------------------------------------------

//  ----------------  Simulation Subsystem -----------------------------------------------------------------------
#include "Scenario.h"
#include "Action_Request.h"
#include "Scenario_Manager.h"
//---------------------------------------------------------------------------------------------------------------

//  ----------------  Chemistry Subsystem -----------------------------------------------------------------------
#include "Molecule_Type.h"
//---------------------------------------------------------------------------------------------------------------

//  ----------------  Reactor Subsystem -----------------------------------------------------------------------
#include "Reactor.h"
//---------------------------------------------------------------------------------------------------------------

//  ----------------  User Interface Manager Subsystem -----------------------------------------------------------------------
#include  "Simulation_Display_API.h"  //  Use only the API (keeps the subsystem dependencies simple)
//---------------------------------------------------------------------------------------------------------------

void  Scenario::Add_ActReq_To_Scen(Action_Request*  ActReq_Pt)

//  Add the specified Action to the Scenario, sorted according to Action_Time

{   TRACE("Scenario::Add_ActReq_To_Scen");
    Scenario_ActReq_Ls.push_back(ActReq_Pt);
    Scenario_ActReq_Ls.sort(Action_Request::Compare_Time);
}
//-----------------------------------------------------------------------------------------------------------------

void Scenario::Clear()

//  Delete all of the Scenario's child objects and clear the list of Action Requests

{
    TRACE("Scenario::Clear");
                                                    // to define the scanario
//  Clear any scheduled Action Requests
    while (not ActReq_Sched_Ls.empty())
    BEGIN
        Action_Request* ActReq_Pt = ActReq_Sched_Ls.top();
        DELETE_OBJECT(Action_Request, ActReq_Pt);
        ActReq_Sched_Ls.pop();
    END
    
//  Clear the Action Requests in the scenario definition
    list <Action_Request*>::iterator  Iter;    
    for (Iter = Scenario_ActReq_Ls.begin();
         Iter not_eq Scenario_ActReq_Ls.end();
         Iter++)
    BEGIN
        DELETE_OBJECT(Action_Request, *Iter);
    END
    
    Scenario_ActReq_Ls.clear();
}
//----------------------------------------------------------------------------------------------------------------

void  Scenario::Display_Simulation()

//  Display Network and Statistics every at the required interval

{
    TRACE("Scenario::Display_Simulation");

    if (Reactor::Time() % Statistics_Collection_Interval == 0)
    THEN
        Simulation_Display_API::Display_Chemistry();
        Simulation_Display_API::Display_Reactor();
        Simulation_Display_API::Display_Statistics_Optimised(true);
        Simulation_Display_API::Display_Object_Statistics();
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

Action_Request*  Scenario::Get_Curr_Scen_ActReq_Pt()

//  Return a pointer to the curent Action_Request

{
    Assert (not Scenario_ActReq_Ls.empty(),"Scenario::Get_Curr_Scen_ActReq_Pt - No Action Requests exist for Scenario");
    return *Curr_Scen_ActReq_It;
}
//-----------------------------------------------------------------------------------------------------------------

Scenario::Execution_State  Scenario::Get_Execution_State()

//  Return the Scenario's Execution State

{
    return Exec_State;
}
//----------------------------------------------------------------------------------------------------------------

void  Scenario::Handle_Reactor_Alerts()

//  Get and output Reactor Alerts

{
    list<string> Reactor_Alerts = Reactor::Get_And_Clear_Reactor_Alerts();
    list<string>::iterator  Iter;
    for (Iter = Reactor_Alerts.begin();
            Iter not_eq Reactor_Alerts.end();
            Iter++)
    BEGIN     
        Simulation_Display_API::Output_Log_Entry(*Iter);
    END
}
//----------------------------------------------------------------------------------------------------------------


// Action_Request*  Scenario::Get_Next_Sched_ActReq_Pt()
// 
// //  Get a pointer to the next scheduled Action Request,
// //  and remove it from the schedule
// {
//     Action_Request* ActReq_Pt = ActReq_Sched_Ls.top();
//     ActReq_Sched_Ls.pop();
//     return  ActReq_Pt;
// }
//-----------------------------------------------------------------------------------------------------------------
bool Scenario::Iter_First_Scen_ActReq()

//  Set the Current Scenario Action_Request to the first Action Request. 
//  Return false if there are no Action Requests for the Scenario

{
    if(not Scenario_ActReq_Ls.empty())
    THEN
        Curr_Scen_ActReq_It = Scenario_ActReq_Ls.begin();
        return true;
    ELSE
        return false;
    ENDIF
}   
//-----------------------------------------------------------------------------------------------------------------

bool    Scenario::Iter_Next_Scen_ActReq()

//  Move the Current scenario Action_Request iterator 
//  to point to the next Action Request 
//  for the Scenario. Return false if already
//  on last item or there are no Action Requests

{
    if(++Curr_Scen_ActReq_It not_eq Scenario_ActReq_Ls.end())
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}
//---------------------------------------------------------------------------------------------------------------

// void  Scenario::Output_Summary_Statistics()
// 
// {
//     Assert(!Reactor::Get_Reactor__Ptr()->Is_Enabled(), "Scenario::Output_Summary_Statistics - Called while Reactor is enabled");
// 
//     double Max_Interaction_Damping_Rate;
// 
//     int Max_MolTypes;
//     int Max_Num_Molecules;
//     int Max_R_Int_Types;
//     double Max_Int_Rate;
// 
//     int Min_MolTypes;
//     int Min_Num_Molecules;
//     int Min_R_Int_Types;
//     double Min_Int_Rate;
// 
//     int Max_MolTypes_P_I_Transient;
//     int Max_Num_Molecules_P_I_Transient;
//     int Max_R_Int_Types_P_I_Transient;
//     double Max_Int_Rate_P_I_Transient;
// 
//     int Min_MolTypes_P_I_Transient;
//     int Min_Num_Molecules_P_I_Transient;
//     int Min_R_Int_Types_P_I_Transient;
//     double Min_Int_Rate_P_I_Transient;
// 
//     double Avg_Molecules_P_I_T;
//     double Avg_MolTypes_P_I_T;
//     double Avg_R_Int_Types_P_I_T;
//     double Avg_Int_Rate_P_I_T;
// 
//     int  Init_Transient_Time;
// 
//     Reactor::Get_Reactor__Ptr()->Get_Run_Summary_Statistics(Max_Interaction_Damping_Rate,
//                                                             Max_MolTypes,
//                                                             Max_Num_Molecules,
//                                                             Max_R_Int_Types,
//                                                             Max_Int_Rate,
// 
//                                                             Min_MolTypes,
//                                                             Min_Num_Molecules,
//                                                             Min_R_Int_Types,
//                                                             Min_Int_Rate,
// 
//                                                             Max_MolTypes_P_I_Transient,
//                                                             Max_Num_Molecules_P_I_Transient,
//                                                             Max_R_Int_Types_P_I_Transient,
//                                                             Max_Int_Rate_P_I_Transient,
// 
//                                                             Min_MolTypes_P_I_Transient,
//                                                             Min_Num_Molecules_P_I_Transient,
//                                                             Min_R_Int_Types_P_I_Transient,
//                                                             Min_Int_Rate_P_I_Transient,
// 
//                                                             Avg_Molecules_P_I_T,
//                                                             Avg_MolTypes_P_I_T,
//                                                             Avg_R_Int_Types_P_I_T,
//                                                             Avg_Int_Rate_P_I_T,
//                                                             Init_Transient_Time);
// 
//     string Summary_Output =   To_String(Name) + " "
//                             + To_String(Max_Mass_For_Scen) + " " 
//                             + To_String(Num_MolTypes) + " " 
//                             + To_String(Num_Maint_MolTypes) + " " 
//                             + To_String(Max_Rate_Const_Construction) + " " 
//                             + To_String(Max_Rate_Const_Fission) + " " 
//                             + To_String(Num_Cat_Trans) + " " 
//                             + To_String(Random_Seed) + " " 
//                             + To_String(End_Time) + " " 
//                             + To_String(Stats_Collection_Interval) + " "
//                             + To_String(Init_Transient_Time) + " "
// 
//                             + To_String(Max_Interaction_Damping_Rate) + " "
//                 
//                             + To_String(Max_MolTypes) + " "      + To_String(Max_MolTypes_P_I_Transient) + " "
//                             + To_String(Min_MolTypes) + " "      + To_String(Min_MolTypes_P_I_Transient) + " "
//                                                                 + To_String(Avg_MolTypes_P_I_T) + " "
// 
//                             + To_String(Max_Num_Molecules) + " " + To_String(Max_Num_Molecules_P_I_Transient) + " "
//                             + To_String(Min_Num_Molecules) + " " + To_String(Min_Num_Molecules_P_I_Transient) + " "
//                                                                 + To_String(Avg_Molecules_P_I_T) + " "
// 
//                             + To_String(Max_R_Int_Types) + " "   + To_String(Max_R_Int_Types_P_I_Transient) + " "
//                             + To_String(Min_R_Int_Types) + " "   + To_String(Min_R_Int_Types_P_I_Transient) + " "
//                                                                 + To_String(Avg_R_Int_Types_P_I_T)  + " "
// 
//                             + To_String(Max_Int_Rate) + " "      + To_String(Max_Int_Rate_P_I_Transient) + " "
//                             + To_String(Min_Int_Rate) + " "      + To_String(Min_Int_Rate_P_I_Transient) + " "
//                                                                 + To_String(Avg_Int_Rate_P_I_T);
//                                                     
// 
//     cout << Summary_Output  << endl;
//     
// //    filebuf* Buffer_Ptr = Scenario_Manager::Get_Ptr()->Get_Scenario_Output_Buffer_Ptr();
// //    ostream output(Buffer_Ptr);
// //    output << Summary_Output << endl;
//     Simulation_Display_API::Save_Statistics_Display(&Name);
// }
//----------------------------------------------------------------------------------------------------------------

void  Scenario::Log_ActReq_Exec_Outcome_If_Reqd(Action_Request*  ActReq_Pt) const

//  Log the result of the Action Request execution if required;
//  That is, if this is not a repeat or if its the first to fail in a repeat series
 
{
    TRACE("Scenario::Log_ActReq_Exec_Outcome_If_Reqd");
    if (not ActReq_Pt->Get_Repeat_Req_Fl()
                    or
        (ActReq_Pt->Get_Fail_Fl() and
            (not ActReq_Pt->Get_Prev_Fail_Fl())))
    THEN
        string  Rslt_Tx = ActReq_Pt->Get_Act_Rslt_Tx();
        Simulation_Display_API::Output_Log_Entry(Rslt_Tx );
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void  Scenario::Output_Warning(string Warning_Messsage)

{
    cout << Warning_Messsage  << endl;
}
//-----------------------------------------------------------------------------------------------------------------

bool  Scenario::Run(int Num_Timesteps)

//  Run the Scenario for the specified number of timesteps
//  or until the Reactor is Disabled. Return true if Reactor still Enabled, otherwise false

{   
    TRACE("Scenario::Run");
    Assert (Num_Timesteps > 0, "Scenario::Run - Invalid number of timesteps requested");
    Reactor* Reactor_Ptr = Reactor::Get_Reactor__Ptr();
    
//  Main simulation loop
    for (int Iloop = 0;  Iloop < Num_Timesteps; Iloop++)  // One iteration per timestep
    BEGIN      
//      Execute Action Requests for this timestep
        int Reactor_Time = Reactor_Ptr->Get_Reactor_Time();
        while ((not ActReq_Sched_Ls.empty()) 
                        and not 
                (ActReq_Sched_Ls.top()->Get_Time() > Reactor_Time))  
        BEGIN
            Action_Request* Curr_ActReq_Pt = ActReq_Sched_Ls.top();
            ActReq_Sched_Ls.pop();   // Remove the Action Request from the schedule

//===============================================================================
            Curr_ActReq_Pt->Execute();  
//===============================================================================

            Log_ActReq_Exec_Outcome_If_Reqd(Curr_ActReq_Pt); 

            if (not Schedule_Repeat_Exec_If_Reqd(Curr_ActReq_Pt))                      
            THEN
                DELETE_OBJECT(Action_Request, Curr_ActReq_Pt);
            ENDIF
        END
 
//      Now invoke the main simulation logic for this timestep, and log any alerts
//======================================================================================        
        bool Alert_Flag = Reactor_Ptr->Single_Timestep(Statistics_Collection_Interval);
//======================================================================================
        if(Alert_Flag) Handle_Reactor_Alerts();

        Display_Simulation();
            
//      Stop if the Reactor has been disabled
        if (not Reactor_Ptr->Is_Enabled())  break;
    END
    return Reactor_Ptr->Is_Enabled();
}
//-----------------------------------------------------------------------------------------------------------------

Scenario::Scenario()

//  Default Constructor - Setup Catalytic Transformation Oscillating Network Scenario
//  AISB Set 22 - Run 20, Oscillating Network

{
    int Max_Mass_For_Scen = 5;
    int Num_MolTypes = 200;
    int Num_Maint_MolTypes = 3;
    double Max_Rate_Const_Construction = 0.05;
    double Max_Rate_Const_Fission = 0.05;
    int Num_Cat_Trans = 235;
    int Random_Seed = 62987;
    int End_Time = 15000;
    double  Trackers_Per_Maint_Mol = 0.01;
    int Stats_Collection_Interval = 2000;
    string  Name = string("Osciilating_Network");
    
    Setup(Max_Mass_For_Scen, 
             Num_MolTypes,          
             Num_Maint_MolTypes,  
             Max_Rate_Const_Construction,  
             Max_Rate_Const_Fission,      
             Num_Cat_Trans,       
             Random_Seed, 
             End_Time,            
             Trackers_Per_Maint_Mol,
             Stats_Collection_Interval,
             Name);
}
//---------------------------------------------------------------------------------------------------------------

Scenario::Scenario(int Max_Mass_For_Scen, 
                   int Num_MolTypes,          
                   int Num_Maint_MolTypes,   
                   double Max_Rate_Const_Construction,  
                   double Max_Rate_Const_Fission,      
                   int Num_Cat_Trans,        
                   int Random_Seed, 
                   int End_Time,          
                   double Trackers_Per_Maint_Mol,    
                   int Stats_Collection_Interval,
                   string Name)
{
    Setup(Max_Mass_For_Scen, 
          Num_MolTypes,          
          Num_Maint_MolTypes,   
          Max_Rate_Const_Construction,  
          Max_Rate_Const_Fission,      
          Num_Cat_Trans,        
          Random_Seed, 
          End_Time,              
          Trackers_Per_Maint_Mol,
          Stats_Collection_Interval,
          Name);
}
//----------------------------------------------------------------------------------------------------------------

bool  Scenario::Compare_ActReq_Time::operator() (const Action_Request* x, const Action_Request* y)
       
{
    return Action_Request::Compare_Time(y,x);
}
//---------------------------------------------------------------------------------------------------------------

Scenario::Scenario(string  Scenario_Name)

{/*
    if(Scenario_Name == "Tracker_Test")
    THEN
        Setup_Tracker_Test();
    ELSEIF(Scenario_Name == "Tracker_Test_2")
    THEN
        Setup_Tracker_Test_2();
    ELSEIF(Scenario_Name == "Energy_Test_1")
    THEN
        Setup_Energy_Test_1();
    ELSEIF(Scenario_Name == "Energy_Test_2")
    THEN
        Setup_Energy_Test_2();
    ELSEIF(Scenario_Name == "Energy_Test_3")
    THEN
        Setup_Energy_Test_3();
    ELSEIF(Scenario_Name == "Energy_Test_4")
    THEN
        Setup_Energy_Test_4();
    ELSEIF(Scenario_Name == "Energy_Test_5")
    THEN
        Setup_Energy_Test_5();
    ELSEIF(Scenario_Name == "Mutation_Test_1")
    THEN
        Setup_Mutation_Test_1();
    ELSEIF(Scenario_Name == "Mutation_Test_2")
    THEN
        Setup_Mutation_Test_2();
    ELSEIF(Scenario_Name == "Mutation_Test_3")
    THEN
        Setup_Mutation_Test_3();
    ELSEIF(Scenario_Name == "Mutation_Test_4")
    THEN
        Setup_Mutation_Test_4();
    ELSEIF(Scenario_Name == "Mutation_Test_5")
    THEN
        Setup_Mutation_Test_5();
    ELSEIF(Scenario_Name == "Mutation_Test_6")
    THEN
        Setup_Mutation_Test_6();
    ELSEIF(Scenario_Name == "Mutation_Test_7")
    THEN
        Setup_Mutation_Test_7();
    ELSEIF(Scenario_Name == "Mutation_Test_8")
    THEN
        Setup_Mutation_Test_8();
    ELSEIF(Scenario_Name == "Mutation_Test_9")
    THEN
        Setup_Mutation_Test_9();
    ELSEIF (Scenario_Name == "Cat_Trans_Diagnostic_Test_1")
    THEN
        Setup_Cat_Trans_Diagnostic_Test_1();
    ELSE
        Assert (true == false, "Scenario::Scenario - Invalid Scenario Name");
		ENDIF  */
}
//-------------------------------------------------------------------------------------------------------------

void  Scenario::Schedule_ActReq(Action_Request*  ActReq_Pt)

//  Schedule the specified Action Request for execution

{   TRACE("Scenario::Schedule_ActReq");
    ActReq_Sched_Ls.push(ActReq_Pt);
}
//---------------------------------------------------------------------------------------------------------------
bool  Scenario::Schedule_Repeat_Exec_If_Reqd(Action_Request*  ActReq_Pt) 

//  Schedule a repeat execution of the specified Action Request if required.

{
    TRACE("Scenario::Schedule_Repeat_Exec_If_Reqd");

    bool Repeat_Req_Fl = ActReq_Pt->Setup_Repeat_If_Reqd();
    if(Repeat_Req_Fl ) 
    THEN
        ActReq_Sched_Ls.push(ActReq_Pt);
    ENDIF
    return Repeat_Req_Fl;
}
//----------------------------------------------------------------------------------------------------------------

void Scenario::Setup( int Max_Mass_For_Scen,
                      int Num_MolTypes,           
                      int Num_Maint_MolTypes,     
                      double Max_Rate_Const_Construction, 
                      double Max_Rate_Const_Fission,     
                      int Num_Cat_Trans,         
                      int Random_Seed, 
                      int End_Time,       
                      double  Trackers_Per_Maint_Mol,    
                      int Stats_Collection_Interval,
                      string Name)

{
    Exec_State = New;

    Scenario_Setup_Log_Outputs.push_back(string("PRE-DEFINED SCENARIO:-"));
    Scenario_Setup_Log_Outputs.push_back(string("Scenario: Oscillator"));
    Scenario_Setup_Log_Outputs.push_back(string("Description: This scenario has a Chemistry in which Constructions and Fissions are paired to form Catalytic Transformations. When the simulation runs, the network adopts an oscillating behaviour. This can be seen from the data series plots on the Simulation Statistics tab. The Manhattan Plot, also on the Simulation Statistics tab, shows a striking diagonal pattern. This demonstrates that the Reactor is periodcally returning to the same state (as opposed to following a random path through the space of possible states). To see the repeating behaviour, run the simulation for at least 400,000 timesteps"));
//    Simulation_Display_API::Output_Simple_Log_Entry(string(""));

    this->Max_Mass_For_Scen = Max_Mass_For_Scen;
    this->Random_Seed  = Random_Seed;
    this->Name = Name;
    this->Num_MolTypes = Num_MolTypes;    
    this->Num_Maint_MolTypes = Num_Maint_MolTypes;
    this->Max_Rate_Const_Construction = Max_Rate_Const_Construction;
    this->Max_Rate_Const_Fission = Max_Rate_Const_Fission;
    this->Num_Cat_Trans = Num_Cat_Trans;      
    this->End_Time = End_Time;      
    this->Trackers_Per_Maint_Mol = Trackers_Per_Maint_Mol;
    this->Stats_Collection_Interval = Stats_Collection_Interval;

    Set_Statistics_Collection_Interval(Stats_Collection_Interval);

    Scenario_Setup_Log_Outputs.push_back(string("Maximum Molecule Mass for scenario is ") + To_String(Max_Mass_For_Scen));
 
    Scenario_Setup_Log_Outputs.push_back(string("Random Number Seed for scenario is ") + To_String(Random_Seed));

/*
    Action_Request*  Action_Req_Ptr; 
    
    int Number_To_Add;     
    int Action_Time = 0;
    int Repeat_Int = 0; 
  
    string Mass_Specifier = "1";
    Number_To_Add = Num_MolTypes;
    CREATE_OBJECT(Add_MolType_Req, Action_Req_Ptr = new Add_MolType_Req(Action_Time, Number_To_Add, Repeat_Int, false, Mass_Specifier));
    Add_Action_Req(Action_Req_Ptr);  
  
    Action_Time = 0;
    Repeat_Int = 0;  
    Number_To_Add = Num_Maint_MolTypes;
    CREATE_OBJECT(Add_Maint_MolType_Spec_Req, Action_Req_Ptr = new Add_Maint_MolType_Spec_Req(Action_Time, Number_To_Add, Trackers_Per_Maint_Mol, Repeat_Int, false));
    Add_Action_Req(Action_Req_Ptr);  
    
    Action_Time = 0;
    double Max_Rate_Const = Max_Rate_Const_Construction;
    CREATE_OBJECT(Set_Max_Rate_Const_Construction_Req, Action_Req_Ptr = new Set_Max_Rate_Const_Construction_Req(Action_Time, Max_Rate_Const));
    Add_Action_Req(Action_Req_Ptr);   
    
    
    Action_Time = 0;
    Max_Rate_Const = Max_Rate_Const_Fission;
    CREATE_OBJECT(Set_Max_Rate_Const_Fission_Req, Action_Req_Ptr = new Set_Max_Rate_Const_Fission_Req(Action_Time, Max_Rate_Const));
    Add_Action_Req(Action_Req_Ptr); 
      

    Action_Time = 0;
    Number_To_Add = 1;
    Repeat_Int = 0;
    string R1_Spec = "0";
    string R2_Spec = "10";
    string Prod_Spec = "49";
    CREATE_OBJECT(Add_Construction_Req, Action_Request* Act_Req_Ptr = new Add_Construction_Req(Action_Time, 
                                                                                              Number_To_Add, 
                                                                                              Repeat_Int, 
                                                                                              R1_Spec, 
                                                                                              R2_Spec, 
                                                                                              Prod_Spec));

    Add_Action_Req(Act_Req_Ptr);


    Action_Time = 0;
    Number_To_Add = Num_Cat_Trans;
    Repeat_Int = 0; 
    string R_Spec = string("Random");
    string C_Spec = string("Random");
    string P_Spec = string("Random");
  
    CREATE_OBJECT(Add_Cat_Trans_Req, Action_Req_Ptr = new Add_Cat_Trans_Req(Action_Time, Number_To_Add, Repeat_Int, R_Spec, C_Spec, P_Spec)); 
    Add_Action_Req(Action_Req_Ptr);

    Action_Time = End_Time;
    CREATE_OBJECT(Terminate_Scenario_Req, Action_Req_Ptr = new Terminate_Scenario_Req(Action_Time)); 
    Add_Action_Req(Action_Req_Ptr);  */
}  
//-----------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------
